@@ -1,7 +1,7 @@
 /*
  *   cda - Command-line CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,7 @@
  */
 
 #ifndef LINT
-static char *_visual_c_ident_ = "@(#)visual.c	6.56 98/05/18";
+static char *_visual_c_ident_ = "@(#)visual.c	6.68 00/01/22";
 #endif
 
 #ifndef NOVISUAL
@@ -69,8 +69,9 @@ static char *_visual_c_ident_ = "@(#)visual.c	6.56 98/05/18";
 
 extern appdata_t	app_data;
 extern cddb_incore_t	cur_db;
-extern char		*errmsg,
-			emsg[],
+extern char		*emsgp,
+			**cda_devlist,
+			errmsg[],
 			spipe[],
 			rpipe[];
 extern int		cda_sfd[],
@@ -220,9 +221,15 @@ cda_wgetstr(WINDOW *win, char *inbuf, int max)
 			break;
 
 		case KEY_UP:
-		case KEY_DOWN:
 		case '\036':	/* ctrl-^ */
+		case KEY_DOWN:
 		case '\026':	/* ctrl-v */
+		case KEY_PPAGE:
+		case '\002':	/* ctrl-b */
+		case '\025':	/* ctrl-u */
+		case KEY_NPAGE:
+		case '\006':	/* ctrl-f */
+		case '\004':	/* ctrl-d */
 			if (n > 0) {
 				n--;
 				getyx(win, y, x);
@@ -231,19 +238,35 @@ cda_wgetstr(WINDOW *win, char *inbuf, int max)
 				wmove(win, y, x-1);
 			}
 
-			if (ch == KEY_UP || ch == '\036')
+			switch (ch) {
+			case KEY_UP:
+			case '\036':
 				scroll_line--;
-			else
+				break;
+			case KEY_DOWN:
+			case '\026':
 				scroll_line++;
+				break;
+			case KEY_PPAGE:
+			case '\002':
+				scroll_line -= (LINES - 8);
+				break;
+			case '\025':
+				scroll_line -= ((LINES - 8) / 2);
+				break;
+			case KEY_NPAGE:
+			case '\006':
+				scroll_line += (LINES - 8);
+				break;
+			case '\004':
+				scroll_line += ((LINES - 8) / 2);
+				break;
+			}
 
-			if (scroll_line < 0) {
-				beep();
+			if (scroll_line < 0)
 				scroll_line = 0;
-			}
-			if (scroll_line > (scroll_length - 1)) {
-				beep();
+			else if (scroll_line > (scroll_length - 1))
 				scroll_line = scroll_length - 1;
-			}
 
 			prefresh(info_win, scroll_line, 0, 0, 0,
 				 LINES-8, COLS-1);
@@ -351,8 +374,8 @@ cda_v_errhandler(int code)
 		track = -2;	/* Force redisplay of version info */
 		scroll_line = 0;
 	}
-	else
-		wprintw(status_win, "%s", errmsg);
+	else if (code != CDA_DAEMON_EXIT)
+		wprintw(status_win, "%s", emsgp);
 
 	wrefresh(status_win);
 }
@@ -489,7 +512,7 @@ cda_v_auth(int retrycnt)
 	if (input[0] == '\0')
 		return 0;
 
-	name = (char *) MEM_ALLOC(strlen(input) + 1);
+	name = (char *) MEM_ALLOC("v_auth_name", strlen(input) + 1);
 	if (name == NULL) {
 		CDA_FATAL(app_data.str_nomemory);
 		return 0;
@@ -506,7 +529,7 @@ cda_v_auth(int retrycnt)
 	if (input[0] == '\0')
 		pass = NULL;
 	else {
-		pass = (char *) MEM_ALLOC(strlen(input) + 1);
+		pass = (char *) MEM_ALLOC("v_auth_pass", strlen(input) + 1);
 		if (pass == NULL) {
 			CDA_FATAL(app_data.str_nomemory);
 			return 0;
@@ -542,24 +565,25 @@ cda_screen(int signo)
 	word32_t	arg[CDA_NARGS],
 			discid,
 			queryid,
-			discnum = 0,
 			astat0 = 0,
 			astat1 = 0;
 	int		x,
 			y,
 			i,
 			retcode,
-			trkno,
-			min,
-			sec,
 			vol,
 			bal,
 			rptcnt;
+	unsigned int	discnum = 0,
+			trk,
+			idx,
+			min,
+			sec;
 	char		*p;
 	curstat_t	*s = curstat_addr();
 	bool_t		playing;
 	static bool_t	cddb = FALSE;
-	static word32_t	odiscnum;
+	static unsigned int odiscnum;
 
 	/* Need to refresh function key labels? */
 	if (refresh_fkeys) {
@@ -693,12 +717,28 @@ cda_screen(int signo)
 		bal = (int) arg[4];
 		route = (int) arg[5];
 		discid = arg[6];
-		discnum = arg[7];
+		discnum = (unsigned int) arg[7];
+		s->cur_disc = arg[7];
+
+		switch (app_data.chg_method) {
+		case CHG_SCSI_LUN:
+			s->curdev = cda_devlist[s->cur_disc - 1];
+			break;
+		case CHG_SCSI_MEDCHG:
+		case CHG_OS_IOCTL:
+		case CHG_NONE:
+		default:
+			s->curdev = cda_devlist[0];
+			break;
+		}
 
 		if (RD_ARG_MODE(astat0) == MOD_NODISC || odiscnum != discnum)
 			loaddb = TRUE;
 
 		if (discid != 0 && loaddb) {
+			/* Initialize history list */
+			cda_hist_init(s);
+
 			/* Load CD database entry */
 			loaddb = FALSE;
 			cda_clrstatus();
@@ -889,14 +929,23 @@ cda_screen(int signo)
 				wprintw(status_win, "No Disc   ");
 			}
 			else {
+				trk = (unsigned int) RD_ARG_TRK(astat1);
+				idx = (unsigned int) RD_ARG_IDX(astat1);
+				min = (unsigned int) RD_ARG_MIN(astat1);
+				sec = (unsigned int) RD_ARG_SEC(astat1);
+
+				if (idx == 0 && app_data.subq_lba) {
+					/* In LBA mode the time information
+					 * isn't meaningful when in the
+					 * lead-in, so just set to zero.
+					 */
+					min = sec = 0;
+				}
+
 				wprintw(status_win,
-				"Disc %d Track %02u Index %02u %s%02u:%02u  ",
-					discnum,
-					RD_ARG_TRK(astat1),
-					RD_ARG_IDX(astat1),
-					(RD_ARG_IDX(astat1) == 0) ? "-" : "+",
-					RD_ARG_MIN(astat1),
-					RD_ARG_SEC(astat1));
+				"Disc %u Track %02u Index %02u %s%02u:%02u  ",
+					discnum, trk, idx,
+					(idx == 0) ? "-" : "+", min, sec);
 			}
 
 			wprintw(status_win, "Volume %3u%% Balance %3u%% ",
@@ -1057,8 +1106,8 @@ cda_screen(int signo)
 					       PATCHLEVEL, di_vers());
 				break;
 			}
-			wprintw(info_win, "%s\n%s\n%s\n\n%s\n",
-				COPYRIGHT, EMAIL, WWWURL, GNU_BANNER);
+			wprintw(info_win, "%s\nURL: %s\nE-mail: %s\n\n%s\n",
+				COPYRIGHT, XMCD_URL, EMAIL, GNU_BANNER);
 
 			wprintw(info_win, "\nDevice: %s\n", app_data.device);
 			if (stat_on) {
@@ -1130,7 +1179,7 @@ cda_screen(int signo)
 			cur_db.discid = arg[0];
 			ntrks = (int) (arg[0] & 0xff);
 
-			wprintw(info_win, "Disc %d  ID: %s %08x%s",
+			wprintw(info_win, "Disc %u  ID: %s %08x%s",
 				discnum,
 				(cur_db.category[0] == '\0') ?
 					"(no category)" : cur_db.category,
@@ -1149,10 +1198,10 @@ cda_screen(int signo)
 				wprintw(info_win, "(unknown disc title)\n\n");
 
 			for (i = 0; i < (int) ntrks; i++) {
-				RD_ARG_TOC(arg[i+1], trkno, playing, min, sec);
+				RD_ARG_TOC(arg[i+1], trk, playing, min, sec);
 				wprintw(info_win, "%s%02u %02u:%02u  ",
 					playing ? ">" : " ",
-					trkno, min, sec);
+					trk, min, sec);
 				if (cddb && cur_db.trklist[i] != NULL) {
 					wprintw(info_win, "%s%s\n",
 						cur_db.trklist[i],
@@ -1166,7 +1215,7 @@ cda_screen(int signo)
 				}
 			}
 
-			RD_ARG_TOC(arg[i+1], trkno, playing, min, sec);
+			RD_ARG_TOC(arg[i+1], trk, playing, min, sec);
 			wprintw(info_win, "\nTotal Time: %02u:%02u\n",
 				min, sec);
 		}
@@ -1534,7 +1583,7 @@ cda_v_shufprog(int inp, word32_t arg[])
 
 			arg[0] = (word32_t) -j;
 			if (!cda_sendcmd(CDA_PROGRAM, arg, &retcode)) {
-				errmsg = NULL;
+				emsgp = NULL;
 				beep();
 			}
 		}
@@ -1862,19 +1911,40 @@ cda_v_repaint(int inp, word32_t arg[])
 STATIC void
 cda_v_scroll(int inp, word32_t arg[])
 {
-	if (inp == KEY_UP || inp == '^' || inp == '\036')
+	switch (inp) {
+	case KEY_UP:
+	case '^':
+	case '\036':	/* ctrl-^ */
 		scroll_line--;
-	else
+		break;
+	case KEY_DOWN:
+	case 'v':
+	case 'V':
+	case '\026':	/* ctrl-v */
 		scroll_line++;
+		break;
+	case KEY_PPAGE:
+	case '\002':	/* ctrl-b */
+		scroll_line -= (LINES - 8);
+		break;
+	case '\025':	/* ctrl-u */
+		scroll_line -= ((LINES - 8) / 2);
+		break;
+	case KEY_NPAGE:
+	case '\006':	/* ctrl-f */
+		scroll_line += (LINES - 8);
+		break;
+	case '\004':	/* ctrl-d */
+		scroll_line += ((LINES - 8) / 2);
+		break;
+	default:
+		return;
+	}
 
-	if (scroll_line < 0) {
-		beep();
+	if (scroll_line < 0)
 		scroll_line = 0;
-	}
-	if (scroll_line > (scroll_length - 1)) {
-		beep();
+	else if (scroll_line > (scroll_length - 1))
 		scroll_line = scroll_length - 1;
-	}
 
 	prefresh(info_win, scroll_line, 0, 0, 0, LINES-8, COLS-1);
 }
@@ -2097,7 +2167,7 @@ cda_vtidy(void)
 		endwin();
 	}
 
-	(void) printf("%s\n", errmsg != NULL ? errmsg : "Goodbye!");
+	(void) printf("%s\n", emsgp != NULL ? emsgp : "Goodbye!");
 }
 
 
@@ -2150,6 +2220,12 @@ STATIC keytbl_t	ktbl[] = {
 	{ '\036',	cda_v_scroll		},
 	{ '\026',	cda_v_scroll		},
 	{ 'V',		cda_v_scroll		},
+	{ KEY_PPAGE,	cda_v_scroll		},
+	{ KEY_NPAGE,	cda_v_scroll		},
+	{ '\002',	cda_v_scroll		},
+	{ '\025',	cda_v_scroll		},
+	{ '\006',	cda_v_scroll		},
+	{ '\004',	cda_v_scroll		},
 	{ 'd',		cda_v_debug		},
 	{ '\0',		cda_v_dirtrk		},
 	{ '\0',		NULL			}
@@ -2181,6 +2257,9 @@ cda_visual(void)
 	(void) signal(SIGCONT, oncont);
 #endif
 
+	/* Set up device list */
+	cda_parse_devlist(s);
+
 	/* Initialize curses and paint initial screen */
 	initscr();
 	isvisual = TRUE;
@@ -2219,6 +2298,7 @@ cda_visual(void)
 	/* Main processing loop */
 	for (;;) {
 		inp = cda_wgetch(status_win);
+
 		if (inp == KEY_F(8) || inp == 'q' || inp == 'Q')
 			break;
 
@@ -2232,7 +2312,7 @@ cda_visual(void)
 		/* Execute command function */
 		for (i = 0; ktbl[i].keyfunc != NULL; i++) {
 			if (ktbl[i].key == inp || ktbl[i].key == '\0') {
-				ktbl[i].keyfunc(inp, arg);
+				(*ktbl[i].keyfunc)(inp, arg);
 				break;
 			}
 		}
@@ -2243,12 +2323,12 @@ cda_visual(void)
 
 	/* Tidy up and go home */
 	(void) alarm(0);
-	errmsg = NULL;
+	emsgp = NULL;
 
 	if (cda_daemon_alive()) {
-		(void) sprintf(emsg, "CD audio daemon pid=%d still running.",
+		(void) sprintf(errmsg, "CD audio daemon pid=%d still running.",
 			       (int) daemon_pid);
-		errmsg = emsg;
+		emsgp = errmsg;
 	}
 
 	cda_quit(s);
