@@ -1,7 +1,7 @@
 /*
  *   cddb - CD Database Management Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __CDDB_H__
 
 #ifndef LINT
-static char *_cddb_h_ident_ = "@(#)cddb.h	6.53 98/01/02";
+static char *_cddb_h_ident_ = "@(#)cddb.h	6.67 00/01/09";
 #endif
 
 #ifndef NOREMOTE
@@ -33,6 +33,7 @@ static char *_cddb_h_ident_ = "@(#)cddb.h	6.53 98/01/02";
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
 #if defined(_AIX) || defined(__QNX__)
 #include <sys/select.h>
@@ -99,7 +100,7 @@ typedef struct fd_set {
 
 
 /* CDDB protocol level used */
-#define CDDBP_PROTO		1
+#define CDDBP_PROTO_LVL		4
 
 /* HTTP mode version string */
 #define CDDB_HTTP_VER		"HTTP/1.0"
@@ -132,6 +133,12 @@ typedef struct fd_set {
 #define CDDB_GET_STAT(code)		((code) & 0xffff)
 #define CDDB_GET_ARG(code)		((code) >> 16)
 
+#ifdef __VMS
+#define CH_RET(code)		return CDDB_SET_CODE((code), (errno))
+#else
+#define CH_RET(code)		exit((code))
+#endif
+
 /* Error status codes */
 #define OPEN_ERR		50
 #define CLOSE_ERR		51
@@ -219,6 +226,7 @@ typedef struct lnopts {
 #define CDDB_AUTHFAIL	0x0002			/* Proxy auth failed */
 #define CDDB_CHANGED	0x0003			/* In-core CDDB was edited */
 #define CDDB_AUTOSAVE	0x0004			/* An autosave was done */
+#define CDDB_REQUERY	0x0010			/* 2nd query after dbmatch */
 
 
 /* In-core CD database info structure */
@@ -255,6 +263,7 @@ typedef struct dbpath {
 	char		*host;			/* Host name or IP number */
 	int		port;			/* Remote CDDB port */
 	char		*path;			/* Directory path */
+	char		*category;		/* Category name */
 	struct dbpath	*next;			/* Link to next path */
 } cddb_path_t;
 
@@ -279,6 +288,51 @@ typedef struct {
 } cddb_proxy_t;
 
 
+#define DLIST_BUF_SZ	128
+
+/* Disc list information structure */
+typedef struct cddb_dlist {
+	char		*device;		/* Device */
+	int		discno;			/* Disc number */
+	int		type;			/* Type flags */
+	word32_t	discid;			/* Disc ID */
+	char		*category;		/* CD category */
+	char		*dtitle;		/* CD artist / disc title */
+	time_t		time;			/* Time */
+	void		*aux;			/* Auxiliary pointer */
+	struct cddb_dlist *prev;		/* Prev entry on list */
+	struct cddb_dlist *next;		/* Next entry on list */
+} cddb_dlist_t;
+
+
+/* Private functions: for libcddb internal use only */
+#ifdef _CDDB_INTERN
+extern void		onterm(int);
+extern void		cddb_http_xlat(char *, char *);
+extern int		cddb_sum(int);
+extern bool_t		cddb_add_linkent(char *, char *, word32_t);
+extern bool_t		cddb_add_pathent(char *);
+extern void		cddb_add_ident(cddb_incore_t *, char *);
+extern bool_t		cddb_add_idlist(cddb_incore_t *, char *);
+extern void		cddb_free_idlist(cddb_incore_t *);
+extern cddb_file_t	*cddb_open(cddb_incore_t *, curstat_t *, int);
+extern bool_t		cddb_close(cddb_file_t *);
+#ifdef __VMS
+extern bool_t		cddb_read0(cddb_file_t *, cddb_incore_t *,
+				   curstat_t *);
+#else
+extern bool_t		cddb_read1(cddb_file_t *, cddb_file_t *,
+				   cddb_incore_t *);
+extern bool_t		cddb_read2(cddb_file_t *, cddb_incore_t *,
+				   curstat_t *);
+#endif
+extern bool_t		cddb_write(cddb_file_t *, cddb_incore_t *,
+				   curstat_t *);
+extern void		cddb_autosave(cddb_incore_t *, curstat_t *);
+extern bool_t		cddb_check_entry(cddb_incore_t *);
+#endif /* _CDDB_INTERN */
+
+
 /* Public functions */
 extern void		cddb_init(cddb_client_t *);
 extern void		cddb_halt(curstat_t *);
@@ -287,6 +341,14 @@ extern cddb_ret_t	cddb_save(cddb_incore_t *, curstat_t *);
 extern cddb_ret_t	cddb_link(cddb_incore_t *, char *, char *);
 extern cddb_ret_t	cddb_send(cddb_incore_t *, curstat_t *);
 extern void		cddb_clear(cddb_incore_t *, curstat_t *s, bool_t);
+extern void		cddb_hist_init(void);
+extern cddb_ret_t	cddb_hist_addent(cddb_dlist_t *, bool_t);
+extern cddb_ret_t	cddb_hist_delent(cddb_dlist_t *, bool_t);
+extern cddb_ret_t	cddb_hist_delall(bool_t);
+extern cddb_dlist_t	*cddb_hist_list(void);
+extern void		cddb_chgr_init(void);
+extern cddb_ret_t	cddb_chgr_addent(cddb_dlist_t *);
+extern cddb_dlist_t	*cddb_chgr_list(void);
 extern void		cddb_load_cancel(void);
 extern word32_t		cddb_discid(curstat_t *);
 extern char		*cddb_category(char *);
