@@ -1,7 +1,7 @@
 /*
  *   cddbp - CD Database Protocol
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,13 +23,14 @@
 #define __CDDBP_H__
 
 #ifndef LINT
-static char *_cddbp_h_ident_ = "@(#)cddbp.h	6.7 98/01/02";
+static char *_cddbp_h_ident_ = "@(#)cddbp.h	6.10 99/12/27";
 #endif
 
 
 /*
  * CDDBP commands
  */
+#define CDDB_PROTO		"proto"
 #define CDDB_HELLO		"cddb hello"
 #define CDDB_QUERY		"cddb query"
 #define CDDB_READ		"cddb read"
@@ -97,6 +98,12 @@ static char *_cddbp_h_ident_ = "@(#)cddbp.h	6.7 98/01/02";
 #define STAT_BANR_USERS		'3'	/* No connection: too many users */
 #define STAT_BANR_LOAD		'4'	/* No connection: load too high */
 
+/* Set proto level */
+#define STAT_PROTO_OK		'0'	/* OK, display curr and max levels */
+#define STAT_PROTO_SET		'1'	/* OK, set to specified level */
+#define STAT_PROTO_ERR		'1'	/* Illegal level specified */
+#define STAT_PROTO_NOCHG	'2'	/* Already at specified level */
+
 /* Hello handshake */
 #define STAT_HELO_OK		'0'	/* Handshake successful */
 #define STAT_HELO_FAIL		'1'	/* Handshake failed */
