@@ -6,7 +6,7 @@
  *	libdi - CD Audio Player Device Interface Library
  *
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,7 @@
 #define __APPENV_H__
 
 #ifndef LINT
-static char *_appenv_h_ident_ = "@(#)appenv.h	6.99 98/05/14";
+static char *_appenv_h_ident_ = "@(#)appenv.h	6.139 00/01/19";
 #endif
 
 
@@ -51,6 +51,7 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.99 98/05/14";
 #define FILE_BASE_SZ	64			/* Max base name length */
 #define MAXTRACK	100			/* Max number of tracks */
 #define LEAD_OUT_TRACK	0xaa			/* Lead-out track number */
+#define STD_CDROM_BLKSZ	2048			/* Standard CD-ROM blocksize */
 #define FRAME_PER_SEC	75			/* Frames per second */
 #define MSF_OFFSET	150			/* Starting MSF offset */
 
@@ -70,15 +71,20 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.99 98/05/14";
 #ifndef NO_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifndef NO_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include <stdio.h>
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <time.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
+#include <sys/utsname.h>
 #include <memory.h>
 #include <dirent.h>
 #include <pwd.h>
@@ -120,6 +126,7 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.99 98/05/14";
 #include <fcntl.h>
 #include <signal.h>
 #include <string.h>
+#include <utsname.h>
 #include <descrip.h>
 #include <dvidef.h>
 #include <lnmdef.h>
@@ -149,16 +156,20 @@ typedef uint_t		mode_t;
 /* Prototype some external library functions */
 extern uid_t		getuid(void);
 extern gid_t		getgid(void);
-extern int		setuid(uid_t);
-extern int		setgid(gid_t);
-extern void		*malloc(int);
-extern void		free(void *);
-extern void		exit(int);
-extern char		*getenv(char *);
+extern int		setuid(uid_t),
+			setgid(gid_t);
+extern void		*malloc(int),
+			free(void *),
+			exit(int);
+extern char		*getenv(char *),
+			*getlogin(void);
 
 #define PIPE(fds)	pipe(fds, O_RDWR, 1024)
 #define LINK		util_link
 #define UNLINK		util_unlink
+
+#ifdef VMS_USE_OWN_DIRENT
+
 #define OPENDIR		util_opendir
 #define READDIR		util_readdir
 #define CLOSEDIR	util_closedir
@@ -171,6 +182,16 @@ typedef struct {
 	struct dirent	*dd_buf;
 } DIR;
 
+#else
+
+#include <dirent.h>
+
+#define OPENDIR		opendir
+#define READDIR		readdir
+#define CLOSEDIR	closedir
+
+#endif	/* VMS_USE_OWN_DIRENT */
+
 #endif	/* __VMS */
 
 
@@ -189,6 +210,9 @@ typedef struct {
 #ifndef SIGCHLD
 #define SIGCHLD		SIGCLD
 #endif
+#ifndef ENAMETOOLONG
+#define ENAMETOOLONG	78
+#endif
 
 
 /*
@@ -212,9 +236,9 @@ typedef char		bool_t;			/* Boolean */
 #if defined(i286) || defined(__i286__) || \
     defined(i386) || defined(__i386__) || \
     defined(i486) || defined(__i486__) || \
-    defined(__alpha) || \
-    defined(vax) || \
-    (defined(mips) && defined(MIPSEL))
+    defined(__alpha) || defined(vax) || \
+    (defined(mips) && defined(MIPSEL) || \
+    defined(__QNX__))
 #define _BYTE_ORDER_	_L_ENDIAN_
 #else
 #define _BYTE_ORDER_	_B_ENDIAN_
@@ -306,10 +330,22 @@ typedef int			mode_t;
 
 
 /* Memory allocators */
-#define MEM_ALLOC	malloc
-#define MEM_REALLOC	realloc
-#define MEM_CALLOC	calloc
-#define MEM_FREE	free
+#define _MEM_ALLOC		malloc
+#define _MEM_REALLOC		realloc
+#define _MEM_CALLOC		calloc
+#define _MEM_FREE		free
+
+#ifdef MEM_DEBUG
+#define MEM_ALLOC(s,l)		util_dbg_malloc((s),(l))
+#define MEM_REALLOC(s,p,l)	util_dbg_realloc((s),(p),(l))
+#define MEM_CALLOC(s,n,l)	util_dbg_calloc((s),(n),(l))
+#define MEM_FREE(p)		util_dbg_free((p))
+#else
+#define MEM_ALLOC(s,l)		_MEM_ALLOC((l))
+#define MEM_REALLOC(s,p,l)	_MEM_REALLOC((p),(l))
+#define MEM_CALLOC(s,n,l)	_MEM_CALLOC((n),(l))
+#define MEM_FREE(p)		_MEM_FREE((p))
+#endif
 
 
 /*
@@ -322,6 +358,8 @@ typedef int			mode_t;
 #define SYS_DSCFG_PATH	"%s/config/%s"		/* System devspec cfg file */
 #define USR_DSCFG_PATH	"%s/.xmcdcfg/%s"	/* User devspec cfg file */
 #define USR_VINIT_PATH	"%s/.xmcdcfg/%s"	/* User version init file */
+#define USR_HIST_PATH	"%s/.xmcdcfg/history"	/* User xmcd history file */
+#define USR_RMTLOG_PATH	"%s/.xmcdcfg/remote.log"/* User xmcd rmtlog file */
 #define REL_DBDIR_PATH	"%s/cddb/%s"		/* CDDB rel dir path */
 #define CDDBFILE_PATH	"%s/%08x"		/* CDDB file path */
 #define HELPFILE_PATH	"%s/help/%s"		/* Help file path */
@@ -334,6 +372,8 @@ typedef int			mode_t;
 #define SYS_DSCFG_PATH	"%s]device.cfg"		/* System devspec cfg file */
 #define USR_DSCFG_PATH	"%s.xmcdcfg]device.cfg"	/* User devspec cfg file */
 #define USR_VINIT_PATH	"%s.xmcdcfg]%s"		/* User version init file */
+#define USR_HIST_PATH	"%s.xmcdcfg]history"	/* User xmcd history file */
+#define USR_RMTLOG_PATH	"%s.xmcdcfg]remote.log"	/* User xmcd rmtlog file */
 #define REL_DBDIR_PATH	"%s.cddb.%s]"		/* CDDB rel dir path */
 #define CDDBFILE_PATH	"%s%08x."		/* CDDB file path */
 #define HELPFILE_PATH	"%s.help]%s"		/* Help file path */
@@ -342,18 +382,35 @@ typedef int			mode_t;
 #endif
 
 
+/* 
+ * URL template attribute information 
+ */
+typedef struct url_attrib {
+	int		xcnt;			/* Count of %X */
+	int		vcnt;			/* Count of %V */
+	int		ncnt;			/* Count of %N */
+	int		hcnt;			/* Count of %H */
+	int		lcnt;			/* Count of %L */
+	int		ccnt;			/* Count of %C */
+	int		icnt;			/* Count of %I */
+	int		acnt;			/* Count of %A and %a */
+	int		dcnt;			/* Count of %D and %d */
+	int		tcnt;			/* Count of %T and %t */
+} url_attrib_t;
+
 /* 
  * Web search engine information 
  */
-typedef struct srcheng_ent {
+typedef struct urltmpl_ent {
+	int		idx;			/* Index number */
 	char		*name;			/* Search engine name */
 	char		*action;		/* Action URL+args */
+	char		*fallback;		/* Fallback URL+args */
+	url_attrib_t	a_attrib;		/* Action URL attributes */
+	url_attrib_t	f_attrib;		/* Fallback URL attributes */
 	void		*aux;			/* Auxiliary pointer */
-	int		acnt;			/* Number of %A s */
-	int		dcnt;			/* Number of %D s */
-	int		tcnt;			/* Number of %T s */
-	struct srcheng_ent *next;		/* Link to next entry */
-} srcheng_ent_t;
+	struct urltmpl_ent *next;		/* Link to next entry */
+} urltmpl_ent_t;
 
 
 /*
@@ -449,6 +506,7 @@ typedef struct {
 	bool_t		program;		/* Program mode */
 	bool_t		onetrk_prog;		/* Trk list 1-track program */
 	bool_t		caddy_lock;		/* Caddy lock */
+	bool_t		chgrscan;		/* Changer scan in process */
 	byte_t		cddb;			/* CD Database entry type */
 	byte_t		prog_tot;		/* Prog/Shuf total tracks */
 	byte_t		prog_cnt;		/* Prog/Shuf track counter */
@@ -495,16 +553,17 @@ typedef struct {
 #define STR_READY		"ready"
 #define STR_SAMPLE		"samp"
 #define STR_USAGE		"Usage:"
-#define STR_BADOPTS		"The following options are unrecognized:"
+#define STR_BADOPTS		"The following option is unrecognized"
 #define STR_NODISC		"no disc"
 #define STR_BUSY		"cd busy"
 #define STR_UNKNDISC		"unknown disc title"
 #define STR_UNKNTRK		"unknown track title"
 #define STR_DATA		"data"
+#define STR_INFO		"Information"
 #define STR_WARNING		"Warning"
 #define STR_FATAL		"Fatal Error"
 #define STR_CONFIRM		"Confirm"
-#define STR_INFO		"Information"
+#define STR_WORKING		"Working"
 #define STR_ABOUT		"About"
 #define STR_QUIT		"Really Quit?"
 #define STR_NOMEMORY		"Out of memory!"
@@ -518,6 +577,7 @@ typedef struct {
 #define STR_NOLINK		"There is no likely CDDB entry to link to."
 #define STR_NODB		"No CD database directory."
 #define STR_NOCFG		"Cannot open configuration file \"%s\"."
+#define STR_NOINFO		"No information available."
 #define STR_NOTROM		"Device is not a CD-ROM!"
 #define STR_NOTSCSI2		"Device is not SCSI-II compliant."
 #define STR_SEND		"Mail current CDDB entry to \"%s\"?"
@@ -565,6 +625,13 @@ Running as a single-disc player"
 #define STR_AUTHFAIL		\
 "Proxy authorization failed.\n\
 Do you want to re-enter your user name and password?"
+#define STR_NOCLIENT		"Not running on display"
+#define STR_UNSUPPCMD		"Unsupported command"
+#define STR_BADARG		"The specified argument is invalid"
+#define STR_INVCMD		"Invalid command for the current state"
+#define STR_CMDFAIL		"Command failed"
+#define STR_RMT_NOTENB		"Remote control is not enabled"
+#define STR_RMT_NOCMD		"Remote control: No command specified"
 #define STR_BROWSER_START	"Starting web browser..."
 #define STR_BROWSER_FAIL	\
 "Cannot access web browser.\n\
@@ -582,6 +649,18 @@ version.  A correct version of this file must be present in the\n\
 appropriate directory in order for xmcd to run.  Please check\n\
 your xmcd installation."
 #endif
+#define STR_KPMODEDSBL		\
+"The %s mode is enabled.\n\
+Using the keypad will disable it.\n\
+Do you want to proceed?"
+#define STR_DLIST_DELALL	\
+"This will delete all entries of your disc history.\n\
+Do you want to proceed?"
+#define STR_CHGRSCAN		"Scanning CD changer slots..."
+#define STR_LOCALDISCOG		"Local Discography"
+#define STR_FILE		"File"
+#define STR_BROWSE_CATEG	"Browse category"
+#define STR_HOWTO_SETUP		"How to set up"
 
 /*
  * Application resource/option data
@@ -592,6 +671,7 @@ typedef struct {
 	/* X resources */
 	char		*version;		/* app-defaults file version */
 	int		main_mode;		/* Default main window mode */
+	int		modechg_grav;		/* Which corner is fixed */
 	int		normal_width;		/* Normal mode width */
 	int		normal_height;		/* Normal mode height */
 	int		basic_width;		/* Basic mode width */
@@ -599,6 +679,9 @@ typedef struct {
 	int		blinkon_interval;	/* Display blink on (ms) */
 	int		blinkoff_interval;	/* Display blink off (ms) */
 	bool_t		main_showfocus;		/* Highlight kbd focus? */
+	bool_t		instcmap;		/* Install colormap? */
+	bool_t		*remotemode;		/* Remote ctrl mode */
+	char		*remotehost;		/* Remote ctrl client host */
 
 	/*
 	 * Common config parameters
@@ -614,9 +697,13 @@ typedef struct {
 	char		*site1_url;		/* Web site 1 URL */
 	char		*site2_url;		/* Web site 2 URL */
 	char		*site3_url;		/* Web site 3 URL */
+	char		*discog_url;		/* Local Discographies URL */
 	char		*exclude_words;		/* Keywords to exclude */
+	char		*hist_filemode;		/* History file permissions */
 	int		cddb_port;		/* Remote CDDB default port */
 	int		cddb_timeout;		/* Remote CDDB timeout (secs) */
+	int		cddb_rmtautosave;	/* Remote CDDB auto-save */
+	int		cddb_maxhist;		/* Max history count */
 	int		stat_interval;		/* Status poll interval (ms) */
 	int		ins_interval;		/* Insert poll interval (ms) */
 	int		prev_threshold;		/* Previous track/index
@@ -627,12 +714,15 @@ typedef struct {
 	int		tooltip_delay;		/* Tool-tip delay interval */
 	int		tooltip_time;		/* Tool-tip active interval */
 	bool_t		ins_disable;		/* Insert poll disable */
-	bool_t		cddb_rmtautosave;	/* CDDB remote auto-save */
+	bool_t		cddb_rmtdsbl;		/* Remote query disable */
 	bool_t		http_proxy;		/* Using http proxy server */
 	bool_t		scsierr_msg;		/* Print SCSI error msg? */
 	bool_t		sol2_volmgt;		/* Solaris 2.x Vol Mgr */
 	bool_t		write_curfile;		/* Enable curr.XXX output */
 	bool_t		tooltip_enable;		/* Enable tool-tips */
+	bool_t		histfile_dsbl;		/* Disable history file */
+	bool_t		remote_enb;		/* Enable remote control */
+	bool_t		remote_log;		/* Enable rmtctl logfile */
 	bool_t		debug;			/* Verbose debug output */
 
 	/*
@@ -659,9 +749,15 @@ typedef struct {
 	bool_t		balance_supp;		/* Indep vol chg supported */
 	bool_t		chroute_supp;		/* Channel routing support */
 	bool_t		pause_supp;		/* Pause/Resume supported */
+	bool_t		strict_pause_resume;	/* Must resume after pause */
+	bool_t		play_pause_play;	/* Must pause before play */
 	bool_t		caddylock_supp;		/* Caddy lock supported */
 	bool_t		curpos_fmt;		/* Fmt 1 of RdSubch command */
 	bool_t		play_notur;		/* No Tst U Rdy when playing */
+	bool_t		toc_lba;		/* Rd TOC in LBA mode */
+	bool_t		subq_lba;		/* Rd Subchan in LBA mode */
+	int		drv_blksz;		/* Drive native block size */
+	int		spinup_interval;	/* spin up delay (sec) */
 
 	/* User-modifiable */
 	int		vol_taper;		/* Volume control taper */
@@ -675,6 +771,7 @@ typedef struct {
 	bool_t		load_spindown;		/* Spin down disc on CD load */
 	bool_t		load_play;		/* Auto play on CD load */
 	bool_t		done_eject;		/* Auto eject on done */
+	bool_t		done_exit;		/* Auto exit on done */
 	bool_t		exit_eject;		/* Eject disc on exit? */
 	bool_t		exit_stop;		/* Stop disc on exit? */
 	bool_t		eject_exit;		/* Exit upon disc eject? */
@@ -709,10 +806,11 @@ typedef struct {
 	char		*str_unkndisc;		/* unknown disc title */
 	char		*str_unkntrk;		/* unknown track title */
 	char		*str_data;		/* Data */
+	char		*str_info;		/* Information */
 	char		*str_warning;		/* Warning */
 	char		*str_fatal;		/* Fatal error */
 	char		*str_confirm;		/* Confirm */
-	char		*str_info;		/* Information */
+	char		*str_working;		/* Working */
 	char		*str_about;		/* About */
 	char		*str_quit;		/* Really Quit? */
 	char		*str_nomemory;		/* Out of memory */
@@ -725,6 +823,7 @@ typedef struct {
 	char		*str_nolink;		/* No likely CDDB link */
 	char		*str_nodb;		/* No CDDB directory */
 	char		*str_nocfg;		/* Can't open config file */
+	char		*str_noinfo;		/* No information avail */
 	char		*str_notrom;		/* Not a CD-ROM device */
 	char		*str_notscsi2;		/* Not SCSI-II compliant */
 	char		*str_send;		/* Send CDDB confirm msg */
@@ -755,9 +854,23 @@ typedef struct {
 	char		*str_medchg_noinit;	/* Cannot init medium changer */
 	char		*str_proxyerr;		/* Proxy server config error */
 	char		*str_authfail;		/* Proxy auth failure */
+	char		*str_noclient;		/* Can't find client */
+	char		*str_unsuppcmd;		/* Unsupp remote command */
+	char		*str_badarg;		/* Specified arg is bad */
+	char		*str_invcmd;		/* Invalid cmd for current */
+	char		*str_cmdfail;		/* Command failed */
+	char		*str_rmt_notenb;	/* Remote ctl not enabled */
+	char		*str_rmt_nocmd;		/* Remote ctl no command */
 	char		*str_browser_start;	/* Starting web browser */
 	char		*str_browser_fail;	/* Cannot start browser */
 	char		*str_appdef;		/* app-defaults file error */
+	char		*str_kpmodedsbl;	/* Keypad mode dsbl prompt */
+	char		*str_dlist_delall;	/* Delete all dlist entries */
+	char		*str_chgrscan;		/* CD Changer scanning */
+	char		*str_localdiscog;	/* Local Discography */
+	char		*str_file;		/* File */
+	char		*str_browse_categ;	/* Browse category */
+	char		*str_howto_setup;	/* How to set up */
 
 	/*
 	 * Short-cut key definitions
@@ -769,7 +882,7 @@ typedef struct {
 	char		*eject_key;		/* Eject */
 	char		*quit_key;		/* Quit */
 	char		*dbprog_key;		/* Database/Program popup */
-	char		*help_key;		/* Help popup */
+	char		*wwwwarp_key;		/* wwwWarp/Help popup */
 	char		*options_key;		/* Options */
 	char		*time_key;		/* Time */
 	char		*ab_key;		/* A->B */
