@@ -6,7 +6,7 @@
  *	libdi - CD Audio Player Device Interface Library
  *
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,15 +28,15 @@
 #define __PATCHLEVEL_H__
 
 #ifndef LINT
-static char *_patchlevel_h_ident_ = "@(#)patchlevel.h	6.26 98/05/22";
+static char *_patchlevel_h_ident_ = "@(#)patchlevel.h	6.35 00/02/04";
 #endif
 
-#define VERSION		"2.3"		/* The version number */
+#define VERSION		"2.6"		/* The version number */
 #define VERSION_EXT	""		/* Version number extension */
 #define PATCHLEVEL	0		/* The patch level */
-#define COPYRIGHT	"Copyright (C) 1993-1998  Ti Kan"
-#define EMAIL		"E-mail: xmcd@amb.org"
-#define WWWURL		"URL: http://sunsite.unc.edu/~cddb/xmcd/"
+#define COPYRIGHT	"Copyright (C) 1993-2000  Ti Kan"
+#define EMAIL		"xmcd@amb.org"
+#define XMCD_URL	"http://metalab.unc.edu/tkan/xmcd/"
 #define GNU_BANNER	\
 "This is free software and comes with no warranty.\n\
 See the GNU General Public License for details."
