@@ -6,7 +6,7 @@
  *   libdi - CD Audio Player Device Interface Library
  *
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
  *
  */
 #ifndef LINT
-static char *_util_c_ident_ = "@(#)util.c	6.45 98/05/18";
+static char *_util_c_ident_ = "@(#)util.c	6.63 00/01/09";
 #endif
 
 #include "common_d/appenv.h"
@@ -55,6 +55,7 @@ extern FILE		*errfp;
 
 STATIC uid_t		ouid = 30001;	/* Default to something safe */
 STATIC gid_t		ogid = 30001;	/* Default to something safe */
+STATIC struct utsname	un;		/* utsname */
 
 /*
  * Data used by util_text_reduce()
@@ -68,6 +69,15 @@ STATIC char		*delete_words[] = {
 	"\\n", "\\r", "\\t"
 };
 
+/*
+ * For util_monname()
+ */
+STATIC char		*mon_name[] = {
+	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+};
+
+
 
 /***********************
  *   public routines   *
@@ -80,7 +90,7 @@ STATIC char		*delete_words[] = {
  *	the calling program does a setuid.
  *
  * Args:
- *	Nothing
+ *	None
  *
  * Return:
  *	Nothing
@@ -91,6 +101,11 @@ util_init(void)
 	/* Save original uid and gid */
 	ouid = getuid();
 	ogid = getgid();
+
+	/* Set uname */
+	if (uname(&un) < 0) {
+		DBGPRN(errfp, "uname(2) failed (errno=%d)\n", errno);
+	}
 }
 
 
@@ -99,7 +114,7 @@ util_init(void)
  *	Start up the libutil module.
  *
  * Args:
- *	Nothing
+ *	None
  *
  * Return:
  *	Nothing
@@ -127,7 +142,10 @@ util_start(void)
 	}
 	excnt = n;
 
-	exclude_words = (char **) MEM_ALLOC(excnt * sizeof(char *));
+	exclude_words = (char **) MEM_ALLOC(
+		"exclude_words",
+		excnt * sizeof(char *)
+	);
 	if (exclude_words == NULL) {
 		fprintf(errfp, "Out of virtual memory.\n");
 		exit(1);
@@ -143,7 +161,10 @@ util_start(void)
 				continue;
 			c = *q;
 			*q = '\0';
-			exclude_words[n] = (char *) MEM_ALLOC(strlen(p) + 1);
+			exclude_words[n] = (char *) MEM_ALLOC(
+				"exclude_words[n]",
+				strlen(p) + 1
+			);
 			if (exclude_words[n] == NULL) {
 				fprintf(errfp, "Out of virtual memory.\n");
 				exit(1);
@@ -156,7 +177,10 @@ util_start(void)
 				p++;
 			q = p;
 		}
-		exclude_words[n] = (char *) MEM_ALLOC(strlen(p) + 1);
+		exclude_words[n] = (char *) MEM_ALLOC(
+			"exclude_words[n]",
+			strlen(p) + 1
+		);
 		if (exclude_words[n] == NULL) {
 			fprintf(errfp, "Out of virtual memory.\n");
 			exit(1);
@@ -164,7 +188,8 @@ util_start(void)
 		(void) strcpy(exclude_words[n], p);
 	}
 
-	if ((exlen = (int *) MEM_ALLOC(sizeof(int) * excnt)) == NULL) {
+	exlen = (int *) MEM_ALLOC("exlen", sizeof(int) * excnt);
+	if (exlen == NULL) {
 		fprintf(errfp, "Out of virtual memory.\n");
 		exit(1);
 	}
@@ -174,7 +199,8 @@ util_start(void)
 
 	delcnt = sizeof(delete_words) / sizeof(char *);
 
-	if ((dellen = (int *) MEM_ALLOC(sizeof(int) * delcnt)) == NULL) {
+	dellen = (int *) MEM_ALLOC("dellen", sizeof(int) * delcnt);
+	if (dellen == NULL) {
 		fprintf(errfp, "Out of virtual memory.\n");
 		exit(1);
 	}
@@ -189,7 +215,7 @@ util_start(void)
  *	Get original user ID
  *
  * Args:
- *	Nothing
+ *	None
  *
  * Return:
  *	Original uid value.
@@ -206,7 +232,7 @@ util_get_ouid(void)
  *	Get original group ID
  *
  * Args:
- *	Nothing
+ *	None
  *
  * Return:
  *	Original gid value.
@@ -218,6 +244,52 @@ util_get_ogid(void)
 }
 
 
+/*
+ * util_set_ougid
+ *	Change user ID and group ID to original setting.
+ *
+ * Args:
+ *	None.
+ *
+ * Return:
+ *	TRUE on success, FALSE on failure.
+ */
+bool_t
+util_set_ougid(void)
+{
+	DBGPRN(errfp, "\nSetting uid/gid: %d/%d\n", (int) ouid, (int) ogid);
+
+	/* Force uid and gid to original setting */
+	if (setuid(ouid) < 0) {
+		DBGPRN(errfp, "Failed to set uid.\n");
+		return FALSE;
+	}
+	if (setgid(ogid) < 0) {
+		DBGPRN(errfp, "Failed to set gid.\n");
+		return FALSE;
+	}
+	return (TRUE);
+}
+
+
+/*
+ * util_get_uname
+ *	Get the utsname structure for the running system
+ *	(See uname(2)).
+ *
+ * Args:
+ *	None
+ *
+ * Return:
+ *	Pointer to the utsname structure.
+ */
+struct utsname *
+util_get_uname(void)
+{
+	return (&un);
+}
+
+
 /*
  * util_ltobcd
  *	32-bit integer to BCD conversion routine
@@ -285,37 +357,11 @@ util_stob(char *s)
  */
 char *
 util_basename(char *path)
-{
-	char	*p;
-
-	if (path == NULL)
-		return NULL;
-
-#ifndef __VMS
-	if ((p = strrchr(path, '/')) == NULL)
-#else
-	if ((p = strrchr(path, ']')) == NULL)
-#endif
-		return (path);
-	
-	return (p + 1);
-}
-
-
-/*
- * util_dirname
- *	Return the dirname of a file path
- *
- * Args:
- *	path - The file path string
- *
- * Return:
- *	The dirname string
- */
-char *
-util_dirname(char *path)
 {
 	char		*p;
+#ifdef __VMS
+	char		*q;
+#endif
 	static char	buf[FILE_PATH_SZ];
 
 	if (path == NULL)
@@ -334,11 +380,83 @@ util_dirname(char *path)
 #endif
 		return (buf);
 
-#ifdef __VMS
 	p++;
+
+#ifdef __VMS
+	if (*p == '\0') {
+		/* The supplied path is a directory - special handling */
+		*--p = '\0';
+		if ((q = strrchr(buf, '.')) != NULL)
+			p = q + 1;
+		else if ((q = strrchr(buf, '[')) != NULL)
+			p = q + 1;
+		else if ((q = strrchr(buf, ':')) != NULL)
+			p = q + 1;
+		else {
+			p = buf;
+			*p = '\0';
+		}
+	}
 #endif
 
+	return (p);
+}
+
+
+/*
+ * util_dirname
+ *	Return the dirname of a file path
+ *
+ * Args:
+ *	path - The file path string
+ *
+ * Return:
+ *	The dirname string
+ */
+char *
+util_dirname(char *path)
+{
+	char		*p;
+#ifdef __VMS
+	char		*q;
+#endif
+	static char	buf[FILE_PATH_SZ];
+
+	if (path == NULL)
+		return NULL;
+
+	if ((int) strlen(path) >= FILE_PATH_SZ)
+		/* Error: path name too long */
+		return NULL;
+
+	(void) strcpy(buf, path);
+
+#ifdef __VMS
+	if ((p = strrchr(buf, ']')) == NULL)
+#else
+	if ((p = strrchr(buf, '/')) == NULL)
+#endif
+		return (buf);
+
+#ifdef __VMS
+	if (*++p == '\0') {
+		/* The supplied path is a directory - special handling */
+		if ((q = strrchr(buf, '.')) != NULL) {
+			*q = ']';
+			*++q = '\0';
+		}
+		else if ((q = strrchr(buf, ':')) != NULL)
+			*++q = '\0';
+		else
+			p = buf;
+	}
 	*p = '\0';
+#else
+	if (p == buf)
+		*++p = '\0';
+	else
+		*p = '\0';
+#endif
 
 	return (buf);
 }
@@ -357,9 +475,13 @@ util_dirname(char *path)
 char *
 util_loginname(void)
 {
-#ifndef __VMS
-	struct passwd	*pw;
 	char		*cp;
+#ifdef __VMS
+	cp = getlogin();
+	if (cp != NULL)
+		return (cp);
+#else
+	struct passwd	*pw;
 
 	/* Get login name from the password file if possible */
 	setpwent();
@@ -515,6 +637,57 @@ util_mkdir(char *path, mode_t mode)
 }
 
 
+/*
+ * util_setperm
+ *	Set the file permissions of a file.
+ *
+ * Args:
+ *	path - file path name
+ *
+ * Return:
+ *	Nothing
+ */
+void
+util_setperm(char *path, char *modestr)
+{
+	unsigned int	mode;
+
+	/* Set the database file permissions */
+	(void) sscanf(modestr, "%o", &mode);
+
+	/* Make sure the file is at least readable to the user just
+	 * in case mode is bogus.
+	 */
+	mode |= S_IRUSR;
+
+	/* Turn off extraneous bits */
+	mode &= ~(S_ISUID | S_ISGID | S_IXUSR | S_IXGRP | S_IXOTH);
+
+	/* Set file permission */
+	(void) chmod(path, (mode_t) mode);
+}
+
+
+/*
+ * util_monname
+ *	Convert an interger month to an abbreviated 3-letter month
+ *	name string.
+ *
+ * Args:
+ *	mon - The integer month (0 is January, 11 is December).
+ *
+ * Return:
+ *	The month name string
+ */
+char *
+util_monname(int mon)
+{
+	if (mon < 0 || mon >= 12)
+		return ("???");
+	return (mon_name[mon]);
+}
+
+
 /*
  * util_isexecutable
  *	Verify executability, given a path to a program
@@ -633,7 +806,10 @@ util_checkcmd(char *cmd)
 			c2 = *r;
 			*r = '\0';
 
-			path = (char *) MEM_ALLOC(strlen(q) + strlen(cmd) + 2);
+			path = (char *) MEM_ALLOC(
+				"checkcmd_path",
+				strlen(q) + strlen(cmd) + 2
+			);
 			if (path == NULL)
 				return FALSE;	/* shrug */
 
@@ -650,7 +826,10 @@ util_checkcmd(char *cmd)
 		}
 		if (!isexe) {
 			/* Check last component in PATH */
-			path = (char *) MEM_ALLOC(strlen(q) + strlen(cmd) + 2);
+			path = (char *) MEM_ALLOC(
+				"checkcmd_path",
+				strlen(q) + strlen(cmd) + 2
+			);
 			if (path == NULL)
 				return FALSE;	/* shrug */
 
@@ -687,10 +866,8 @@ util_checkcmd(char *cmd)
 int
 util_runcmd(char *cmd, void (*workproc)(int), int workarg)
 {
-#ifdef __VMS
-	return 0;	/* not supported on VMS */
-#else
 	int		ret;
+#ifndef __VMS
 	pid_t		cpid;
 	waitret_t	stat_val;
 
@@ -731,23 +908,27 @@ util_runcmd(char *cmd, void (*workproc)(int), int workarg)
 		return (ret);
 	}
 
-	DBGPRN(errfp, "\nSetting uid to %d, gid to %d\n",
-		(int) util_get_ouid(), (int) util_get_ogid());
-
 	/* Force uid and gid to original setting */
-	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
+	if (!util_set_ougid())
 		exit(errno);
+#endif	/* __VMS */
 
 	/* Do the command */
-	DBGPRN(errfp, "\nCommand: [%s]\n", cmd);
-	stat_val = system(cmd);
+	DBGPRN(errfp, "Command: [%s]\n", cmd);
+	ret = system(cmd);
+
+#ifdef __VMS
+	DBGPRN(errfp, "Command exit status %d\n", ret);
+	return (ret);
+#else
+	stat_val = (waitret_t) ret;
+
 	if (WIFEXITED(stat_val))
 		ret = WEXITSTATUS(stat_val);
 	else
 		ret = EXE_ERR;
 
 	exit(ret);
-
 	/*NOTREACHED*/
 #endif	/* __VMS */
 }
@@ -911,8 +1092,9 @@ util_strcasecmp(char *s1, char *s2)
 		return 0;
 
 	/* Allocate tmp buffers */
-	if ((buf1 = (char *) MEM_ALLOC(strlen(s1)+1)) == NULL ||
-	    (buf2 = (char *) MEM_ALLOC(strlen(s2)+1)) == NULL) {
+	buf1 = (char *) MEM_ALLOC("strcasecmp_buf1", strlen(s1)+1);
+	buf2 = (char *) MEM_ALLOC("strcasecmp_buf2", strlen(s2)+1);
+	if (buf1 == NULL || buf2 == NULL) {
 		fprintf(errfp, "Error: %s\n", app_data.str_nomemory);
 		exit(1);
 	}
@@ -935,7 +1117,7 @@ util_strcasecmp(char *s1, char *s2)
 
 
 /*
- * util_strcasecmp
+ * util_strncasecmp
  *	Compare two strings a la strncmp(), except it is case-insensitive.
  *
  * Args:
@@ -958,8 +1140,9 @@ util_strncasecmp(char *s1, char *s2, int n)
 		return 0;
 
 	/* Allocate tmp buffers */
-	if ((buf1 = (char *) MEM_ALLOC(strlen(s1)+1)) == NULL ||
-	    (buf2 = (char *) MEM_ALLOC(strlen(s2)+1)) == NULL) {
+	buf1 = (char *) MEM_ALLOC("strncasecmp_buf1", strlen(s1)+1);
+	buf2 = (char *) MEM_ALLOC("strncasecmp_buf2", strlen(s2)+1);
+	if (buf1 == NULL || buf2 == NULL) {
 		fprintf(errfp, "Error: %s\n", app_data.str_nomemory);
 		exit(1);
 	}
@@ -1097,7 +1280,8 @@ util_text_reduce(char *str)
 			**t,
 			*newstr;
 
-	if ((newstr = (char *) MEM_ALLOC(strlen(str) + 1)) == NULL)
+	if ((newstr = (char *) MEM_ALLOC("text_reduce_newstr",
+					 strlen(str) + 1)) == NULL)
 		return NULL;
 
 	p1 = str;
@@ -1179,7 +1363,8 @@ util_cgi_xlate(char *str)
 		*q,
 		*new_str;
 
-	if ((q = new_str = (char *) MEM_ALLOC(strlen(str) * 3)) == NULL)
+	if ((q = new_str = (char *) MEM_ALLOC("cgi_xlate_newstr",
+					      (strlen(str) * 3) + 5)) == NULL)
 		return NULL;
 
 	/* Skip leading whitespaces */
@@ -1201,8 +1386,11 @@ util_cgi_xlate(char *str)
 				q++;
 			}
 		}
-		else if (!isalnum(*p)) {
-			/* Non-alphanumeric */
+		else if ((ispunct(*p) &&
+			  *p != '_' && *p != '.' &&
+			  *p != '*' && *p != '@') ||
+			 (*p & 0x80)) {
+			/* Need URL-encoding */
 			(void) sprintf(q, "%%%02X", (int) (*p));
 			q += 3;
 			p++;
@@ -1213,6 +1401,8 @@ util_cgi_xlate(char *str)
 			q++;
 			p++;
 		}
+		else
+			p++;
 	}
 	*q = '\0';
 
@@ -1444,6 +1634,9 @@ util_dbgdump(char *title, byte_t *data, int len)
 
 /* Function prototypes */
 extern void	delete();
+
+#ifdef VMS_USE_OWN_DIRENT
+
 extern int	LIB$FIND_FILE();
 extern void	LIB$FIND_FILE_END();
 extern void	SYS$FILESCAN();
@@ -1550,6 +1743,8 @@ util_readdir(DIR *dp)
 		return NULL;
 }
 
+#endif	/* VMS_USE_OWN_DIRENT */
+
 
 /*
  * util_waitpid
@@ -1627,7 +1822,7 @@ util_link(char *file1, char *file2)
 {
 	FILE	*fp1,
 		*fp2;
-	char	buf[STR_BUF_SZ * 2];
+	char	buf[STR_BUF_SZ * 16];
 
 	fp1 = fopen(file1, "r");
 	fp2 = fopen(file2, "w");
@@ -1646,3 +1841,69 @@ util_link(char *file1, char *file2)
 
 #endif	/* __VMS */
 
+#ifdef MEM_DEBUG
+/*
+ * For memory allocation debugging
+ */
+
+
+/*
+ * util_dbg_malloc
+ *	Wrapper for malloc(3).
+ */
+void *
+util_dbg_malloc(char *name, size_t size)
+{
+	void	*ptr;
+
+	ptr = _MEM_ALLOC(size);
+	(void) fprintf(stderr, "Malloc(%s, %d) => 0x%x\n", name, size, ptr);
+	return (ptr);
+}
+
+
+/*
+ * util_dbg_realloc
+ *	Wrapper for realloc(3).
+ */
+void *
+util_dbg_realloc(char *name, void *ptr, size_t size)
+{
+	void	*nptr;
+
+	nptr = _MEM_REALLOC(ptr, size);
+	(void) fprintf(stderr, "Realloc(%s, 0x%x, %d) => 0x%x\n",
+			name, ptr, size, nptr);
+	return (nptr);
+}
+
+
+/*
+ * util_dbg_calloc
+ *	Wrapper for calloc(3).
+ */
+void *
+util_dbg_calloc(char *name, size_t nelem, size_t elsize)
+{
+	void	*ptr;
+
+	ptr = _MEM_CALLOC(nelem, elsize);
+	(void) fprintf(stderr, "Calloc(%s, %d, %d) => 0x%x\n",
+			name, nelem, elsize, ptr);
+	return (ptr);
+}
+
+
+/*
+ * util_dbg_free
+ *	Wrapper for free(3).
+ */
+void
+util_dbg_free(void *ptr)
+{
+	(void) fprintf(stderr, "Free(0x%x)\n", ptr);
+	_MEM_FREE(ptr);
+}
+
+#endif	/* MEM_DEBUG */
+
