@@ -6,7 +6,7 @@
  *   libdi - CD Audio Player Device Interface Library
  *
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -28,7 +28,7 @@
 #define __UTIL_H__
 
 #ifndef LINT
-static char *_util_h_ident_ = "@(#)util.h	6.17 98/05/14";
+static char *_util_h_ident_ = "@(#)util.h	6.24 00/01/09";
 #endif
 
 
@@ -45,6 +45,8 @@ extern void		util_init(void);
 extern void		util_start(void);
 extern uid_t		util_get_ouid(void);
 extern gid_t		util_get_ogid(void);
+extern bool_t		util_set_ougid(void);
+extern struct utsname	*util_get_uname(void);
 extern sword32_t	util_ltobcd(sword32_t);
 extern sword32_t	util_bcdtol(sword32_t);
 extern bool_t		util_stob(char *);
@@ -54,6 +56,8 @@ extern char		*util_loginname(void);
 extern char		*util_homedir(uid_t);
 extern char		*util_uhomedir(char *);
 extern bool_t		util_mkdir(char *, mode_t);
+extern void		util_setperm(char *, char *);
+extern char		*util_monname(int);
 extern bool_t		util_isexecutable(char *);
 extern bool_t		util_checkcmd(char *);
 extern int		util_runcmd(char *, void (*)(int), int);
@@ -85,5 +89,14 @@ extern int		util_link(char *, char *);
 extern int		util_unlink(char *);
 #endif
 
+#ifdef MEM_DEBUG
+void			*util_dbg_malloc(char *name, size_t size);
+void			*util_dbg_realloc(char *name, void *ptr, size_t size);
+void			*util_dbg_calloc(char *name, size_t nelem,
+					 size_t elsize);
+void			util_dbg_free(void *ptr);
+#endif
+
 #endif	/* __UTIL_H__ */
 
+
