@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
  */
 
 #ifndef LINT
-static char *_aixioc_c_ident_ = "@(#)aixioc.c	6.45 98/05/18";
+static char *_aixioc_c_ident_ = "@(#)aixioc.c	6.62 00/01/31";
 #endif
 
 #include "common_d/appenv.h"
@@ -46,7 +46,8 @@ extern FILE		*errfp;
 extern di_client_t	*di_clinfo;
 
 
-STATIC bool_t	aixioc_run_ab(curstat_t *),
+STATIC bool_t	aixioc_pause_resume(bool_t),
+		aixioc_run_ab(curstat_t *),
 		aixioc_run_sample(curstat_t *),
 		aixioc_run_prog(curstat_t *),
 		aixioc_run_repeat(curstat_t *),
@@ -75,7 +76,9 @@ STATIC bool_t	aixioc_not_open = TRUE,		/* Device not opened yet */
 		aixioc_start_search,		/* Start FF/REW play segment */
 		aixioc_idx_pause,		/* Prev/next index pausing */
 		aixioc_fake_stop,		/* Force a completion status */
-		aixioc_playing;			/* Currently playing */
+		aixioc_playing,			/* Currently playing */
+		aixioc_paused,			/* Currently paused */
+		aixioc_override_ap;		/* Override auto-play */
 STATIC word32_t	aixioc_ab_start_addr,		/* A->B mode start block */
 		aixioc_ab_end_addr,		/* A->B mode end block */
 		aixioc_sav_end_addr;		/* Err recov saved end addr */
@@ -514,17 +517,30 @@ aixioc_start_stop(bool_t start, bool_t loej)
 	struct cd_audio_cmd	cmd;
 	bool_t			ret = FALSE;
 
+	if (app_data.strict_pause_resume && aixioc_paused)
+		(void) aixioc_pause_resume(TRUE);
+
 	if (start) {
 		/* Can't do anything about it */
-		return FALSE;
+		return (!loej);
 	}
 	else {
 		aixioc_playing = FALSE;
 
 		cmd.audio_cmds = CD_STOP_AUDIO;
 		ret = aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE);
-		if (ret && loej)
+		if (ret && loej) {
 			ret = aixioc_send(IDE_CDEJECT, 0, app_data.debug);
+
+			/* Delay a bit to let the CD load or eject.
+			 * This is a hack to work around firmware bugs
+			 * in some CD-ROM drives.  These drives don't
+			 * handle new commands well when the CD is
+			 * loading/ejecting with the IMMED bit set in
+			 * the Start/Stop Unit command.
+	 		 */
+			(void) sleep((app_data.ins_interval + 1000 - 1) / 1000);
+		}
 	}
 
 	return (ret);
@@ -542,13 +558,22 @@ aixioc_start_stop(bool_t start, bool_t loej)
  *	TRUE - success
  *	FALSE - failure
  */
-bool_t
+STATIC bool_t
 aixioc_pause_resume(bool_t resume)
 {
 	struct cd_audio_cmd	cmd;
+	bool_t			ret;
 
-	cmd.audio_cmds = (resume) ? CD_RESUME_AUDIO : CD_PAUSE_AUDIO;
-	return (aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE));
+	if (!app_data.pause_supp)
+		return FALSE;
+
+	cmd.audio_cmds = resume ? CD_RESUME_AUDIO : CD_PAUSE_AUDIO;
+
+	ret = aixioc_send(IDE_CDAUDIO, (void *) &cmd, TRUE);
+	if (ret)
+		aixioc_paused = !resume;
+
+	return (ret);
 }
 
 
@@ -633,9 +658,18 @@ aixioc_do_playaudio(
 	do_playti = (addr_fmt & ADDR_TRKIDX) && app_data.playti_supp;
 
 	if (do_playmsf || do_playti) {
-		if (aixioc_playing)
-			/* Pause playback first */
-			(void) aixioc_pause_resume(FALSE);
+		if (aixioc_paused) {
+			if (app_data.strict_pause_resume) {
+				/* Resume first */
+				(void) aixioc_pause_resume(TRUE);
+			}
+		}
+		else if (aixioc_playing) {
+			if (app_data.play_pause_play) {
+				/* Pause first */
+				(void) aixioc_pause_resume(FALSE);
+			}
+		}
 	}
 
 	if (do_playmsf)
@@ -644,8 +678,10 @@ aixioc_do_playaudio(
 	if (!ret && do_playti)
 		ret = aixioc_play_trkidx(trk, idx, trk, idx);
 
-	if (ret)
+	if (ret) {
 		aixioc_playing = TRUE;
+		aixioc_paused = FALSE;
+	}
 
 	return (ret);
 }
@@ -889,6 +925,7 @@ aixioc_get_playstatus(curstat_t *s)
 	case CD_COMPLETED:
 	case CD_NOT_VALID:
 	case CD_NO_AUDIO:
+	default:
 		done = TRUE;
 
 		if (!aixioc_fake_stop)
@@ -920,11 +957,6 @@ aixioc_get_playstatus(curstat_t *s)
 		}
 
 		break;
-
-	default:
-		/* Something is wrong with the data. */
-		done = FALSE;
-		break;
 	}
 
 	if (done) {
@@ -948,6 +980,10 @@ aixioc_get_playstatus(curstat_t *s)
 			/* Spin down the disc */
 			aixioc_start_stop(FALSE, FALSE);
 		}
+		if (app_data.done_exit) {
+			/* Exit */
+			di_clinfo->quit(s);
+		}
 
 		in_aixioc_get_playstatus = FALSE;
 		return FALSE;
@@ -977,6 +1013,9 @@ aixioc_cfg_vol(int vol, curstat_t *s, bool_t query)
 	int			vol1,
 				vol2;
 
+	if (!app_data.mselvol_supp)
+		return 0;
+
 	if (query) {
 		cmd.audio_cmds = CD_INFO_AUDIO;
 
@@ -1285,6 +1324,9 @@ aixioc_stop_stat_poll(void)
 STATIC void
 aixioc_start_insert_poll(curstat_t *s)
 {
+	int	delay;
+	bool_t	first = TRUE;
+
 	if (aixioc_insert_polling || app_data.ins_disable ||
 	    (s->mode != MOD_BUSY && s->mode != MOD_NODISC))
 		return;
@@ -1298,10 +1340,17 @@ aixioc_start_insert_poll(curstat_t *s)
 	if (aixioc_ins_interval < 500)
 		aixioc_ins_interval = 500;
 
+	if (first) {
+		first = FALSE;
+		delay = 50;
+	}
+	else
+		delay = aixioc_ins_interval;
+
 	/* Start poll timer */
 	if (di_clinfo->timeout != NULL) {
 		aixioc_insert_id = di_clinfo->timeout(
-			aixioc_ins_interval,
+			delay,
 			aixioc_insert_poll,
 			(byte_t *) s
 		);
@@ -1514,7 +1563,8 @@ aixioc_disc_ready(curstat_t *s)
 
 	if (app_data.load_play) {
 		/* Start auto-play */
-		aixioc_play_pause(s);
+		if (!aixioc_override_ap)
+			aixioc_play_pause(s);
 	}
 	else if (app_data.load_spindown) {
 		/* Spin down disc in case the user isn't going to
@@ -1846,8 +1896,9 @@ aixioc_run_sample(curstat_t *s)
 			MSF_OFFSET
 		);
 
-		if (aixioc_do_playaudio(ADDR_BLK | ADDR_MSF,
-				 saddr, eaddr, &smsf, &emsf, 0, 0)) {
+		if (s->trkinfo[aixioc_next_sam].type != TYP_AUDIO ||
+		    aixioc_do_playaudio(ADDR_BLK | ADDR_MSF,
+				        saddr, eaddr, &smsf, &emsf, 0, 0)) {
 			aixioc_next_sam++;
 			return TRUE;
 		}
@@ -1961,7 +2012,7 @@ aixioc_run_prog(curstat_t *s)
 	aixioc_wait_display(start_addr, s);
 
 	if (s->mode == MOD_PAUSE) {
-		aixioc_pause_resume(FALSE);
+		(void) aixioc_pause_resume(FALSE);
 
 		/* Restore volume */
 		aixioc_mute_off(s);
@@ -2027,7 +2078,7 @@ aixioc_run_repeat(curstat_t *s)
 		);
 
 		if (s->mode == MOD_PAUSE) {
-			aixioc_pause_resume(FALSE);
+			(void) aixioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			aixioc_mute_off(s);
@@ -2267,12 +2318,25 @@ aixioc_shuffle(curstat_t *s, bool_t enable)
 void
 aixioc_load_eject(curstat_t *s)
 {
-	if (!aixioc_disc_ready(s)) {
-		/* Try ejecting anyway.  We can still open the tray, right? */
-		if (!aixioc_start_stop(FALSE, TRUE))
+	int	i;
+	bool_t	ret;
+
+	if (!aixioc_disc_ready(s) && app_data.load_supp) {
+		/* Disc not ready: try loading the disc */
+		if (!aixioc_start_stop(TRUE, TRUE))
 			DO_BEEP();
 
-		return;
+		/* Wait for CD to become ready */
+		for (i = 0; i < 3; i++) {
+			if ((ret = aixioc_disc_ready(s)) == TRUE)
+				break;
+			sleep(1);
+		}
+
+		if (ret || !app_data.eject_supp)
+			return;
+
+		/* Fall through and open tray again */
 	}
 
 	/* Eject the disc */
@@ -2309,7 +2373,7 @@ aixioc_load_eject(curstat_t *s)
 	DBCLEAR(s, FALSE);
 	DPY_ALL(s);
 
-	aixioc_start_stop(FALSE, TRUE);
+	(void) aixioc_start_stop(FALSE, TRUE);
 
 	if (app_data.eject_exit)
 		di_clinfo->quit(s);
@@ -2523,11 +2587,16 @@ aixioc_play_pause(curstat_t *s)
 	msf_t		start_msf,
 			end_msf;
 
+	aixioc_override_ap = TRUE;
+
 	if (!aixioc_disc_ready(s)) {
+		aixioc_override_ap = FALSE;
 		DO_BEEP();
 		return;
 	}
 
+	aixioc_override_ap = FALSE;
+
 	if (s->mode == MOD_NODISC)
 		s->mode = MOD_STOP;
 
@@ -2705,10 +2774,8 @@ aixioc_stop(curstat_t *s, bool_t stop_disc)
 	 * immediately go into play state again.  Not spinning down
 	 * the drive makes things a little faster...
 	 */
-	if (!aixioc_disc_ready(s)) {
-		DO_BEEP();
+	if (!aixioc_disc_ready(s))
 		return;
-	}
 
 	switch (s->mode) {
 	case MOD_PLAY:
@@ -2733,7 +2800,6 @@ aixioc_stop(curstat_t *s, bool_t stop_disc)
 		break;
 
 	default:
-		DO_BEEP();
 		break;
 	}
 }
@@ -2920,7 +2986,7 @@ aixioc_prevtrk(curstat_t *s)
 			aixioc_wait_display(start_addr, s);
 
 			if (s->mode == MOD_PAUSE) {
-				aixioc_pause_resume(FALSE);
+				(void) aixioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				aixioc_mute_off(s);
@@ -3073,7 +3139,7 @@ aixioc_nexttrk(curstat_t *s)
 			aixioc_wait_display(start_addr, s);
 
 			if (s->mode == MOD_PAUSE) {
-				aixioc_pause_resume(FALSE);
+				(void) aixioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				aixioc_mute_off(s);
@@ -3175,6 +3241,9 @@ aixioc_previdx(curstat_t *s)
 			return;
 		}
 
+		/* A small delay to make sure the command took effect */
+		util_delayms(10);
+
 		aixioc_idx_pause = TRUE;
 
 		if (!aixioc_pause_resume(FALSE)) {
@@ -3217,7 +3286,7 @@ aixioc_previdx(curstat_t *s)
 		aixioc_idx_pause = FALSE;
 
 		if (s->mode == MOD_PAUSE) {
-			aixioc_pause_resume(FALSE);
+			(void) aixioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			aixioc_mute_off(s);
@@ -3292,6 +3361,9 @@ aixioc_nextidx(curstat_t *s)
 			return;
 		}
 
+		/* A small delay to make sure the command took effect */
+		util_delayms(10);
+
 		aixioc_idx_pause = TRUE;
 
 		if (!aixioc_pause_resume(FALSE)) {
@@ -3334,7 +3406,7 @@ aixioc_nextidx(curstat_t *s)
 		aixioc_idx_pause = FALSE;
 
 		if (s->mode == MOD_PAUSE) {
-			aixioc_pause_resume(FALSE);
+			(void) aixioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			aixioc_mute_off(s);
@@ -3465,7 +3537,7 @@ aixioc_rew(curstat_t *s, bool_t start)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				aixioc_pause_resume(FALSE);
+				(void) aixioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				aixioc_mute_off(s);
@@ -3595,7 +3667,7 @@ aixioc_ff(curstat_t *s, bool_t start)
 				}
 			}
 			if (s->mode == MOD_PAUSE) {
-				aixioc_pause_resume(FALSE);
+				(void) aixioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				aixioc_mute_off(s);
@@ -3687,7 +3759,7 @@ aixioc_warp(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				aixioc_pause_resume(FALSE);
+				(void) aixioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				aixioc_mute_off(s);
@@ -3773,7 +3845,10 @@ void
 aixioc_start(curstat_t *s)
 {
 	/* Check to see if disc is ready */
-	(void) aixioc_disc_ready(s);
+	if (di_clinfo->timeout != NULL)
+		aixioc_start_insert_poll(s);
+	else
+		(void) aixioc_disc_ready(s);
 
 	/* Update display */
 	DPY_ALL(s);
