@@ -1,13 +1,13 @@
 #!/bin/sh
 #
-# @(#)config.sh	6.111 98/05/17
+# @(#)config.sh	6.145 00/01/12
 #
 # Script to set up the device-dependent configuration files.
 #
 #    xmcd  - Motif(tm) CD Audio Player
 #    cda   - Command-line CD Audio Player
 #
-#    Copyright (C) 1993-1998  Ti Kan
+#    Copyright (C) 1993-2000  Ti Kan
 #    E-mail: ti@amb.org
 #
 #    This program is free software; you can redistribute it and/or modify
@@ -28,13 +28,14 @@
 PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin:/usr/ucb
 export PATH
 
-VER=2.3
+VER=2.6
 
 # Change the following directory to fit your local configuration
 BINDIR=/usr/bin/X11
 LIBDIR=/usr/lib/X11
 XMCDLIB=/usr/lib/X11/xmcd
-APPDEFAULTSDIR=/usr/lib/X11/app-defaults
+CDDBDIR=/usr/lib/X11/xmcd/cddb
+DISCOGDIR=/usr/lib/X11/xmcd/discog
 
 ERRFILE=/tmp/xmcd.err
 TMPFILE=/tmp/.xmcdcfg.$$
@@ -52,7 +53,7 @@ BROWSERS="netscape Mosaic"
 
 # Utility functions
 
-doexit()
+do_exit()
 {
 	if [ $1 -eq 0 ]
 	then
@@ -70,7 +71,7 @@ doexit()
 	exit $1
 }
 
-logerr()
+log_err()
 {
 	if [ "$1" = "-p" ]
 	then
@@ -79,7 +80,7 @@ logerr()
 	$ECHO "$2" >>$ERRFILE
 }
 
-getstr()
+get_str()
 {
 	$ECHO "$* \c"
 	read ANS
@@ -91,7 +92,7 @@ getstr()
 	fi
 }
 
-getyn()
+get_yn()
 {
 	if [ -z "$YNDEF" ]
 	then
@@ -133,43 +134,68 @@ getyn()
 }
 
 
-dolink()
+do_link()
 {
+	source=$1
+	target=$2
+
 	# Try symlink first
-	ln -s $1 $2 2>/dev/null
+	ln -s $source $target 2>/dev/null
 	if [ $? -ne 0 ]
 	then
 		# Use hard link
-		ln $1 $2 2>/dev/null
+		ln $source $target 2>/dev/null
 	fi
 	RETSTAT=$?
 	if [ $RETSTAT -ne 0 ]
 	then
-		logerr -p "Cannot link $1 -> $2"
+		log_err -p "Cannot link $source -> $target"
 	fi
 	return $RETSTAT
 }
 
 
-makedir()
+do_chown()
 {
-	$ECHO "\t$1"
-	if [ ! -d $1 ]
-	then
-		mkdir -p $1
-	fi
-	if [ $3 != _default_ ]
-	then
-		chown $3 $1 2>/dev/null
-	fi
-	if [ $4 != _default_ ]
-	then
-		chgrp $4 $1 2>/dev/null
-	fi
 	if [ $2 != _default_ ]
 	then
-		chmod $2 $1 2>/dev/null
+		chown $1 $2 2>/dev/null
 	fi
+}
+
+
+do_chgrp()
+{
+	if [ $2 != _default_ ]
+	then
+		chgrp $1 $2 2>/dev/null
+	fi
+}
+
+
+do_chmod()
+{
+	if [ $2 != _default_ ]
+	then
+		chmod $1 $2 2>/dev/null
+	fi
+}
+
+
+make_dir()
+{
+	dir=$1
+	perm=$2
+	owner=$3
+	group=$4
+	$ECHO "\t$dir"
+	if [ ! -d $dir ]
+	then
+		mkdir -p $dir
+	fi
+	do_chown $owner $dir 2>/dev/null
+	do_chgrp $group $dir 2>/dev/null
+	do_chmod $perm $dir 2>/dev/null
 	return 0
 }
 
@@ -181,13 +207,13 @@ ask_scsi_config()
 	$ECHO "  xmcd will probably not work."
 
 	YNDEF=n
-	if getyn "\n  Do you want to continue"
+	if get_yn "\n  Do you want to continue"
 	then
 		METHOD=0
 		VENDOR=0
 		VOLBASE=0
 		VOLTAPER=0
-		PLAYNOTUR=0
+		PLAYNOTUR=False
 	else
 		return 1
 	fi
@@ -196,7 +222,7 @@ ask_scsi_config()
 	$ECHO "  your drive.  If you don't know the answer, try accepting"
 	$ECHO "  the default values, and if problems occur when using"
 	$ECHO "  xmcd, reconfigure the settings by running this script"
-	$ECHO "  again, or editing the $CONFIG"
+	$ECHO "  again, or editing the $CFGDIR/$CONFIGFILE"
 	$ECHO "  file."
 	$ECHO "\n  If you get an unlisted drive working with xmcd in this"
 	$ECHO "  manner, the author of xmcd would like to hear from you"
@@ -206,17 +232,17 @@ ask_scsi_config()
 	while :
 	do
 	    YNDEF=n
-	    if getyn "\n  Is your drive on $XMCD_DEV a multi-disc changer"
+	    if get_yn "\n  Is your drive on $XMCD_DEV a multi-disc changer"
 	    then
 		$ECHO "\n  Select one of the following changer methods:\n"
 		$ECHO "  1.\tSCSI LUN addressing method"
 		$ECHO "  2.\tSCSI medium changer method"
 		$ECHO "  3.\tother"
-		$ECHO "  q.\tquit"
+		$ECHO "  q.\tquit (abort configuration)"
 
 		while :
 		do
-		    if getstr "\n  Enter choice: [1]"
+		    if get_str "\n  Enter choice: [1]"
 		    then
 			case "$ANS" in
 			[1-3])
@@ -249,7 +275,7 @@ ask_scsi_config()
 		    CHGMETHOD="$ANS"
 		    while :
 		    do
-			if getstr \
+			if get_str \
 			    "\n  How many discs does the drive support?:"
 			then
 			    case "$ANS" in
@@ -283,7 +309,7 @@ ask_scsi_config()
 	    $ECHO "\n  Does your drive on $XMCD_DEV support the following:\n"
 
 	    YNDEF=y
-	    if getyn "  - The Play_Audio_MSF SCSI command"
+	    if get_yn "  - The Play_Audio_MSF SCSI command"
 	    then
 		    PLAYMSF=True
 	    else
@@ -291,7 +317,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=n
-	    if getyn "  - The Play_Audio(12) SCSI command"
+	    if get_yn "  - The Play_Audio(12) SCSI command"
 	    then
 		    PLAY12=True
 	    else
@@ -299,7 +325,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn "  - The Play_Audio(10) SCSI command"
+	    if get_yn "  - The Play_Audio(10) SCSI command"
 	    then
 		    PLAY10=True
 	    else
@@ -307,7 +333,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn "  - The Play_Audio_Track/Index command"
+	    if get_yn "  - The Play_Audio_Track/Index command"
 	    then
 		    PLAYTI=True
 	    else
@@ -315,7 +341,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=n
-	    if getyn "  - Caddy load via the Start_Stop_Unit SCSI command"
+	    if get_yn "  - Caddy load via the Start_Stop_Unit SCSI command"
 	    then
 		    LOAD=True
 	    else
@@ -323,7 +349,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn "  - Caddy eject via the Start_Stop_Unit SCSI command"
+	    if get_yn "  - Caddy eject via the Start_Stop_Unit SCSI command"
 	    then
 		    EJECT=True
 	    else
@@ -331,7 +357,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn \
+	    if get_yn \
 		"  - Disable block descriptor in the Mode_Sense SCSI command"
 	    then
 		    MODEDBD=True
@@ -340,11 +366,11 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn \
+	    if get_yn \
 		"  - Audio volume control via the Mode_Select SCSI command"
 	    then
 		    YNDEF=y
-		    if getyn \
+		    if get_yn \
 		    "  - Independent SCSI Mode_Select volume control for each channel"
 		    then
 			    VOLSUPP=True
@@ -355,7 +381,7 @@ ask_scsi_config()
 		    fi
 
 		    YNDEF=y
-		    if getyn "  - Audio channel routing via SCSI Mode_Select"
+		    if get_yn "  - Audio channel routing via SCSI Mode_Select"
 		    then
 			    CHRSUPP=True
 		    else
@@ -369,7 +395,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn "  - The Pause/Resume SCSI command"
+	    if get_yn "  - The Pause/Resume SCSI command"
 	    then
 		    PAUSE=True
 	    else
@@ -377,7 +403,7 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=y
-	    if getyn "  - The Prevent/Allow_Medium_Removal SCSI command"
+	    if get_yn "  - The Prevent/Allow_Medium_Removal SCSI command"
 	    then
 		    CADDYLOCK=True
 	    else
@@ -385,13 +411,37 @@ ask_scsi_config()
 	    fi
 
 	    YNDEF=n
-	    if getyn "  - Data Format 1 of the Read_Subchannel SCSI command"
+	    if get_yn "  - Data Format 1 of the Read_Subchannel SCSI command"
 	    then
 		    CURPOSFMT=True
 	    else
 		    CURPOSFMT=False
 	    fi
 
+	    if [ "$PAUSE" = True ]
+	    then
+		YNDEF=n
+		if get_yn \
+		"\n  Does the drive require strict pause/resume symmetry"
+		then
+			STRICTPAUSERESUME=True
+		else
+			STRICTPAUSERESUME=False
+		fi
+
+		YNDEF=n
+		if get_yn \
+		"\n  Does the drive require a pause before reissuing play"
+		then
+			PLAYPAUSEPLAY=True
+		else
+			PLAYPAUSEPLAY=False
+		fi
+	    else
+		    STRICTPAUSERESUME=False
+		    PLAYPAUSEPLAY=False
+	    fi
+
 	    $ECHO "\n  This is the configuration for ${XMCD_DEV}:\n"
 	    $ECHO "  logicalDriverNumber:   $DRVNO"
 	    $ECHO "  mediumChangeMethod:    $CHGMETHOD"
@@ -406,12 +456,14 @@ ask_scsi_config()
 	    $ECHO "  volumeControlSupport:  $VOLSUPP"
 	    $ECHO "  balanceControlSupport: $BALSUPP"
 	    $ECHO "  pauseResumeSupport:    $PAUSE"
+	    $ECHO "  strictPauseResume:     $STRICTPAUSERESUME"
+	    $ECHO "  playPausePlay:         $PLAYPAUSEPLAY"
 	    $ECHO "  caddyLockSupport:      $CADDYLOCK"
 	    $ECHO "  curposFormat:          $CURPOSFMT"
 	    $ECHO "  multiPlay:             $MULTIPLAY"
 
 	    YNDEF=y
-	    if getyn "\n  Is this acceptable"
+	    if get_yn "\n  Is this acceptable"
 	    then
 		    break
 	    fi
@@ -426,14 +478,14 @@ ask_scsi_config()
 ask_nonscsi_config()
 {
 	$ECHO "\n  Non-SCSI drives are currently supported only on the BSDI,"
-	$ECHO "  Digital UNIX, FreeBSD, IBM AIX, Linux, NetBSD, OpenBSD, QNX,"
-	$ECHO "  SCO Open Server and Sun Solaris/x86 platforms.  You must"
-	$ECHO "  have the appropriate CD-ROM driver configured in your kernel."
-	$ECHO "  QNX users must use the QNX ioctl method, even for SCSI"
-	$ECHO "  drives."
+	$ECHO "  Digital UNIX, FreeBSD, HP-UX, IBM AIX, Linux, NetBSD,"
+	$ECHO "  OpenBSD, QNX, SCO Open Server and Sun Solaris platforms."
+	$ECHO "  You must have the appropriate CD-ROM driver configured in"
+	$ECHO "  your kernel.  QNX users must use the QNX ioctl method,"
+	$ECHO "  even for SCSI drives."
 
 	YNDEF=y
-	getyn "\n  Do you want to continue"
+	get_yn "\n  Do you want to continue"
 	if [ $? -ne 0 ]
 	then
 		return 1
@@ -448,26 +500,27 @@ ask_nonscsi_config()
 	$ECHO "  3.\tIBM AIX IDE ioctl method"
 	$ECHO "  4.\tBSDI BSD/OS ATAPI"
 	$ECHO "  5.\tSCO Open Server ATAPI BTLD"
-	$ECHO "  6.\tDigital UNIX ATAPI"
-	$ECHO "  q.\tquit"
+	$ECHO "  6.\tCompaq Tru64 UNIX / Digital UNIX ATAPI"
+	$ECHO "  7.\tHP-UX ATAPI"
+	$ECHO "  q.\tquit (abort configuration)"
 
 	while :
 	do
-		if getstr "\n  Enter choice:"
+		if get_str "\n  Enter choice:"
 		then
 			case "$ANS" in
-			[1-6])
+			[1-7])
 				break
 				;;
 			q)
 				return 1
 				;;
 			*)
-				$ECHO "  Please answer 1 to 6."
+				$ECHO "  Please answer 1 to 7."
 				;;
 			esac
 		else
-			$ECHO "  Please answer 1 to 6."
+			$ECHO "  Please answer 1 to 7."
 		fi
 	done
 
@@ -476,11 +529,11 @@ ask_nonscsi_config()
 	if [ -z "$NUMDISCS" ]
 	then
 		YNDEF=n
-		if getyn "\n  Is your drive on $XMCD_DEV a multi-disc changer"
+		if get_yn "\n  Is your drive on $XMCD_DEV a multi-disc changer"
 		then
 			while :
 			do
-				if getstr \
+				if get_str \
 				"\n  How many discs does the drive support?:"
 				then
 				    case "$ANS" in
@@ -531,11 +584,13 @@ ask_nonscsi_config()
 		CHRSUPP=False
 		VOLCTL=3
 		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
 		CADDYLOCK=False
 		CURPOSFMT=False
 		PLAYNOTUR=False
 		;;
-	2)	# FreeBS/NetBSDD ioctl method
+	2)	# FreeBSD/NetBSD ioctl method
 		METHOD=2
 		VENDOR=0
 		VOLBASE=0
@@ -552,6 +607,8 @@ ask_nonscsi_config()
 		CHRSUPP=False
 		VOLCTL=3
 		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
 		CADDYLOCK=True
 		CURPOSFMT=False
 		PLAYNOTUR=False
@@ -573,6 +630,8 @@ ask_nonscsi_config()
 		CHRSUPP=False
 		VOLCTL=3
 		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
 		CADDYLOCK=True
 		CURPOSFMT=False
 		PLAYNOTUR=False
@@ -595,6 +654,8 @@ ask_nonscsi_config()
 		CHRSUPP=False
 		VOLCTL=0
 		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
 		CADDYLOCK=False
 		CURPOSFMT=True
 		PLAYNOTUR=False
@@ -620,6 +681,8 @@ ask_nonscsi_config()
 		CHRSUPP=False
 		VOLCTL=3
 		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
 		CADDYLOCK=False
 		CURPOSFMT=False
 		PLAYNOTUR=False
@@ -627,7 +690,7 @@ ask_nonscsi_config()
 		NUMDISCS=1
 		MULTIPLAY=False
 		;;
-	6)	# Digital UNIX ATAPI (SCSI emulation)
+	6)	# Compaq Tru64 UNIX / Digital UNIX ATAPI (SCSI emulation)
 		# Set up as a "generic" SCSI-2 drive
 		METHOD=0
 		VENDOR=0
@@ -645,6 +708,35 @@ ask_nonscsi_config()
 		CHRSUPP=True
 		VOLCTL=7
 		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
+		CADDYLOCK=True
+		CURPOSFMT=True
+		PLAYNOTUR=False
+		CHGMETHOD=0
+		NUMDISCS=1
+		MULTIPLAY=False
+		;;
+	7)	# HP-UX ATAPI (SCSI emulation)
+		# Set up as a "generic" SCSI-2 drive
+		METHOD=0
+		VENDOR=0
+		VOLBASE=0
+		VOLTAPER=0
+		PLAYMSF=True
+		PLAY12=False
+		PLAY10=False
+		PLAYTI=True
+		LOAD=True
+		EJECT=True
+		MODEDBD=False
+		VOLSUPP=True
+		BALSUPP=True
+		CHRSUPP=True
+		VOLCTL=7
+		PAUSE=True
+		STRICTPAUSERESUME=False
+		PLAYPAUSEPLAY=False
 		CADDYLOCK=True
 		CURPOSFMT=True
 		PLAYNOTUR=False
@@ -678,12 +770,12 @@ config_drive()
 		do
 			if [ -f $i ]
 			then
-				if fgrep tblver=3 $CFGDIR/.tbl/$i \
+				if fgrep tblver=4 $CFGDIR/.tbl/$i \
 			   		>/dev/null 2>&1
 				then
 					$ECHO "$i \c"
 				else
-					logerr -p \
+					log_err -p \
 					"$CFGDIR/.tbl/$i version mismatch"
 				fi
 			fi
@@ -710,7 +802,7 @@ config_drive()
 	$ECHO "  $j.\tother (SCSI)" >>$TMPFILE
 	j=`expr $j + 1`
 	$ECHO "  $j.\tother (non-SCSI)" >>$TMPFILE
-	$ECHO "  q.\tquit" >>$TMPFILE
+	$ECHO "  q.\tquit (abort configuration)" >>$TMPFILE
 
 	$ECHO "\n  Device ($XMCD_DEV) configuration"
 	$ECHO "  Please select the drive brand:\n"
@@ -728,7 +820,7 @@ config_drive()
 
 	while :
 	do
-		if getstr "\n  Enter choice:"
+		if get_str "\n  Enter choice:"
 		then
 			if [ "$ANS" = q ]
 			then
@@ -771,8 +863,11 @@ config_drive()
 
 model_sel()
 {
+	BRAND=$1
+	CFGFILE=$2
+
 	$ECHO "\n  Device ($XMCD_DEV) configuration"
-	$ECHO "  Please select the $1 drive model:\n"
+	$ECHO "  Please select the $BRAND drive model:\n"
 
 	$AWK -F: '
 	BEGIN {
@@ -796,9 +891,9 @@ model_sel()
 	END {
 		printf("  %d.\t%-12s%s\n", n, "other", "SCSI")
 		printf("  %d.\t%-12s%s\n", n+1, "other", "non-SCSI")
-		printf("  q.\tquit\n")
+		printf("  q.\tquit (abort configuration)\n")
 	}
-	' $2 >$TMPFILE
+	' $CFGFILE >$TMPFILE
 
 
 	PGLEN=`wc -l $TMPFILE | $AWK '{ print $1 + 1 }'`
@@ -817,9 +912,9 @@ model_sel()
 
 	while :
 	do
-		if getstr "\n  Enter choice:"
+		if get_str "\n  Enter choice:"
 		then
-			j=`grep -v "^#" $2 | wc -l | sed 's/^[ 	]*//'`
+			j=`grep -v "^#" $CFGFILE | wc -l | sed 's/^[ 	]*//'`
 			j=`expr $j + 2`
 
 			if [ "$ANS" = q ]
@@ -837,7 +932,7 @@ model_sel()
 				ask_nonscsi_config
 				return $?
 			else
-				read_config $2 $ANS
+				read_config $CFGFILE $ANS
 				if [ "$METHOD" != 0 ]
 				then
 					ask_nonscsi_config
@@ -909,20 +1004,28 @@ read_config()
 			else
 				print "PAUSE=True"
 			if ($15 == 0)
+				print "STRICTPAUSERESUME=False"
+			else
+				print "STRICTPAUSERESUME=True"
+			if ($16 == 0)
+				print "PLAYPAUSEPLAY=False"
+			else
+				print "PLAYPAUSEPLAY=True"
+			if ($17 == 0)
 				print "CADDYLOCK=False"
 			else
 				print "CADDYLOCK=True"
-			if ($16 == 0)
+			if ($18 == 0)
 				print "CURPOSFMT=False"
 			else
 				print "CURPOSFMT=True"
-			if ($17 == 0)
+			if ($19 == 0)
 				print "PLAYNOTUR=False"
 			else
 				print "PLAYNOTUR=True"
 
-			printf("CHGMETHOD=%d\n", $18)
-			printf("NUMDISCS=%d\n", $19)
+			printf("CHGMETHOD=%d\n", $20)
+			printf("NUMDISCS=%d\n", $21)
 			if ($18 == 0)
 				print "MULTIPLAY=False"
 			else
@@ -947,9 +1050,9 @@ get_protocol()
 		$ECHO "\n  Protocol Selection\n"
 		$ECHO "  1.\tCDDBP (CD Database Protocol)"
 		$ECHO "  2.\tHTTP  (Hyper-Text Transport Protocol)"
-		$ECHO "  q.\tquit"
+		$ECHO "  q.\tquit (abort configuration)"
 
-		if getstr "\n  Enter Choice: [1]"
+		if get_str "\n  Enter Choice: [1]"
 		then
 			case "$ANS" in
 			1)
@@ -962,8 +1065,8 @@ get_protocol()
 				;;
 			q)
 				$ECHO "\nConfiguration aborted." >&2
-				logerr -n "Configuration aborted by user"
-				doexit 3
+				log_err -n "Configuration aborted by user"
+				do_exit 3
 				;;
 			*)
 				$ECHO "\nInvalid selection.  Try again."
@@ -980,7 +1083,7 @@ get_protocol()
 get_proxy()
 {
 	YNDEF=n
-	getyn "\n  Do you want to use a proxy server"
+	get_yn "\n  Do you want to use a proxy server"
 	if [ $? -ne 0 ]
 	then
 		USEPROXY=False
@@ -991,7 +1094,7 @@ get_proxy()
 	USEPROXY=True
 	while :
 	do
-		if getstr "  Enter the proxy server host name or IP number:"
+		if get_str "  Enter the proxy server host name or IP number:"
 		then
 			if ($ECHO "$ANS" | grep "[ 	]") >/dev/null 2>&1
 			then
@@ -1001,7 +1104,7 @@ get_proxy()
 				PROXYSERVER="$ANS"
 			fi
 
-			if getstr "  Enter the proxy server port number: [80]"
+			if get_str "  Enter the proxy server port number: [80]"
 			then
 				PROXYSERVER="${PROXYSERVER}:${ANS}"
 			fi
@@ -1027,9 +1130,9 @@ get_useserver()
 		$ECHO "\n  CDDB Information\n"
 		$ECHO "  1.\tConnect to CDDB server and get current list"
 		$ECHO "  2.\tUse built-in list"
-		$ECHO "  q.\tQuit\n"
+		$ECHO "  q.\tquit (abort configuration)\n"
 
-		getstr "  Enter choice: [1]"
+		get_str "  Enter choice: [1]"
 		if [ $? -ne 0 ]
 		then
 			ANS=1
@@ -1050,7 +1153,7 @@ get_useserver()
 			;;
 		q)
 			$ECHO "\nConfiguration aborted." >&2
-			doexit 3
+			do_exit 3
 			;;
 		*)
 			$ECHO "\nInvalid input.  Please try again."
@@ -1063,7 +1166,7 @@ get_useserver()
 
 get_host()
 {
-	if getstr "    Server host name or IP number:"
+	if get_str "    Server host name or IP number:"
 	then
 		HOST="$ANS"
 		return 0
@@ -1083,7 +1186,7 @@ get_port()
 		DFLT_PORT=888
 	fi
 
-	if getstr "    TCP port number: [$DFLT_PORT]"
+	if get_str "    TCP port number: [$DFLT_PORT]"
 	then
 		PORT="$ANS"
 	else
@@ -1094,7 +1197,7 @@ get_port()
 
 get_cgipath()
 {
-	if getstr "    Enter CGI path: [$DFLT_CGIPATH]"
+	if get_str "    Enter CGI path: [$DFLT_CGIPATH]"
 	then
 		CGIPATH="$ANS"
 	else
@@ -1106,6 +1209,7 @@ get_cgipath()
 print_servers()
 {
 	$ECHO "\n  CDDB server sites selection"
+	$ECHO "  Number of CDDB servers selected so far: $1"
 	$ECHO "\n\tServer host               Proto Port   Location\n"
 	$AWK '
 	BEGIN	{
@@ -1121,8 +1225,8 @@ print_servers()
 	END	{
 		printf("  %d.\tother site not listed\n", n++)
 		printf("  ?.\tshow this list again\n")
-		printf("  c.\tdone adding servers\n")
-		printf("  q.\tquit\n\n")
+		printf("  q.\tquit (abort configuration)\n")
+		printf("  c.\tdone adding servers\n\n")
 	}
 	' $TMPFILE
 }
@@ -1152,8 +1256,8 @@ set_server_entry()
 
 	if [ -z "$HOST" -o -z "$PORT" ]
 	then
-		logerr -p "Internal error: undefined HOST or PORT!"
-		doexit 1
+		log_err -p "Internal error: undefined HOST or PORT!"
+		do_exit 1
 	fi
 
 	case "$PROTOCOL" in
@@ -1218,10 +1322,10 @@ sel_servers()
 	$ECHO "Type 'c' when done."
 
 	cnt=0
-	print_servers
+	print_servers $cnt
 	while :
 	do
-		if getstr "  Enter CDDB server selection:"
+		if get_str "  Enter CDDB server selection:"
 		then
 			case "$ANS" in
 			[0-9]*)
@@ -1242,7 +1346,7 @@ sel_servers()
 			    fi
 			    ;;
 			'?')
-			    print_servers
+			    print_servers $cnt
 			    ;;
 			c)
 			    $ECHO "\nTotal CDDB servers configured: $cnt"
@@ -1250,8 +1354,8 @@ sel_servers()
 			    ;;
 			q)
 			    $ECHO "\nConfiguration aborted." >&2
-			    logerr -n "Configuration aborted by user"
-			    doexit 3
+			    log_err -n "Configuration aborted by user"
+			    do_exit 3
 			    ;;
 			*)
 			    $ECHO "  Please enter 1-${USRSPEC}, c, ? or q."
@@ -1274,10 +1378,23 @@ check_local_cddb()
 		return
 	fi
 
-	for i in $CDDBCATS
-	do
-		makedir $XMCDLIB/cddb/$i $CDIRPERM $OWNER $GROUP
-	done
+	if [ -n "$CDDBDIR" ]
+	then
+		for i in $CDDBCATS
+		do
+			make_dir $CDDBDIR/$i $CDIRPERM $OWNER $GROUP
+		done
+		for i in $CDDBCATS
+		do
+			make_dir $DISCOGDIR/$i $CDIRPERM $OWNER $GROUP
+		done
+		if [ -x $XMCDLIB/scripts/genidx ]
+		then
+			$XMCDLIB/scripts/genidx
+		fi
+	else
+		$ECHO "\nA local CD database directory was not defined."
+	fi
 }
 
 
@@ -1304,7 +1421,7 @@ cddb_config()
 	$ECHO "If this system is not linked to the Internet, then answer 'n'."
 
 	YNDEF=y
-	getyn "\n  Would you like to use Internet CD database servers"
+	get_yn "\n  Would you like to use Internet CD database servers"
 	if [ $? -ne 0 ]
 	then
 		$ECHO "\nNo Internet CD database servers are configured."
@@ -1441,7 +1558,7 @@ browser_config()
 			$ECHO "\nThe following is found on your system:"
 			$ECHO "    $i/$j"
 			YNDEF=y
-			if getyn "\n  Do you want xmcd to use this browser"
+			if get_yn "\n  Do you want xmcd to use this browser"
 			then
 				BDIR=$i
 				BROWSER=$j
@@ -1473,7 +1590,7 @@ browser_config()
 		    $ECHO "\n  Enter web browser path, type \"c\" to \c"
 		    $ECHO "continue without\n  specifying a browser path, \c"
 		    $ECHO "or type \"q\" to abort."
-		    if getstr "  [$BDIR/$i]:"
+		    if get_str "  [$BDIR/$i]:"
 		    then
 			    if [ "$ANS" = c ]
 			    then
@@ -1481,8 +1598,8 @@ browser_config()
 			    elif [ "$ANS" = q ]
 			    then
 				$ECHO "\nConfiguration aborted." >&2
-				logerr -n "Configuration aborted by user"
-				doexit 3
+				log_err -n "Configuration aborted by user"
+				do_exit 3
 			    fi
 
 			    if [ -x "$ANS" ]
@@ -1491,7 +1608,8 @@ browser_config()
 				i=`basename $ANS`
 				for j in $BROWSERS
 				do
-					if [ $j = $i ]
+					if (echo $i | grep -i $j) \
+						>/dev/null 2>&1
 					then
 						BROWSER=$i
 						break
@@ -1513,23 +1631,23 @@ browser_config()
 		done
 	fi
 
-	#
-	# Make netscape the default setting
-	#
-	BROWSER_RMT="netscape -remote 'openURL(%U)' >/dev/null 2>&1"
-	BROWSER_DIR="netscape '%U' >/dev/null 2>&1 &"
-
 	if [ -n "$BROWSER" ]
 	then
+	    #
+	    # Make netscape the default setting
+	    #
+	    BROWSER_RMT="$BROWSER -remote 'openURL(%U)' >/dev/null 2>&1"
+	    BROWSER_DIR="$BROWSER '%U' >/dev/null 2>&1 &"
+
 	    #
 	    # Set up browser configuration
 	    #
 	    case $BROWSER in
-	    netscape)
+	    *[nN][eE][tT][sS][cC][aA][pP][eE]*)
 		    BROWSER_RMT="$BDIR/$BROWSER_RMT"
 		    BROWSER_DIR="$BDIR/$BROWSER_DIR"
 		    ;;
-	    Mosaic)
+	    *[mM][oO][sS][aA][iI][cC]*)
 		    BROWSER_RMT="$SCRDIR/ncsarmt '%U'"
 		    BROWSER_DIR="$BDIR/Mosaic '%U' >/dev/null 2>&1 &"
 		    ;;
@@ -1540,14 +1658,14 @@ browser_config()
 	    $ECHO "\nNo web browser specified."
 	    $ECHO "\n  Do you have a supported web browser that can be run"
 	    YNDEF=n
-	    if getyn "  from a remote system"
+	    if get_yn "  from a remote system"
 	    then
 		while :
 		do
 		    $ECHO "\n  Enter the remote system host name, \c"
 		    $ECHO "type \"c\" to continue without\n  specifying \c"
 		    $ECHO "a host name, \c"
-		    if getstr "or type \"q\" to abort:"
+		    if get_str "or type \"q\" to abort:"
 		    then
 			case "$ANS" in
 			c)
@@ -1555,8 +1673,8 @@ browser_config()
 			    ;;
 			q)
 			    $ECHO "\nConfiguration aborted." >&2
-			    logerr -n "Configuration aborted by user"
-			    doexit 3
+			    log_err -n "Configuration aborted by user"
+			    do_exit 3
 			    ;;
 			*)
 			    RMTSYS="$ANS"
@@ -1575,7 +1693,7 @@ browser_config()
 			$ECHO "\n  Enter web browser path on \"$RMTSYS\", \c"
 			$ECHO "type \"c\" to\n  continue without specifying \c"
 			$ECHO "a path, or type \"q\" to abort."
-			if getstr "  [$BDIR/$i]: "
+			if get_str "  [$BDIR/$i]: "
 			then
 			    if [ "$ANS" = c ]
 			    then
@@ -1583,15 +1701,15 @@ browser_config()
 			    elif [ "$ANS" = q ]
 			    then
 				$ECHO "\nConfiguration aborted." >&2
-				logerr -n "Configuration aborted by user"
-				doexit 3
+				log_err -n "Configuration aborted by user"
+				do_exit 3
 			    fi
 
 			    BDIR=`dirname $ANS`
 			    i=`basename $ANS`
 			    for j in $BROWSERS
 			    do
-				if [ $j = $i ]
+				if (echo $i | grep -i $j) >/dev/null 2>&1
 				then
 				    BROWSER=$i
 				    break
@@ -1615,10 +1733,10 @@ browser_config()
 	    if [ -n "$BROWSER" ]
 	    then
 		case $BROWSER in
-		netscape)
+		*[nN][eE][tT][sS][cC][aA][pP][eE]*)
 			SCR=$SCRDIR/nswrap
 			;;
-		Mosaic)
+		*[mM][oO][sS][aA][iI][cC]*)
 			SCR=$SCRDIR/ncsawrap
 			;;
 		esac
@@ -1630,6 +1748,10 @@ browser_config()
 		$ECHO "users must have rsh(1) permissions on $RMTSYS"
 		$ECHO "in order to use the wwwWarp feature.  See"
 		$ECHO "rhosts(4) and hosts.equiv(4) for details."
+		$ECHO "\nThe Local Discography feature will work in this"
+		$ECHO "mode, only if the host \"$RMTSYS\" and your"
+		$ECHO "system has a NFS-shared $DISCOGDIR"
+		$ECHO "directory."
 	    else
 		$ECHO "\nNo web browser is configured."
 		$ECHO "The xmcd wwwWarp feature will not function."
@@ -1647,7 +1769,26 @@ trap "rm -f $TMPFILE $TMPSITES; exit 1" 1 2 3 5 15
 
 # Get platform information
 OS_SYS=`(uname -s) 2>/dev/null`
-OS_VER=`(uname -r) 2>/dev/null`
+OS_REL=`(uname -r) 2>/dev/null`
+OS_MACH=`(uname -m) 2>/dev/null`
+OS_NODE=`(uname -n) 2>/dev/null`
+
+if [ -z "$OS_SYS" ]
+then
+	OS_SYS=unkn
+fi
+if [ -z "$OS_REL" ]
+then
+	OS_REL=unkn
+fi
+if [ -z "$OS_MACH" ]
+then
+	OS_MACH=unkn
+fi
+if [ -z "$OS_NODE" ]
+then
+	OS_NODE=unkn
+fi
 
 # Use Sysv echo if possible
 if [ -x /usr/5bin/echo ]				# SunOS SysV echo
@@ -1675,33 +1816,34 @@ then
 fi
 
 $ECHO "\nXmcd version $VER Configuration Program"
+$ECHO "Setting up for host: ${OS_NODE}"
 $ECHO "--------------------------------------"
 
 # Sanity check
 
 if [ ! -w $CFGDIR ]
 then
-	logerr -p "No write permission in $CFGDIR"
-	doexit 1
+	log_err -p "No write permission in $CFGDIR"
+	do_exit 1
 fi
 
 if [ ! -r $CFGDIR/device.cfg ]
 then
-	logerr -p "Cannot find $CFGDIR/device.cfg"
-	doexit 2
+	log_err -p "Cannot find $CFGDIR/device.cfg"
+	do_exit 2
 fi
 
 fgrep cfgver=1 $CFGDIR/device.cfg >/dev/null 2>&1
 if [ $? -ne 0 ]
 then
-	logerr -p "$CFGDIR/device.cfg version mismatch"
-	doexit 2
+	log_err -p "$CFGDIR/device.cfg version mismatch"
+	do_exit 2
 fi
 
 if [ ! -d $CFGDIR/.tbl ]
 then
-	logerr -p "The directory $CFGDIR/.tbl is missing"
-	doexit 2
+	log_err -p "The directory $CFGDIR/.tbl is missing"
+	do_exit 2
 fi
 
 # Configure platform-dependent and device-dependent parameters
@@ -1758,7 +1900,7 @@ then
 	MAILCMD="Mail -s '%S' %A <%F >/dev/null 2>&1"
 elif [ "$OS_SYS" = HP-UX ]
 then
-	case $OS_VER in
+	case "$OS_REL" in
 	[AB].09*)	# HP-UX 9.x
 		DEVPREF=/dev/rdsk/c201d
 		DEVSUFF=s0
@@ -1775,10 +1917,10 @@ then
 		FIRST=4
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
-elif [ "$OS_SYS" = IRIX ]
+elif [ "$OS_SYS" = IRIX -o "$OS_SYS" = IRIX64 ]
 then
 	# SGI IRIX
 	DEVPREF=`hinv | grep CDROM | line | \
@@ -1797,7 +1939,7 @@ elif [ "$OS_SYS" = NetBSD ]
 then
 	# NetBSD
 	DEVPREF=/dev/rcd
-	case "`uname -m`" in
+	case "$OS_MACH" in
 	*86)
 		DEVSUFF=d
 		;;
@@ -1811,7 +1953,7 @@ elif [ "$OS_SYS" = OpenBSD ]
 then
 	# OpenBSD
 	DEVPREF=/dev/rcd
-	case "`uname -m`" in
+	case "$OS_MACH" in
 	*86)
 		DEVSUFF=d
 		;;
@@ -1823,14 +1965,21 @@ then
 	MAILCMD="Mail -s '%S' %A <%F >/dev/null 2>&1"
 elif [ "$OS_SYS" = OSF1 ]
 then
-	case "`uname -m`" in
+	case "$OS_MACH" in
 	alpha)	# Digital OSF/1
-		DEVPREF=/dev/rrz
-		DEVSUFF=c
-		FIRST=4
+		if [ -d /devices/rdisk ]
+		then
+			DEVPREF=/devices/rdisk/cdrom
+			DEVSUFF=c
+			FIRST=2
+		else
+			DEVPREF=/dev/rrz
+			DEVSUFF=c
+			FIRST=4
+		fi
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
 elif [ "$OS_SYS" = QNX ]
@@ -1852,9 +2001,9 @@ then
 	DEVPREF=/dev/ios0/rsdisk
 	DEVSUFF=s0
 	FIRST=006
-elif [ "$OS_SYS" = SunOS ]
+elif [ "$OS_SYS" = SunOS -o "$OS_SYS" = Solaris ]
 then
-	case $OS_VER in
+	case "$OS_REL" in
 	4.*)	# SunOS 4.x
 		case `arch -k` in
 		sun4[cm])
@@ -1864,13 +2013,13 @@ then
 			MAILCMD="Mail -s '%S' %A <%F >/dev/null 2>&1"
 			;;
 		*)
-			OS_VER=unknown
+			OS_REL=unknown
 			;;
 		esac
 		;;
 	5.*)	# SunOS 5.x
-		YNDEF=n
-		if getyn \
+		YNDEF=y
+		if get_yn \
 		"Does your system support the Volume Manager (/usr/sbin/vold)"
 		then
 			DEVPREF=/vol/dev/aliases/cdrom
@@ -1885,12 +2034,12 @@ then
 		fi
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
 elif [ "$OS_SYS" = ULTRIX ]
 then
-	case "`uname -m`" in
+	case "$OS_MACH" in
 	RISC)	# Digital Ultrix
 		DEVPREF=/dev/rrz
 		DEVSUFF=c
@@ -1898,12 +2047,12 @@ then
 		MAILCMD="Mail -s '%S' %A <%F >/dev/null 2>&1"
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
 elif [ -x /bin/ftx ] && ftx
 then
-	case $OS_VER in
+	case "$OS_REL" in
 	4.*)	
 		if [ -x /bin/hppa ] && hppa
 		then
@@ -1913,16 +2062,16 @@ then
 			FIRST=0
 		else
 			# On non-supported FTX variants
-			OS_VER=unknown
+			OS_REL=unknown
 		fi
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
 elif [ -x /bin/i386 -o -x /sbin/i386 ] && i386
 then
-	case $OS_VER in
+	case "$OS_REL" in
 	3.2)	# SCO UNIX/ODT/Open Server
 		DEVPREF=/dev/rcd
 		DEVSUFF=
@@ -1959,19 +2108,19 @@ then
 		FIRST=1
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
 elif [ -x /bin/m88k ] && m88k
 then
-	case $OS_VER in
+	case "$OS_REL" in
 	4.0)	# UNIX SVR4.0/88k
 		DEVPREF=/dev/rdsk/m187_c0d
 		DEVSUFF=s7
 		FIRST=3
 		;;
 	*)
-		OS_VER=unknown
+		OS_REL=unknown
 		;;
 	esac
 elif [ -r /vmunix ] && (strings /vmunix | fgrep NEWS-OS) >/dev/null 2>&1
@@ -1981,22 +2130,22 @@ then
 	DEVSUFF=c
 	FIRST=06
 else
-	OS_VER=unknown
+	OS_REL=unknown
 fi
 
 NOT_SUPPORTED="Error: You are not running an operating system that's\n\
 currently supported by xmcd."
 
-if [ "$OS_VER" = unknown ]
+if [ "$OS_REL" = unknown ]
 then
 	$ECHO "$NOT_SUPPORTED"
 	YNDEF=n
-	getyn "Would you like to proceed anyway"
+	get_yn "Would you like to proceed anyway"
 	if [ $? -ne 0 ]
 	then
 		$ECHO "\nConfiguration aborted." >&2
-		logerr -n "Configuration aborted by user"
-		doexit 3
+		log_err -n "Configuration aborted by user"
+		do_exit 3
 	fi
 fi
 
@@ -2017,8 +2166,8 @@ browser_config
 #
 $ECHO "\n\n*** DRIVE CONFIGURATION ***"
 $ECHO "\nNote that the default device shown is usually a SCSI CD-ROM."
-$ECHO "You will probably need to specify a difference device if your"
-$ECHO "CD-ROM drive is not SCSI."
+$ECHO "On some platforms, you may need to specify a different device"
+$ECHO "if your CD-ROM drive is not SCSI."
 EXITSTAT=0
 DRVNO=0
 DEVNO=$FIRST
@@ -2032,7 +2181,7 @@ do
 
 	while :
 	do
-		if getstr "\n  Enter device path: [$DEFAULT_DEV]"
+		if get_str "\n  Enter device path: [$DEFAULT_DEV]"
 		then
 			XMCD_DEV=$ANS
 		else
@@ -2060,8 +2209,8 @@ do
 		#
 		# Configure app-defaults/XMcd file
 		#
-		chmod 644 $APPDEFAULTSDIR/XMcd 2>/dev/null
-		if [ -w $APPDEFAULTSDIR/XMcd ]
+		chmod 644 $XMCDLIB/app-defaults/XMcd 2>/dev/null
+		if [ -w $XMCDLIB/app-defaults/XMcd ]
 		then
 			$AWK '
 			/^XMcd\.libdir:/ {
@@ -2075,28 +2224,12 @@ do
 			{
 				print $0
 			}' xmcdlib="$XMCDLIB" mailcmd="$MAILCMD" \
-				$APPDEFAULTSDIR/XMcd > /tmp/xmcd.$$
+				$XMCDLIB/app-defaults/XMcd > /tmp/xmcd.$$
 
-			cp /tmp/xmcd.$$ $APPDEFAULTSDIR/XMcd
+			cp /tmp/xmcd.$$ $XMCDLIB/app-defaults/XMcd
 			rm -f /tmp/xmcd.$$
-
-			if [ $LIBDIR != "/usr/lib/X11" -a \
-			     -d /usr/lib/X11/app-defaults ]
-			then
-				rm -f /usr/lib/X11/app-defaults/._trash_
-				>$APPDEFAULTSDIR/._trash_
-
-				if [ ! -f /usr/lib/X11/app-defaults/._trash_ ]
-				then
-					rm -f /usr/lib/X11/app-defaults/XMcd
-					dolink $APPDEFAULTSDIR/XMcd \
-						/usr/lib/X11/app-defaults/XMcd
-				fi
-
-				rm -f $APPDEFAULTSDIR/._trash_
-			fi
 		else
-			logerr -p "Cannot configure $APPDEFAULTSDIR/XMcd"
+			log_err -p "Cannot configure $XMCDLIB/app-defaults/XMcd"
 			EXITSTAT=1
 		fi
 
@@ -2149,27 +2282,30 @@ do
 
 			cp /tmp/xmcd.$$ $CFGDIR/common.cfg
 			rm -f /tmp/xmcd.$$
+			(cd $CFGDIR ;\
+			 rm -f common.cfg-${OS_NODE} ;\
+			 do_link common.cfg common.cfg-${OS_NODE})
 		else
-			logerr -p \
+			log_err -p \
 			    "Cannot configure $CFGDIR/common.cfg"
-			EXITSTAT=1
+			EXITSTAT=1 
 		fi
 	fi
 
-	CONFIG=$CFGDIR/`basename $XMCD_DEV`
+	CONFIGFILE=`basename $XMCD_DEV`
 
 	config_drive
 	CFGSTAT=$?
 	if [ $CFGSTAT -ne 0 ]
 	then
 		$ECHO "\nConfiguration aborted." >&2
-		logerr -n "Configuration aborted by user."
-		doexit $CFGSTAT
+		log_err -n "Configuration aborted by user."
+		do_exit $CFGSTAT
 	fi
 
 	if [ $NUMDISCS -gt 1 ]
 	then
-		echo "\n  This drive is a ${NUMDISCS}-disc changer."
+		$ECHO "\n  This drive is a ${NUMDISCS}-disc changer."
 
 		case $CHGMETHOD in
 		1)
@@ -2178,7 +2314,7 @@ do
 			$ECHO "  nodes for each LUN of the multi-disc changer."
 
 			YNDEF=y
-			if getyn "  Does your OS platforms support this?"
+			if get_yn "  Does your OS platforms support this?"
 			then
 				$ECHO "\n  Please enter the device nodes now."
 
@@ -2222,7 +2358,7 @@ do
 					SAVDEV=$LUNDEV
 					LUNDEV="${LUNPREF}${LUNNUM}${LUNSUFF}"
 
-					if getstr "  Disc $n: [$LUNDEV]"
+					if get_str "  Disc $n: [$LUNDEV]"
 					then
 						LUNDEV=$ANS
 					fi
@@ -2265,23 +2401,18 @@ do
 			DEVLIST=$XMCD_DEV
 
 			YNDEF=y
-			if getyn "  Does your OS platforms support this?"
+			if get_yn "  Does your OS platforms support this?"
 			then
 			    while :
 			    do
 				LUNDEV=/dev/changer
-				if getstr \
+				if get_str \
 				"\n  Enter the medium changer device: [$LUNDEV]"
 				then
 					LUNDEV=$ANS
 				fi
 
-				if [ $BLKDEV = 0 -a ! -c $LUNDEV ]
-				then
-				    $ECHO "  $LUNDEV is invalid.  Try again."
-				    continue
-				fi
-				if [ $BLKDEV = 1 -a ! -b $LUNDEV ]
+				if [ ! -c $LUNDEV ]
 				then
 				    $ECHO "  $LUNDEV is invalid.  Try again."
 				    continue
@@ -2408,9 +2539,9 @@ do
 		;;
 	esac
 
-	$ECHO "\n  Creating the $CONFIG file..."
-
+	$ECHO "\n  Creating the $CFGDIR/$CONFIGFILE file..."
 	GDEVLIST=`echo $DEVLIST | sed 's/\//\\\\\//g'`
+
 	sed \
 	-e "s/^!.*DO NOT MODIFY.*$/! DEVICE CONFIGURATION FILE/" \
 	-e "s/^logicalDriveNumber:.*/logicalDriveNumber:	$DRVNO/" \
@@ -2432,6 +2563,8 @@ do
 	-e "s/^volumeControlTaper:.*/volumeControlTaper:	$VOLTAPER/" \
 	-e "s/^scsiAudioVolumeBase:.*/scsiAudioVolumeBase:	$VOLBASE/" \
 	-e "s/^pauseResumeSupport:.*/pauseResumeSupport:	$PAUSE/" \
+	-e "s/^strictPauseResume:.*/strictPauseResume:	$STRICTPAUSERESUME/" \
+	-e "s/^playPausePlay:.*/playPausePlay:		$PLAYPAUSEPLAY/" \
 	-e "s/^caddyLockSupport:.*/caddyLockSupport:	$CADDYLOCK/" \
 	-e "s/^curposFormat:.*/curposFormat:		$CURPOSFMT/" \
 	-e "s/^noTURWhenPlaying:.*/noTURWhenPlaying:	$PLAYNOTUR/" \
@@ -2441,13 +2574,16 @@ do
 	-e "s/^exitOnEject:.*/exitOnEject:		$EXITONEJECT/" \
 	-e "s/^closeOnEject:.*/closeOnEject:		$CLOSEONEJECT/" \
 	-e "s/^multiPlay:.*/multiPlay:		$MULTIPLAY/" \
-	   < $CFGDIR/device.cfg > $CONFIG
-	chmod 644 $CONFIG 2>/dev/null
-	chown bin $CONFIG 2>/dev/null
-	chgrp bin $CONFIG 2>/dev/null
+	   < $CFGDIR/device.cfg > $CFGDIR/$CONFIGFILE
+	do_chmod 644 $CFGDIR/$CONFIGFILE 2>/dev/null
+	do_chown $OWNER $CFGDIR/$CONFIGFILE 2>/dev/null
+	do_chgrp $GROUP $CFGDIR/$CONFIGFILE 2>/dev/null
+	(cd $CFGDIR ;\
+	 rm -f ${CONFIGFILE}-${OS_NODE} ;\
+	 do_link $CONFIGFILE ${CONFIGFILE}-${OS_NODE})
 
 	YNDEF=n
-	if getyn "\n  Do you have more CD-ROM or CD-R drives on your system"
+	if get_yn "\n  Do you have more CD-ROM or CD-R drives on your system"
 	then
 		DRVNO=`expr $DRVNO + 1`
 
@@ -2478,5 +2614,5 @@ do
 	fi
 done
 
-doexit $EXITSTAT
+do_exit $EXITSTAT
 
