@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -29,7 +29,7 @@
  */
 
 #ifndef LINT
-static char *_fbioc_c_ident_ = "@(#)fbioc.c	6.94 98/05/18";
+static char *_fbioc_c_ident_ = "@(#)fbioc.c	6.119 00/01/31";
 #endif
 
 #include "common_d/appenv.h"
@@ -44,7 +44,8 @@ extern FILE		*errfp;
 extern di_client_t	*di_clinfo;
 
 
-STATIC bool_t	fbioc_run_ab(curstat_t *),
+STATIC bool_t	fbioc_pause_resume(bool_t),
+		fbioc_run_ab(curstat_t *),
 		fbioc_run_sample(curstat_t *),
 		fbioc_run_prog(curstat_t *),
 		fbioc_run_repeat(curstat_t *),
@@ -72,7 +73,9 @@ STATIC bool_t	fbioc_not_open = TRUE,		/* Device not opened yet */
 		fbioc_idx_pause,		/* Prev/next index pausing */
 		fbioc_fake_stop,		/* Force a completion status */
 		fbioc_playing,			/* Currently playing */
-		fbioc_bcd_hack;			/* Track numbers in BCD hack */
+		fbioc_paused,			/* Currently paused */
+		fbioc_bcd_hack,			/* Track numbers in BCD hack */
+		fbioc_override_ap;		/* Override auto-play */
 STATIC word32_t	fbioc_ab_start_addr,		/* A->B mode start block */
 		fbioc_ab_end_addr,		/* A->B mode end block */
 		fbioc_sav_end_addr;		/* Err recov saved end addr */
@@ -258,12 +261,12 @@ fbioc_close(void)
  *	FALSE - failure
  */
 STATIC bool_t
-fbioc_rdsubq(struct cd_sub_channel_info *sub, byte_t msf)
+fbioc_rdsubq(struct cd_sub_channel_info *sub, bool_t msf)
 {
 	struct ioc_read_subchannel	subchnl;
 	bool_t				ret;
 
-	subchnl.address_format = msf;
+	subchnl.address_format = msf ? CD_MSF_FORMAT : CD_LBA_FORMAT;
 	subchnl.data_format = CD_CURRENT_POSITION;
 	subchnl.track = 0;
 	subchnl.data_len = sizeof(struct cd_sub_channel_info);
@@ -292,13 +295,15 @@ fbioc_rdsubq(struct cd_sub_channel_info *sub, byte_t msf)
  *	h - address of pointer to array of toc entrys will be allocated
  *	    by this routine
  *	start - Starting track number for which the TOC data is returned
+ *	msf - Whether to use MSF or logical block address data format
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
 STATIC bool_t
-fbioc_rdtoc(struct ioc_toc_header *h, struct cd_toc_entry **e, int start)
+fbioc_rdtoc(struct ioc_toc_header *h, struct cd_toc_entry **e,
+	    int start, bool_t msf)
 {
 	int				i,
 					j,
@@ -322,8 +327,14 @@ fbioc_rdtoc(struct ioc_toc_header *h, struct cd_toc_entry **e, int start)
 
 	allocsz = (h->ending_track - start + 2) * sizeof(struct cd_toc_entry);
 
-	*e = (struct cd_toc_entry *)(void *) MEM_ALLOC(allocsz);
-	t = (struct cd_toc_entry *)(void *) MEM_ALLOC(allocsz);
+	*e = (struct cd_toc_entry *)(void *) MEM_ALLOC(
+		"cd_toc_entry",
+		allocsz
+	);
+	t = (struct cd_toc_entry *)(void *) MEM_ALLOC(
+		"cd_toc_entry",
+		allocsz
+	);
 
 	if (*e == NULL || t == NULL) {
 		DI_FATAL(app_data.str_nomemory);
@@ -333,7 +344,7 @@ fbioc_rdtoc(struct ioc_toc_header *h, struct cd_toc_entry **e, int start)
 	(void) memset((byte_t *) *e, 0, allocsz);
 	(void) memset((byte_t *) t, 0, allocsz);
 	tocentry.starting_track = start;
-	tocentry.address_format = CD_MSF_FORMAT;
+	tocentry.address_format = msf ? CD_MSF_FORMAT : CD_LBA_FORMAT;
 	tocentry.data_len = allocsz;
 	tocentry.data = t;
 
@@ -527,7 +538,7 @@ fbioc_disc_present(bool_t savstat)
 	}
 
 	allocsz = (tot_trks + 1) * sizeof(struct cd_toc_entry);
-	e = (struct cd_toc_entry *)(void *) MEM_ALLOC(allocsz);
+	e = (struct cd_toc_entry *)(void *) MEM_ALLOC("cd_toc_entry", allocsz);
 	if (e == NULL) {
 		DI_FATAL(app_data.str_nomemory);
 		return FALSE;
@@ -667,6 +678,9 @@ fbioc_start_stop(bool_t start, bool_t loej)
 {
 	bool_t	ret;
 
+	if (app_data.strict_pause_resume && fbioc_paused)
+		(void) fbioc_pause_resume(TRUE);
+
 	if (start) {
 		if (loej)
 			ret = FALSE;
@@ -687,8 +701,19 @@ fbioc_start_stop(bool_t start, bool_t loej)
 	 * don't handle new commands well when the CD is loading/ejecting
 	 * with the IMMED bit set in the Start/Stop Unit command.
 	 */
-	if (ret && loej)
-		(void) sleep((app_data.ins_interval + 1000 - 1) / 1000);
+	if (ret) {
+		if (loej) {
+			int	n;
+
+			n = (app_data.ins_interval + 1000 - 1) / 1000;
+			if (start)
+				n *= 2;
+
+			(void) sleep(n);
+		}
+		else if (start && app_data.spinup_interval > 0)
+			(void) sleep(app_data.spinup_interval);
+	}
 
 	return (ret);
 
@@ -706,10 +731,19 @@ fbioc_start_stop(bool_t start, bool_t loej)
  *	TRUE - success
  *	FALSE - failure
  */
-bool_t
+STATIC bool_t
 fbioc_pause_resume(bool_t resume)
 {
-	return (fbioc_send(resume ? CDIOCRESUME : CDIOCPAUSE, NULL, TRUE));
+	bool_t	ret;
+
+	if (!app_data.pause_supp)
+		return FALSE;
+
+	ret = fbioc_send(resume ? CDIOCRESUME : CDIOCPAUSE, NULL, TRUE);
+	if (ret)
+		fbioc_paused = !resume;
+
+	return (ret);
 }
 
 
@@ -794,12 +828,22 @@ fbioc_do_playaudio(
 	do_playti = (addr_fmt & ADDR_TRKIDX) && app_data.playti_supp;
 
 	if (do_playmsf || do_playti) {
-		if (fbioc_playing)
-			/* Pause playback first */
-			(void) fbioc_pause_resume(FALSE);
-		else
+		if (fbioc_paused) {
+			if (app_data.strict_pause_resume) {
+				/* Resume first */
+				(void) fbioc_pause_resume(TRUE);
+			}
+		}
+		else if (fbioc_playing) {
+			if (app_data.play_pause_play) {
+				/* Pause first */
+				(void) fbioc_pause_resume(FALSE);
+			}
+		}
+		else {
 			/* Spin up CD */
 			(void) fbioc_start_stop(TRUE, FALSE);
+		}
 	}
 
 	if (do_playmsf)
@@ -808,8 +852,10 @@ fbioc_do_playaudio(
 	if (!ret && do_playti)
 		ret = fbioc_play_trkidx(trk, idx, trk, idx);
 
-	if (ret) 
+	if (ret) {
 		fbioc_playing = TRUE;
+		fbioc_paused = FALSE;
+	}
 
 	return (ret);
 }
@@ -913,7 +959,7 @@ fbioc_get_playstatus(curstat_t *s)
 	in_fbioc_get_playstatus = TRUE;
 
 	(void) memset((byte_t *) &sub, 0, sizeof(struct cd_sub_channel_info));
-	if (!fbioc_rdsubq(&sub, CD_MSF_FORMAT)) {
+	if (!fbioc_rdsubq(&sub, !app_data.subq_lba)) {
 		/* Check to see if the disc had been manually ejected */
 		if (!fbioc_disc_ready(s)) {
 			fbioc_sav_end_addr = 0;
@@ -940,39 +986,84 @@ fbioc_get_playstatus(curstat_t *s)
 
 	/* Check the subchannel data */
 
+	if (app_data.subq_lba) {
+		/* LBA mode */
 #ifdef NETBSD_OLDIOC
-	s->cur_tot_min = sub.what.position.absaddr[1];
-	s->cur_tot_sec = sub.what.position.absaddr[2];
-	s->cur_tot_frame = sub.what.position.absaddr[3];
+		s->cur_tot_addr = di_xlate_blk(
+			(sub.what.position.absaddr[3] << 24) |
+			(sub.what.position.absaddr[2] << 16) |
+			(sub.what.position.absaddr[1] << 8) |
+			sub.what.position.absaddr[0]
+		);
 #else
-	s->cur_tot_min = sub.what.position.absaddr.msf.minute;
-	s->cur_tot_sec = sub.what.position.absaddr.msf.second;
-	s->cur_tot_frame = sub.what.position.absaddr.msf.frame;
+		s->cur_tot_addr = di_xlate_blk(
+			util_bswap32(sub.what.position.absaddr.lba)
+		);
 #endif
-	util_msftoblk(
-		s->cur_tot_min,
-		s->cur_tot_sec,
-		s->cur_tot_frame,
-		&s->cur_tot_addr,
-		MSF_OFFSET
-	);
+		util_blktomsf(
+			s->cur_tot_addr,
+			&s->cur_tot_min,
+			&s->cur_tot_sec,
+			&s->cur_tot_frame,
+			MSF_OFFSET
+		);
 
 #ifdef NETBSD_OLDIOC
-	s->cur_trk_min = sub.what.position.reladdr[1];
-	s->cur_trk_sec = sub.what.position.reladdr[2];
-	s->cur_trk_frame = sub.what.position.reladdr[3];
+		s->cur_trk_addr = di_xlate_blk(
+			(sub.what.position.reladdr[3] << 24) |
+			(sub.what.position.reladdr[2] << 16) |
+			(sub.what.position.reladdr[1] << 8) |
+			sub.what.position.reladdr[0]
+		);
 #else
-	s->cur_trk_min = sub.what.position.reladdr.msf.minute;
-	s->cur_trk_sec = sub.what.position.reladdr.msf.second;
-	s->cur_trk_frame = sub.what.position.reladdr.msf.frame;
+		s->cur_trk_addr = di_xlate_blk(
+			util_bswap32(sub.what.position.reladdr.lba)
+		);
 #endif
-	util_msftoblk(
-		s->cur_trk_min,
-		s->cur_trk_sec,
-		s->cur_trk_frame,
-		&s->cur_trk_addr,
-		0
-	);
+		util_blktomsf(
+			s->cur_trk_addr,
+			&s->cur_trk_min,
+			&s->cur_trk_sec,
+			&s->cur_trk_frame,
+			0
+		);
+	}
+	else {
+		/* MSF mode */
+#ifdef NETBSD_OLDIOC
+		s->cur_tot_min = sub.what.position.absaddr[1];
+		s->cur_tot_sec = sub.what.position.absaddr[2];
+		s->cur_tot_frame = sub.what.position.absaddr[3];
+#else
+		s->cur_tot_min = sub.what.position.absaddr.msf.minute;
+		s->cur_tot_sec = sub.what.position.absaddr.msf.second;
+		s->cur_tot_frame = sub.what.position.absaddr.msf.frame;
+#endif
+		util_msftoblk(
+			s->cur_tot_min,
+			s->cur_tot_sec,
+			s->cur_tot_frame,
+			&s->cur_tot_addr,
+			MSF_OFFSET
+		);
+
+#ifdef NETBSD_OLDIOC
+		s->cur_trk_min = sub.what.position.reladdr[1];
+		s->cur_trk_sec = sub.what.position.reladdr[2];
+		s->cur_trk_frame = sub.what.position.reladdr[3];
+#else
+		s->cur_trk_min = sub.what.position.reladdr.msf.minute;
+		s->cur_trk_sec = sub.what.position.reladdr.msf.second;
+		s->cur_trk_frame = sub.what.position.reladdr.msf.frame;
+#endif
+		util_msftoblk(
+			s->cur_trk_min,
+			s->cur_trk_sec,
+			s->cur_trk_frame,
+			&s->cur_trk_addr,
+			0
+		);
+	}
 
 	/* Update time display */
 	DPY_TIME(s, FALSE);
@@ -1071,6 +1162,7 @@ fbioc_get_playstatus(curstat_t *s)
 	case CD_AS_PLAY_COMPLETED:
 	case CD_AS_NO_STATUS:
 	case CD_AS_AUDIO_INVALID:
+	default:
 		done = TRUE;
 
 		if (!fbioc_fake_stop)
@@ -1102,11 +1194,6 @@ fbioc_get_playstatus(curstat_t *s)
 		}
 
 		break;
-
-	default:
-		/* Something is wrong with the data. */
-		done = FALSE;
-		break;
 	}
 
 	if (done) {
@@ -1130,6 +1217,10 @@ fbioc_get_playstatus(curstat_t *s)
 			/* Spin down the disc */
 			fbioc_start_stop(FALSE, FALSE);
 		}
+		if (app_data.done_exit) {
+			/* Exit */
+			di_clinfo->quit(s);
+		}
 
 		in_fbioc_get_playstatus = FALSE;
 		return FALSE;
@@ -1158,6 +1249,9 @@ fbioc_cfg_vol(int vol, curstat_t *s, bool_t query)
 	struct ioc_vol	volctrl;
 	static bool_t	first = TRUE;
 
+	if (!app_data.mselvol_supp)
+		return 0;
+
 	(void) memset((byte_t *) &volctrl, 0, sizeof(struct ioc_vol));
 
 	if (query) {
@@ -1286,7 +1380,7 @@ fbioc_get_toc(curstat_t *s)
 				ntrks;
 
 	(void) memset((byte_t *) &h, 0, sizeof(struct ioc_toc_header));
-	if (!fbioc_rdtoc(&h, &e, 0))
+	if (!fbioc_rdtoc(&h, &e, 0, !app_data.toc_lba))
 		return FALSE;
 
 	/* Fill curstat structure with TOC data */
@@ -1297,23 +1391,48 @@ fbioc_get_toc(curstat_t *s)
 
 	for (i = 0; i <= ntrks; i++) {
 		s->trkinfo[i].trkno = p->track;
-		s->trkinfo[i].type = TYP_AUDIO;	/* shrug */
+		s->trkinfo[i].type =
+			(p->control & 0x04) ? TYP_DATA : TYP_AUDIO;
+
+		if (app_data.toc_lba) {
+			/* LBA mode */
 #ifdef NETBSD_OLDIOC
-		s->trkinfo[i].min = p->addr[1];
-		s->trkinfo[i].sec = p->addr[2];
-		s->trkinfo[i].frame = p->addr[3];
+			s->trkinfo[i].addr = di_xlate_blk(
+				(p->addr[3] << 24) | (p->addr[2] << 16) |
+				(p->addr[2] << 8)  | p->addr[0]
+			);
 #else
-		s->trkinfo[i].min = p->addr.msf.minute;
-		s->trkinfo[i].sec = p->addr.msf.second;
-		s->trkinfo[i].frame = p->addr.msf.frame;
+			s->trkinfo[i].addr = di_xlate_blk(
+				util_bswap32(p->addr.lba)
+			);
 #endif
-		util_msftoblk(
-			s->trkinfo[i].min,
-			s->trkinfo[i].sec,
-			s->trkinfo[i].frame,
-			&s->trkinfo[i].addr,
-			MSF_OFFSET
-		);
+			util_blktomsf(
+				s->trkinfo[i].addr,
+				&s->trkinfo[i].min,
+				&s->trkinfo[i].sec,
+				&s->trkinfo[i].frame,
+				MSF_OFFSET
+			);
+		}
+		else {
+			/* MSF mode */
+#ifdef NETBSD_OLDIOC
+			s->trkinfo[i].min = p->addr[1];
+			s->trkinfo[i].sec = p->addr[2];
+			s->trkinfo[i].frame = p->addr[3];
+#else
+			s->trkinfo[i].min = p->addr.msf.minute;
+			s->trkinfo[i].sec = p->addr.msf.second;
+			s->trkinfo[i].frame = p->addr.msf.frame;
+#endif
+			util_msftoblk(
+				s->trkinfo[i].min,
+				s->trkinfo[i].sec,
+				s->trkinfo[i].frame,
+				&s->trkinfo[i].addr,
+				MSF_OFFSET
+			);
+		}
 
 		if (p->track == LEAD_OUT_TRACK || i == (MAXTRACK - 1)) {
 			s->tot_min = s->trkinfo[i].min;
@@ -1399,6 +1518,9 @@ fbioc_stop_stat_poll(void)
 STATIC void
 fbioc_start_insert_poll(curstat_t *s)
 {
+	int	delay;
+	bool_t	first = TRUE;
+
 	if (fbioc_insert_polling || app_data.ins_disable ||
 	    (s->mode != MOD_BUSY && s->mode != MOD_NODISC))
 		return;
@@ -1412,10 +1534,17 @@ fbioc_start_insert_poll(curstat_t *s)
 	if (fbioc_ins_interval < 500)
 		fbioc_ins_interval = 500;
 
+	if (first) {
+		first = FALSE;
+		delay = 50;
+	}
+	else
+		delay = fbioc_ins_interval;
+
 	/* Start poll timer */
 	if (di_clinfo->timeout != NULL) {
 		fbioc_insert_id = di_clinfo->timeout(
-			fbioc_ins_interval,
+			delay,
 			fbioc_insert_poll,
 			(byte_t *) s
 		);
@@ -1626,7 +1755,8 @@ fbioc_disc_ready(curstat_t *s)
 
 	if (app_data.load_play) {
 		/* Start auto-play */
-		fbioc_play_pause(s);
+		if (!fbioc_override_ap)
+			fbioc_play_pause(s);
 	}
 	else if (app_data.load_spindown) {
 		/* Spin down disc in case the user isn't going to
@@ -1958,8 +2088,9 @@ fbioc_run_sample(curstat_t *s)
 			MSF_OFFSET
 		);
 
-		if (fbioc_do_playaudio(ADDR_BLK | ADDR_MSF,
-				 saddr, eaddr, &smsf, &emsf, 0, 0)) {
+		if (s->trkinfo[fbioc_next_sam].type != TYP_AUDIO ||
+		    fbioc_do_playaudio(ADDR_BLK | ADDR_MSF,
+				       saddr, eaddr, &smsf, &emsf, 0, 0)) {
 			fbioc_next_sam++;
 			return TRUE;
 		}
@@ -2070,7 +2201,7 @@ fbioc_run_prog(curstat_t *s)
 	);
 
 	if (s->mode == MOD_PAUSE) {
-		fbioc_pause_resume(FALSE);
+		(void) fbioc_pause_resume(FALSE);
 
 		/* Restore volume */
 		fbioc_mute_off(s);
@@ -2136,7 +2267,7 @@ fbioc_run_repeat(curstat_t *s)
 		);
 
 		if (s->mode == MOD_PAUSE) {
-			fbioc_pause_resume(FALSE);
+			(void) fbioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			fbioc_mute_off(s);
@@ -2388,12 +2519,25 @@ fbioc_shuffle(curstat_t *s, bool_t enable)
 void
 fbioc_load_eject(curstat_t *s)
 {
-	if (!fbioc_disc_ready(s)) {
+	int	i;
+	bool_t	ret;
+
+	if (!fbioc_disc_ready(s) && app_data.load_supp) {
 		/* Disc not ready: try loading the disc */
 		if (!fbioc_start_stop(TRUE, TRUE))
 			DO_BEEP();
 
-		return;
+		/* Wait for CD to become ready */
+		for (i = 0; i < 3; i++) {
+			if ((ret = fbioc_disc_ready(s)) == TRUE)
+				break;
+			sleep(1);
+		}
+
+		if (ret || !app_data.eject_supp)
+			return;
+
+		/* Fall through and open tray again */
 	}
 
 	/* Eject the disc */
@@ -2431,7 +2575,7 @@ fbioc_load_eject(curstat_t *s)
 	DBCLEAR(s, FALSE);
 	DPY_ALL(s);
 
-	fbioc_start_stop(FALSE, TRUE);
+	(void) fbioc_start_stop(FALSE, TRUE);
 
 	if (app_data.eject_exit)
 		di_clinfo->quit(s);
@@ -2645,11 +2789,16 @@ fbioc_play_pause(curstat_t *s)
 	msf_t		start_msf,
 			end_msf;
 
+	fbioc_override_ap = TRUE;
+
 	if (!fbioc_disc_ready(s)) {
+		fbioc_override_ap = FALSE;
 		DO_BEEP();
 		return;
 	}
 
+	fbioc_override_ap = FALSE;
+
 	if (s->mode == MOD_NODISC)
 		s->mode = MOD_STOP;
 
@@ -2827,10 +2976,8 @@ fbioc_stop(curstat_t *s, bool_t stop_disc)
 	 * immediately go into play state again.  Not spinning down
 	 * the drive makes things a little faster...
 	 */
-	if (!fbioc_disc_ready(s)) {
-		DO_BEEP();
+	if (!fbioc_disc_ready(s))
 		return;
-	}
 
 	switch (s->mode) {
 	case MOD_PLAY:
@@ -2855,7 +3002,6 @@ fbioc_stop(curstat_t *s, bool_t stop_disc)
 		break;
 
 	default:
-		DO_BEEP();
 		break;
 	}
 }
@@ -3038,7 +3184,7 @@ fbioc_prevtrk(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				fbioc_pause_resume(FALSE);
+				(void) fbioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				fbioc_mute_off(s);
@@ -3186,7 +3332,7 @@ fbioc_nexttrk(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				fbioc_pause_resume(FALSE);
+				(void) fbioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				fbioc_mute_off(s);
@@ -3288,6 +3434,9 @@ fbioc_previdx(curstat_t *s)
 			return;
 		}
 
+		/* A small delay to make sure the command took effect */
+		util_delayms(10);
+
 		fbioc_idx_pause = TRUE;
 
 		if (!fbioc_pause_resume(FALSE)) {
@@ -3330,7 +3479,7 @@ fbioc_previdx(curstat_t *s)
 		fbioc_idx_pause = FALSE;
 
 		if (s->mode == MOD_PAUSE) {
-			fbioc_pause_resume(FALSE);
+			(void) fbioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			fbioc_mute_off(s);
@@ -3405,6 +3554,9 @@ fbioc_nextidx(curstat_t *s)
 			return;
 		}
 
+		/* A small delay to make sure the command took effect */
+		util_delayms(10);
+
 		fbioc_idx_pause = TRUE;
 
 		if (!fbioc_pause_resume(FALSE)) {
@@ -3447,7 +3599,7 @@ fbioc_nextidx(curstat_t *s)
 		fbioc_idx_pause = FALSE;
 
 		if (s->mode == MOD_PAUSE) {
-			fbioc_pause_resume(FALSE);
+			(void) fbioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			fbioc_mute_off(s);
@@ -3578,7 +3730,7 @@ fbioc_rew(curstat_t *s, bool_t start)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				fbioc_pause_resume(FALSE);
+				(void) fbioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				fbioc_mute_off(s);
@@ -3708,7 +3860,7 @@ fbioc_ff(curstat_t *s, bool_t start)
 				}
 			}
 			if (s->mode == MOD_PAUSE) {
-				fbioc_pause_resume(FALSE);
+				(void) fbioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				fbioc_mute_off(s);
@@ -3800,7 +3952,7 @@ fbioc_warp(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				fbioc_pause_resume(FALSE);
+				(void) fbioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				fbioc_mute_off(s);
@@ -3862,7 +4014,10 @@ void
 fbioc_start(curstat_t *s)
 {
 	/* Check to see if disc is ready */
-	(void) fbioc_disc_ready(s);
+	if (di_clinfo->timeout != NULL)
+		fbioc_start_insert_poll(s);
+	else
+		(void) fbioc_disc_ready(s);
 
 	/* Update display */
 	DPY_ALL(s);
