@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef LINT
-static char *_libdi_c_ident_ = "@(#)libdi.c	6.102 98/05/15";
+static char *_libdi_c_ident_ = "@(#)libdi.c	6.144 00/01/20";
 #endif
 
 #include "common_d/appenv.h"
@@ -58,8 +58,9 @@ di_client_t		*di_clinfo;
 /* Device list table */
 char			**di_devlist;
 
-/* Web search engines list */
-srcheng_ent_t		*srcheng_list_head;
+/* Web sites and search engines lists */
+STATIC urltmpl_ent_t	*sites_list_head;
+STATIC urltmpl_ent_t	*srcheng_list_head;
 
 /* Device lock file */
 STATIC char		lockfile[FILE_PATH_SZ];
@@ -68,6 +69,10 @@ STATIC char		lockfile[FILE_PATH_SZ];
 STATIC di_devreg_t	*di_reghead;
 
 
+/***********************
+ *  internal routines  *
+ ***********************/
+
 
 /*
  * di_boolstr
@@ -116,6 +121,8 @@ di_prncfg(void)
 		       app_data.version == NULL ? "(undef)" : app_data.version);
 	(void) fprintf(errfp, "\tmainWindowMode:\t\t\t%d\n",
 		       app_data.main_mode);
+	(void) fprintf(errfp, "\tmodeChangeGravity:\t\t%d\n",
+		       app_data.modechg_grav);
 	(void) fprintf(errfp, "\tnormalMainWidth:\t\t%d\n",
 		       app_data.normal_width);
 	(void) fprintf(errfp, "\tnormalMainHeight:\t\t%d\n",
@@ -130,6 +137,8 @@ di_prncfg(void)
 		       app_data.blinkoff_interval);
 	(void) fprintf(errfp, "\tmainShowFocus:\t\t\t%s\n",
 		       di_boolstr(app_data.main_showfocus));
+	(void) fprintf(errfp, "\tinstallColormap:\t\t%s\n",
+		       di_boolstr(app_data.instcmap));
 
 	(void) fprintf(errfp, "\nCommon parameters:\n");
 
@@ -157,13 +166,24 @@ di_prncfg(void)
 	(void) fprintf(errfp, "\twebSite3URL:\t\t\t%s\n",
 		       app_data.site3_url == NULL ?
 		           "(undef)" : app_data.site3_url);
+	(void) fprintf(errfp, "\tdiscographyURL:\t\t\t%s\n",
+		       app_data.discog_url == NULL ?
+		           "(undef)" : app_data.discog_url);
 	(void) fprintf(errfp, "\texcludeWords:\t\t\t%s\n",
 		       app_data.exclude_words == NULL ?
 		           "(undef)" : app_data.exclude_words);
+	(void) fprintf(errfp, "\thistoryFileMode:\t\t%s\n",
+		       app_data.hist_filemode);
+	(void) fprintf(errfp, "\thistoryFileDisable:\t\t%s\n",
+		       di_boolstr(app_data.histfile_dsbl));
+	(void) fprintf(errfp, "\tmaximumHistory:\t\t\t%d\n",
+		       app_data.cddb_maxhist);
 	(void) fprintf(errfp, "\tcddbDefaultPort:\t\t%d\n",
 		       app_data.cddb_port);
 	(void) fprintf(errfp, "\tcddbTimeOut:\t\t\t%d\n",
 		       app_data.cddb_timeout);
+	(void) fprintf(errfp, "\tcddbRemoteAutoSave:\t\t%d\n",
+		       app_data.cddb_rmtautosave);
 	(void) fprintf(errfp, "\tstatusPollInterval:\t\t%d\n",
 		       app_data.stat_interval);
 	(void) fprintf(errfp, "\tinsertPollInterval:\t\t%d\n",
@@ -192,8 +212,8 @@ di_prncfg(void)
 		       app_data.sample_blks);
 	(void) fprintf(errfp, "\tstartupVolume:\t\t\t%d\n",
 		       app_data.startup_vol);
-	(void) fprintf(errfp, "\tcddbRemoteAutoSave:\t\t%s\n",
-		       di_boolstr(app_data.cddb_rmtautosave));
+	(void) fprintf(errfp, "\tcddbRemoteDisable:\t\t%s\n",
+		       di_boolstr(app_data.cddb_rmtdsbl));
 	(void) fprintf(errfp, "\tcddbUseHttpProxy:\t\t%s\n",
 		       di_boolstr(app_data.http_proxy));
 	(void) fprintf(errfp, "\tshowScsiErrMsg:\t\t\t%s\n",
@@ -202,6 +222,10 @@ di_prncfg(void)
 		       di_boolstr(app_data.sol2_volmgt));
 	(void) fprintf(errfp, "\ttooltipEnable:\t\t\t%s\n",
 		       di_boolstr(app_data.tooltip_enable));
+	(void) fprintf(errfp, "\tremoteControlEnable:\t\t%s\n",
+		       di_boolstr(app_data.remote_enb));
+	(void) fprintf(errfp, "\tremoteControlLog:\t\t%s\n",
+		       di_boolstr(app_data.remote_log));
 	(void) fprintf(errfp, "\tdebug:\t\t\t\t%s\n",
 		       di_boolstr(app_data.debug));
 
@@ -245,12 +269,24 @@ di_prncfg(void)
 		       di_boolstr(app_data.chroute_supp));
 	(void) fprintf(errfp, "\tpauseResumeSupport:\t\t%s\n",
 		       di_boolstr(app_data.pause_supp));
+	(void) fprintf(errfp, "\tstrictPauseResume:\t\t%s\n",
+		       di_boolstr(app_data.strict_pause_resume));
+	(void) fprintf(errfp, "\tplayPausePlay:\t\t\t%s\n",
+		       di_boolstr(app_data.play_pause_play));
 	(void) fprintf(errfp, "\tcaddyLockSupport:\t\t%s\n",
 		       di_boolstr(app_data.caddylock_supp));
 	(void) fprintf(errfp, "\tcurposFormat:\t\t\t%s\n",
 		       di_boolstr(app_data.curpos_fmt));
 	(void) fprintf(errfp, "\tnoTURWhenPlaying:\t\t%s\n",
 		       di_boolstr(app_data.play_notur));
+	(void) fprintf(errfp, "\ttocLBA:\t\t\t\t%s\n",
+		       di_boolstr(app_data.toc_lba));
+	(void) fprintf(errfp, "\tsubChannelLBA:\t\t\t%s\n",
+		       di_boolstr(app_data.subq_lba));
+	(void) fprintf(errfp, "\tdriveBlockSize:\t\t\t%d\n",
+		       app_data.drv_blksz);
+	(void) fprintf(errfp, "\tspinUpInterval:\t\t\t%d\n",
+		       app_data.spinup_interval);
 
 	(void) fprintf(errfp,
 		       "\nDevice-specific (user-modifiable) parameters:\n");
@@ -265,6 +301,8 @@ di_prncfg(void)
 		       di_boolstr(app_data.load_play));
 	(void) fprintf(errfp, "\tejectOnDone:\t\t\t%s\n",
 		       di_boolstr(app_data.done_eject));
+	(void) fprintf(errfp, "\texitOnExit:\t\t\t%s\n",
+		       di_boolstr(app_data.done_exit));
 	(void) fprintf(errfp, "\tejectOnExit:\t\t\t%s\n",
 		       di_boolstr(app_data.exit_eject));
 	(void) fprintf(errfp, "\tstopOnExit:\t\t\t%s\n",
@@ -325,7 +363,7 @@ di_parse_devlist(void)
 		/* SCSI LUN addressing method */
 		listsz = n * sizeof(char *);
 
-		di_devlist = (char **) MEM_ALLOC(listsz);
+		di_devlist = (char **) MEM_ALLOC("di_devlist", listsz);
 		if (di_devlist == NULL) {
 			DI_FATAL(app_data.str_nomemory);
 			return;
@@ -349,7 +387,10 @@ di_parse_devlist(void)
 			if (q != NULL)
 				*q = '\0';
 
-			di_devlist[i] = (char *) MEM_ALLOC(strlen(p) + 1);
+			di_devlist[i] = (char *) MEM_ALLOC(
+				"di_devlist[i]",
+				strlen(p) + 1
+			);
 			if (di_devlist[i] == NULL) {
 				DI_FATAL(app_data.str_nomemory);
 				return;
@@ -385,12 +426,13 @@ di_parse_devlist(void)
 			app_data.devlist = app_data.device;
 		}
 
-		di_devlist = (char **) MEM_ALLOC(sizeof(char *));
+		di_devlist = (char **) MEM_ALLOC("di_devlist", sizeof(char *));
 		if (di_devlist == NULL) {
 			DI_FATAL(app_data.str_nomemory);
 			return;
 		}
 		di_devlist[0] = (char *) MEM_ALLOC(
+			"di_devlist[0]",
 			strlen(app_data.devlist) + 1
 		);
 		if (di_devlist[0] == NULL) {
@@ -407,6 +449,324 @@ di_parse_devlist(void)
 }
 
 
+/*
+ * di_scan_url_attrib
+ *	Scan a URL template string and record its attributes.
+ *
+ * Args:
+ *	url - The URL template string
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+di_scan_url_attrib(char *url, url_attrib_t *up)
+{
+	char	*cp;
+
+	/* Scan URL string to get a count of the number of
+	 * substitutions needed.
+	 */
+	up->xcnt = up->vcnt = 0;
+	up->ncnt = up->hcnt = 0;
+	up->lcnt = up->ccnt = 0;
+	up->icnt = up->acnt = 0;
+	up->dcnt = up->tcnt = 0;
+	for (cp = url; *cp != '\0'; cp++) {
+		if (*cp == '%') {
+			switch ((int) *(++cp)) {
+			case 'X':
+				up->xcnt++;
+				break;
+			case 'V':
+				up->vcnt++;
+				break;
+			case 'N':
+				up->ncnt++;
+				break;
+			case 'H':
+				up->hcnt++;
+				break;
+			case 'L':
+				up->lcnt++;
+				break;
+			case 'C':
+				up->ccnt++;
+				break;
+			case 'I':
+				up->icnt++;
+				break;
+			case 'A':
+			case 'a':
+				up->acnt++;
+				break;
+			case 'D':
+			case 'd':
+				up->dcnt++;
+				break;
+			case 'T':
+			case 't':
+				up->tcnt++;
+				break;
+			case 'B':
+			case 'b':
+				up->acnt++;
+				up->dcnt++;
+				break;
+			default:
+				break;
+			}
+		}
+	}
+}
+
+
+/*
+ * di_add_site_ent
+ *	Add an entry to the web sites information list.
+ *
+ * Args:
+ *	def - The web site definition string from common.cfg
+ *	name - The name of this entry
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+di_add_site_ent(char *def, char *name, int siteno)
+{
+	urltmpl_ent_t		*p;
+	bool_t			newent;
+	static urltmpl_ent_t	*lastp;
+
+	newent = FALSE;
+
+	/* Find an existing entry that matches the name */
+	for (p = sites_list_head; p != NULL; p = p->next) {
+		if (p->idx == siteno)
+			break;
+	}
+
+	if (p == NULL) {
+		newent = TRUE;
+
+		/* Allocate a new list entry */
+		p = (urltmpl_ent_t *)(void *) MEM_ALLOC(
+			"urltmpl_ent_t",
+			sizeof(urltmpl_ent_t)
+		);
+		if (p == NULL) {
+			DI_FATAL(app_data.str_nomemory);
+			return;
+		}
+		p->name = NULL;
+		p->action = NULL;
+		p->fallback = NULL;
+		p->aux = NULL;
+	}
+
+	if (p->action != NULL)
+		MEM_FREE(p->action);
+	if (p->fallback != NULL)
+		MEM_FREE(p->fallback);
+
+	p->idx = siteno;
+	p->action = def;
+	p->name = name;
+	di_scan_url_attrib(def, &p->a_attrib);
+
+	if (newent) {
+		/* Add new entry to list */
+		p->next = NULL;
+		if (sites_list_head == NULL)
+			sites_list_head = lastp = p;
+		else {
+			lastp->next = p;
+			lastp = p;
+		}
+	}
+
+	if (app_data.debug) {
+		(void) fprintf(errfp, "%s site %s: %s\n",
+			newent ? "Add" : "Upd",
+			p->name,
+			p->action);
+	}
+}
+
+
+/*
+ * di_add_srcheng_ent
+ *	Add an entry to the search engines information list.
+ *
+ * Args:
+ *	def - The search engine definition string from common.cfg
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+di_add_srcheng_ent(char *def)
+{
+	char			*cp1,
+				*cp2,
+				*name,
+				*action,
+				*fallback;
+	urltmpl_ent_t		*p;
+	bool_t			line_ent,
+				new_ent;
+	char			savch;
+	static urltmpl_ent_t	*lastp;
+
+	if ((cp1 = strchr(def, ':')) == NULL) {
+		/* Bad definition */
+		return;
+	}
+
+	name = def;
+	action = NULL;
+	*cp1 = '\0';
+	p = NULL;
+
+	line_ent = (bool_t) (strcmp(name, "-") == 0);
+	if (!line_ent) {
+		cp2 = cp1 + 1;
+
+		/* Skip white space */
+		while (*cp2 == ' ' || *cp2 == '\t')
+			cp2++;
+
+		if (*cp2 == '\0' || *cp2 == '\n') {
+			/* Bad definition */
+			*cp1 = ':';
+			return;
+		}
+
+		action = cp2;
+
+		/* Find an existing entry that matches the name */
+		for (p = srcheng_list_head; p != NULL; p = p->next) {
+			if (strcmp(name, p->name) == 0)
+				break;
+		}
+	}
+
+	new_ent = FALSE;
+	if (p == NULL) {
+		/* Allocate a new list entry */
+		p = (urltmpl_ent_t *)(void *) MEM_ALLOC(
+			"urltmpl_ent_t",
+			sizeof(urltmpl_ent_t)
+		);
+		if (p == NULL) {
+			DI_FATAL(app_data.str_nomemory);
+			return;
+		}
+		p->idx = -1;
+		p->action = NULL;
+		p->fallback = NULL;
+		p->aux = NULL;
+		new_ent = TRUE;
+	}
+
+	/* Name */
+	p->name = (char *) MEM_ALLOC("p->name", strlen(name) + 1);
+	if (p->name == NULL) {
+		DI_FATAL(app_data.str_nomemory);
+		return;
+	}
+	(void) strcpy(p->name, name);
+
+	/* Restore character */
+	*cp1 = ':';
+
+	if (!line_ent) {
+		savch = '\0';
+		for (cp2 = action; *cp2 != '\0'; cp2++) {
+			if (*cp2 == ' ' || *cp2 == '\t') {
+				savch = *cp2;
+				*cp2 = '\0';
+				break;
+			}
+		}
+
+		/* Action URL */
+
+		if (p->action != NULL)
+			MEM_FREE(p->action);
+		p->action = (char *) MEM_ALLOC(
+			"p->action", strlen(action) + 1
+		);
+		if (p->action == NULL) {
+			DI_FATAL(app_data.str_nomemory);
+			return;
+		}
+		(void) strcpy(p->action, action);
+
+		di_scan_url_attrib(action, &p->a_attrib);
+
+		if (savch != '\0') {
+			*cp2 = savch;
+
+			/* Skip white space */
+			while (*cp2 == ' ' || *cp2 == '\t')
+				cp2++;
+
+			if (*cp2 != '\0' && *cp2 != '\n') {
+				fallback = cp2;
+
+				/* Fallback URL */
+
+				if (p->fallback != NULL)
+					MEM_FREE(p->fallback);
+				p->fallback = (char *) MEM_ALLOC(
+					"p->fallback", strlen(fallback) + 1
+				);
+				if (p->fallback == NULL) {
+					DI_FATAL(app_data.str_nomemory);
+					return;
+				}
+				(void) strcpy(p->fallback, fallback);
+
+				di_scan_url_attrib(fallback, &p->f_attrib);
+			}
+			else
+				p->fallback = NULL;
+		}
+	}
+
+	if (app_data.debug) {
+		if (line_ent)
+			(void) fprintf(errfp, "Add srch ---\n");
+		else
+			(void) fprintf(errfp, "%s srch %s: %s%s%s%s\n",
+				new_ent ? "Add" : "Upd",
+				(p->name == NULL) ? "-" : p->name,
+				(p->action == NULL) ? "-" : p->action,
+				(p->fallback == NULL) ? "" : " [",
+				(p->fallback == NULL) ? "" : p->fallback,
+				(p->fallback == NULL) ? "" : "]");
+	}
+
+	if (new_ent) {
+		/* Add new entry to list */
+		p->next = NULL;
+		if (srcheng_list_head == NULL)
+			srcheng_list_head = lastp = p;
+		else {
+			lastp->next = p;
+			lastp = p;
+		}
+	}
+}
+
+
+/***********************
+ *   public routines   *
+ ***********************/
+
+
 /*
  * di_init
  *	Top-level function to initialize the libdi modules.
@@ -958,8 +1318,8 @@ di_dump_curstat(curstat_t *s)
 	di_devreg_t	*rp;
 
 	(void) fprintf(errfp,
-		       "\nDumping contents of curstat_t structure at 0x%x:\n",
-		       (int) s);
+		       "\nDumping contents of curstat_t structure at 0x%lx:\n",
+		       (unsigned long) s);
 	(void) fprintf(errfp,
 		       "curdev=%s\nfirst_disc=%d last_disc=%d\n",
 		       s->curdev, s->first_disc, s->last_disc);
@@ -1005,8 +1365,9 @@ di_dump_curstat(curstat_t *s)
 		"onetrk_prog=%d caddy_lk=%d cddb=%d prog_tot=%d prog_cnt=%d\n",
 		s->onetrk_prog, s->caddy_lock, s->cddb,
 		s->prog_tot, s->prog_cnt);
-	(void) fprintf(errfp, "level=%d level_left=%d level_right=%d\n",
-		s->level, s->level_left, s->level_right);
+	(void) fprintf(errfp,
+		"chgrscan=%d level=%d level_left=%d level_right=%d\n",
+		s->chgrscan, s->level, s->level_left, s->level_right);
 	(void) fprintf(errfp,
 		"vendor=\"%-8s\" prod=\"%-16s\" revnum=\"%-4s\"\n",
 		s->vendor, s->prod, s->revnum);
@@ -1019,146 +1380,38 @@ di_dump_curstat(curstat_t *s)
 
 
 /*
- * di_add_srcheng_ent
- *	Add an entry to the search engines information list.
+ * di_sites_list
+ *	Return the head of the web sites information list.
  *
  * Args:
- *	def - The search engine definition string from common.cfg
+ *	None.
  *
  * Return:
- *	Nothing.
+ *	Pointer to the head of the web sites information list,
+ *	or NULL if the list is empty.
  */
-void
-di_add_srcheng_ent(char *def)
+urltmpl_ent_t *
+di_sites_list(void)
 {
-	char			*cp1,
-				*cp2,
-				*cp3,
-				*name,
-				*action;
-	srcheng_ent_t		*p;
-	bool_t			line_ent,
-				new_ent;
-	static srcheng_ent_t	*lastp;
+	return (sites_list_head);
+}
 
-	if ((cp1 = strchr(def, ':')) == NULL) {
-		/* Bad definition */
-		return;
-	}
 
-	name = def;
-	action = NULL;
-	*cp1 = '\0';
-	p = NULL;
-
-	line_ent = (bool_t) (strcmp(name, "-") == 0);
-	if (!line_ent) {
-		cp2 = cp1 + 1;
-
-		/* Skip white space */
-		while (*cp2 == ' ' || *cp2 == '\t')
-			cp2++;
-
-		if (*cp2 == '\0' || *cp2 == '\n') {
-			/* Bad definition */
-			*cp1 = ':';
-			return;
-		}
-
-		action = cp2;
-
-		/* Find an existing entry that matches the name */
-		for (p = srcheng_list_head; p != NULL; p = p->next) {
-			if (strcmp(name, p->name) == 0)
-				break;
-		}
-	}
-
-	new_ent = FALSE;
-	if (p == NULL) {
-		/* Allocate a new list entry */
-		p = (srcheng_ent_t *)(void *) MEM_ALLOC(sizeof(srcheng_ent_t));
-		if (p == NULL) {
-			DI_FATAL(app_data.str_nomemory);
-			return;
-		}
-		p->action = NULL;
-		p->aux = NULL;
-		new_ent = TRUE;
-	}
-
-	if (app_data.debug) {
-		if (line_ent)
-			(void) fprintf(errfp, "Add    ---\n");
-		else
-			(void) fprintf(errfp, "%s %s: %s\n",
-				new_ent ? "Add   " : "Update",
-				name, action);
-	}
-
-	/* Name */
-	p->name = (char *) MEM_ALLOC(strlen(name) + 1);
-	if (p->name == NULL) {
-		DI_FATAL(app_data.str_nomemory);
-		return;
-	}
-	(void) strcpy(p->name, name);
-
-	if (!line_ent) {
-		/* Action URL */
-		if (p->action != NULL)
-			MEM_FREE(p->action);
-		p->action = (char *) MEM_ALLOC(strlen(action) + 1);
-		if (p->action == NULL) {
-			DI_FATAL(app_data.str_nomemory);
-			return;
-		}
-		(void) strcpy(p->action, action);
-
-		/* Scan URL action string to get a count of the number of
-		 * substitutions needed.
-		 */
-		p->acnt = p->dcnt = p->tcnt = 0;
-		for (cp3 = action; *cp3 != '\0'; cp3++) {
-			if (*cp3 == '%') {
-				switch ((int) *(++cp3)) {
-				case 'A':
-				case 'a':
-					p->acnt++;
-					break;
-				case 'D':
-				case 'd':
-					p->dcnt++;
-					break;
-				case 'T':
-				case 't':
-					p->tcnt++;
-					break;
-				case 'B':
-				case 'b':
-					p->acnt++;
-					p->dcnt++;
-					break;
-				default:
-					break;
-				}
-			}
-		}
-	}
-
-	/* Restore character */
-	*cp1 = ':';
-
-	if (new_ent) {
-		/* Add new entry to list */
-		p->next = NULL;
-		if (srcheng_list_head == NULL)
-			srcheng_list_head = lastp = p;
-		else {
-			lastp->next = p;
-			lastp = p;
-		}
-	}
+/*
+ * di_srcheng_list
+ *	Return the head of the search engines information list.
+ *
+ * Args:
+ *	None.
+ *
+ * Return:
+ *	Pointer to the head of the search engines information list,
+ *	or NULL if the list is empty.
+ */
+urltmpl_ent_t *
+di_srcheng_list(void)
+{
+	return (srcheng_list_head);
 }
 
 
@@ -1179,7 +1432,10 @@ di_common_parmload(char *path, bool_t priv)
 	FILE		*fp;
 	char		*buf,
 			*parm,
-			errstr[ERR_BUF_SZ];
+			errstr[ERR_BUF_SZ],
+			trypath[FILE_PATH_SZ];
+	struct utsname	*un;
+	bool_t		notry2;
 	static bool_t	dev_cmdline = FALSE,
 			force_debug = FALSE;
 #ifndef __VMS
@@ -1189,6 +1445,7 @@ di_common_parmload(char *path, bool_t priv)
 			pfd[2];
 	FILE		*wfp;
 
+	un = util_get_uname();
 
 	if (priv && di_isdemo())
 		app_data.device = "(sim1)";
@@ -1210,26 +1467,49 @@ di_common_parmload(char *path, bool_t priv)
 		/* Close un-needed pipe descriptor */
 		(void) close(pfd[0]);
 
-		DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
-			(int) util_get_ouid(), (int) util_get_ogid());
-
 		/* Force uid and gid to original setting */
-		if (setuid(util_get_ouid()) < 0 ||
-		    setgid(util_get_ogid()) < 0) {
+		if (!util_set_ougid()) {
 			(void) close(pfd[1]);
 			exit(1);
 		}
 
 		/* Allocate temporary buffer */
-		if ((buf = (char *) MEM_ALLOC(PARM_BUF_SZ)) == NULL) {
+		if ((buf = (char *) MEM_ALLOC("parmbuf",
+					      PARM_BUF_SZ)) == NULL) {
 			(void) close(pfd[1]);
 			exit(2);
 		}
 
-		DBGPRN(errfp, "Loading common parameters: %s\n", path);
+		notry2 = FALSE;
+		if (((int) strlen(path) + (int) strlen(un->nodename) + 2)
+		    > FILE_PATH_SZ) {
+			DBGPRN(errfp, "NOTICE: %s: %s\n",
+				"Host-specific config files not used",
+				"Name too long");
+			(void) strcpy(trypath, path);
+			notry2 = TRUE;
+		}
+		else
+			/* Try host-specific config file first */
+			(void) sprintf(trypath, "%s-%s", path, un->nodename);
+
+		DBGPRN(errfp, "Loading common parameters: %s\n", trypath);
+		fp = fopen(trypath, "r");
+
+		if (fp == NULL && !notry2) {
+			DBGPRN(errfp, "Cannot open %s\n", trypath);
+
+			/* Try generic config file */
+			(void) strcpy(trypath, path);
+
+			DBGPRN(errfp, "Loading common parameters: %s\n",
+				trypath);
+			fp = fopen(trypath, "r");
+		}
+
+		if (fp == NULL) {
+			DBGPRN(errfp, "Cannot open %s\n", trypath);
 
-		if ((fp = fopen(path, "r")) == NULL) {
-			DBGPRN(errfp, "Cannot open %s\n", path);
 			(void) close(pfd[1]);
 
 			if (priv && !di_isdemo())
@@ -1285,10 +1565,36 @@ di_common_parmload(char *path, bool_t priv)
 		break;
 	}
 #else
-	DBGPRN(errfp, "Loading common parameters: %s\n", path);
+	un = util_get_uname();
 
-	if ((fp = fopen(path, "r")) == NULL) {
-		DBGPRN(errfp, "Cannot open %s\n", path);
+	notry2 = FALSE;
+	if (((int) strlen(path) + (int) strlen(un->nodename) + 2)
+	    > FILE_PATH_SZ) {
+		DBGPRN(errfp, "NOTICE: %s: %s\n",
+			"Host-specific config files not used",
+			"Name too long");
+		(void) strcpy(trypath, path);
+		notry2 = TRUE;
+	}
+	else
+		/* Try host-specific config file first */
+		(void) sprintf(trypath, "%s-%s", path, un->nodename);
+
+	DBGPRN(errfp, "Loading common parameters: %s\n", trypath);
+	fp = fopen(trypath, "r");
+
+	if (fp == NULL && !notry2) {
+		DBGPRN(errfp, "Cannot open %s\n", trypath);
+
+		/* Try generic config file */
+		(void) strcpy(trypath, path);
+
+		DBGPRN(errfp, "Loading common parameters: %s\n", trypath);
+		fp = fopen(trypath, "r");
+	}
+
+	if (fp == NULL) {
+		DBGPRN(errfp, "Cannot open %s\n", trypath);
 		if (priv && !di_isdemo()) {
 			/* Cannot open system config file. */
 			(void) sprintf(errstr, app_data.str_nocfg, path);
@@ -1299,8 +1605,8 @@ di_common_parmload(char *path, bool_t priv)
 #endif	/* __VMS */
 
 	/* Allocate temporary buffer */
-	if ((buf = (char *) MEM_ALLOC(PARM_BUF_SZ)) == NULL ||
-	    (parm = (char *) MEM_ALLOC(PARM_BUF_SZ)) == NULL) {
+	if ((buf = (char *) MEM_ALLOC("buf", PARM_BUF_SZ)) == NULL ||
+	    (parm = (char *) MEM_ALLOC("parmbuf", PARM_BUF_SZ)) == NULL) {
 		DI_FATAL(app_data.str_nomemory);
 		return;
 	}
@@ -1324,6 +1630,7 @@ di_common_parmload(char *path, bool_t priv)
 					MEM_FREE(app_data.device);
 
 				app_data.device = (char *) MEM_ALLOC(
+					"app_data.device",
 					strlen(parm) + 1
 				);
 				if (app_data.device == NULL) {
@@ -1340,6 +1647,7 @@ di_common_parmload(char *path, bool_t priv)
 					MEM_FREE(app_data.cddb_path);
 
 				app_data.cddb_path = (char *) MEM_ALLOC(
+					"app_data.cddb_path",
 					strlen(parm) + 1
 				);
 				if (app_data.cddb_path == NULL) {
@@ -1354,6 +1662,7 @@ di_common_parmload(char *path, bool_t priv)
 			if (app_data.proxy_server != NULL)
 				MEM_FREE(app_data.proxy_server);
 			app_data.proxy_server = (char *) MEM_ALLOC(
+				"app_data.proxy_server",
 				strlen(parm) + 1
 			);
 			if (app_data.proxy_server == NULL) {
@@ -1367,6 +1676,7 @@ di_common_parmload(char *path, bool_t priv)
 			if (app_data.browser_dir != NULL)
 				MEM_FREE(app_data.browser_dir);
 			app_data.browser_dir = (char *) MEM_ALLOC(
+				"app_data.browser_dir",
 				strlen(parm) + 1
 			);
 			if (app_data.browser_dir == NULL) {
@@ -1380,6 +1690,7 @@ di_common_parmload(char *path, bool_t priv)
 			if (app_data.browser_rmt != NULL)
 				MEM_FREE(app_data.browser_rmt);
 			app_data.browser_rmt = (char *) MEM_ALLOC(
+				"app_data.browser_rmt",
 				strlen(parm) + 1
 			);
 			if (app_data.browser_rmt == NULL) {
@@ -1390,9 +1701,9 @@ di_common_parmload(char *path, bool_t priv)
 			continue;
 		}
 		if (sscanf(buf, "webSite1URL: %[^\n]\n", parm) > 0) {
-			if (app_data.site1_url != NULL)
-				MEM_FREE(app_data.site1_url);
+			/* Old allocated memory is freed in di_add_site_ent */
 			app_data.site1_url = (char *) MEM_ALLOC(
+				"app_data.site1_url",
 				strlen(parm) + 1
 			);
 			if (app_data.site1_url == NULL) {
@@ -1400,12 +1711,18 @@ di_common_parmload(char *path, bool_t priv)
 				return;
 			}
 			(void) strcpy(app_data.site1_url, parm);
+
+			di_add_site_ent(
+				app_data.site1_url,
+				"webSite1URL",
+				0
+			);
 			continue;
 		}
 		if (sscanf(buf, "webSite2URL: %[^\n]\n", parm) > 0) {
-			if (app_data.site2_url != NULL)
-				MEM_FREE(app_data.site2_url);
+			/* Old allocated memory is freed in di_add_site_ent */
 			app_data.site2_url = (char *) MEM_ALLOC(
+				"app_data.site2_url",
 				strlen(parm) + 1
 			);
 			if (app_data.site2_url == NULL) {
@@ -1413,12 +1730,18 @@ di_common_parmload(char *path, bool_t priv)
 				return;
 			}
 			(void) strcpy(app_data.site2_url, parm);
+
+			di_add_site_ent(
+				app_data.site2_url,
+				"webSite2URL",
+				1
+			);
 			continue;
 		}
 		if (sscanf(buf, "webSite3URL: %[^\n]\n", parm) > 0) {
-			if (app_data.site3_url != NULL)
-				MEM_FREE(app_data.site3_url);
+			/* Old allocated memory is freed in di_add_site_ent */
 			app_data.site3_url = (char *) MEM_ALLOC(
+				"app_data.site3_url",
 				strlen(parm) + 1
 			);
 			if (app_data.site3_url == NULL) {
@@ -1426,12 +1749,37 @@ di_common_parmload(char *path, bool_t priv)
 				return;
 			}
 			(void) strcpy(app_data.site3_url, parm);
+
+			di_add_site_ent(
+				app_data.site3_url,
+				"webSite3URL",
+				2
+			);
+			continue;
+		}
+		if (sscanf(buf, "discographyURL: %[^\n]\n", parm) > 0) {
+			app_data.discog_url = (char *) MEM_ALLOC(
+				"app_data.discog_url",
+				strlen(parm) + 1
+			);
+			if (app_data.discog_url == NULL) {
+				DI_FATAL(app_data.str_nomemory);
+				return;
+			}
+			(void) strcpy(app_data.discog_url, parm);
+
+			di_add_site_ent(
+				app_data.discog_url,
+				"discographyURL",
+				3
+			);
 			continue;
 		}
 		if (sscanf(buf, "excludeWords: %[^\n]\n", parm) > 0) {
 			if (app_data.exclude_words != NULL)
 				MEM_FREE(app_data.exclude_words);
 			app_data.exclude_words = (char *) MEM_ALLOC(
+				"app_data.exclude_words",
 				strlen(parm) + 1
 			);
 			if (app_data.exclude_words == NULL) {
@@ -1441,6 +1789,10 @@ di_common_parmload(char *path, bool_t priv)
 			(void) strcpy(app_data.exclude_words, parm);
 			continue;
 		}
+		if (sscanf(buf, "maximumHistory: %s\n", parm) > 0) {
+			app_data.cddb_maxhist = atoi(parm);
+			continue;
+		}
 		if (sscanf(buf, "cddbDefaultPort: %s\n", parm) > 0) {
 			app_data.cddb_port = atoi(parm);
 			continue;
@@ -1450,7 +1802,11 @@ di_common_parmload(char *path, bool_t priv)
 			continue;
 		}
 		if (sscanf(buf, "cddbRemoteAutoSave: %s\n", parm) > 0) {
-			app_data.cddb_rmtautosave = util_stob(parm);
+			app_data.cddb_rmtautosave = atoi(parm);
+			continue;
+		}
+		if (sscanf(buf, "cddbRemoteDisable: %s\n", parm) > 0) {
+			app_data.cddb_rmtdsbl = util_stob(parm);
 			continue;
 		}
 		if (sscanf(buf, "cddbUseHttpProxy: %s\n", parm) > 0) {
@@ -1463,6 +1819,7 @@ di_common_parmload(char *path, bool_t priv)
 					MEM_FREE(app_data.cddb_filemode);
 
 				app_data.cddb_filemode = (char *) MEM_ALLOC(
+					"app_data.cddb_filemode",
 					strlen(parm) + 1
 				);
 				if (app_data.cddb_filemode == NULL) {
@@ -1473,6 +1830,23 @@ di_common_parmload(char *path, bool_t priv)
 			}
 			continue;
 		}
+		if (sscanf(buf, "historyFileMode: %s\n", parm) > 0) {
+			if (app_data.hist_filemode == NULL) {
+				if (app_data.hist_filemode != NULL)
+					MEM_FREE(app_data.hist_filemode);
+
+				app_data.hist_filemode = (char *) MEM_ALLOC(
+					"app_data.hist_filemode",
+					strlen(parm) + 1
+				);
+				if (app_data.hist_filemode == NULL) {
+					DI_FATAL(app_data.str_nomemory);
+					return;
+				}
+				(void) strcpy(app_data.hist_filemode, parm);
+			}
+			continue;
+		}
 		if (sscanf(buf, "statusPollInterval: %s\n", parm) > 0) {
 			app_data.stat_interval = atoi(parm);
 			continue;
@@ -1521,6 +1895,18 @@ di_common_parmload(char *path, bool_t priv)
 			app_data.tooltip_time = atoi(parm);
 			continue;
 		}
+		if (sscanf(buf, "historyFileDisable: %s\n", parm) > 0) {
+			app_data.histfile_dsbl = util_stob(parm);
+			continue;
+		}
+		if (sscanf(buf, "remoteControlEnable: %s\n", parm) > 0) {
+			app_data.remote_enb = util_stob(parm);
+			continue;
+		}
+		if (sscanf(buf, "remoteControlLog: %s\n", parm) > 0) {
+			app_data.remote_log = util_stob(parm);
+			continue;
+		}
 		if (sscanf(buf, "debugMode: %s\n", parm) > 0) {
 			if (!force_debug)
 				app_data.debug = util_stob(parm);
@@ -1546,6 +1932,8 @@ di_common_parmload(char *path, bool_t priv)
 		app_data.tooltip_delay = 1000;
 	if (app_data.tooltip_time < 0)
 		app_data.tooltip_delay = 3000;
+	if (app_data.cddb_maxhist < 0)
+		app_data.cddb_maxhist = 0;
 
 #ifndef __VMS
 	while ((ret = WAITPID(cpid, &stat_val, 0)) != cpid) {
@@ -1600,7 +1988,10 @@ di_devspec_parmload(char *path, bool_t priv)
 	FILE		*fp;
 	char		*buf,
 			*parm,
-			errstr[ERR_BUF_SZ];
+			errstr[ERR_BUF_SZ],
+			trypath[FILE_PATH_SZ];
+	struct utsname	*un;
+	bool_t		notry2;
 #ifndef __VMS
 	pid_t		cpid;
 	waitret_t	stat_val;
@@ -1609,11 +2000,16 @@ di_devspec_parmload(char *path, bool_t priv)
 	FILE		*wfp;
 
 
+	un = util_get_uname();
+
 	if (priv && di_isdemo()) {
 		char	*cp;
 
 		cp = "(sim1);(sim2);(sim3);(sim4);(sim5);(sim6);(sim7);(sim8)";
-		app_data.devlist = MEM_ALLOC(strlen(cp) + 1);
+		app_data.devlist = MEM_ALLOC(
+			"app_data.devlist",
+			strlen(cp) + 1
+		);
 		if (app_data.devlist == NULL) {
 			DI_FATAL(app_data.str_nomemory);
 			return;
@@ -1641,26 +2037,50 @@ di_devspec_parmload(char *path, bool_t priv)
 		/* Close un-needed pipe descriptor */
 		(void) close(pfd[0]);
 
-		DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
-			(int) util_get_ouid(), (int) util_get_ogid());
-
 		/* Force uid and gid to original setting */
-		if (setuid(util_get_ouid()) < 0 ||
-		    setgid(util_get_ogid()) < 0) {
+		if (!util_set_ougid()) {
 			(void) close(pfd[1]);
 			exit(1);
 		}
 
 		/* Allocate temporary buffer */
-		if ((buf = (char *) MEM_ALLOC(PARM_BUF_SZ)) == NULL) {
+		if ((buf = (char *) MEM_ALLOC("parmbuf",
+					      PARM_BUF_SZ)) == NULL) {
 			(void) close(pfd[1]);
 			exit(2);
 		}
+		notry2 = FALSE;
+		if (((int) strlen(path) + (int) strlen(un->nodename) + 2)
+		    > FILE_PATH_SZ) {
+			DBGPRN(errfp, "NOTICE: %s: %s\n",
+				"Host-specific config files not used",
+				"Name too long");
+			(void) strcpy(trypath, path);
+			notry2 = TRUE;
+		}
+		else
+			/* Try host-specific config file first */
+			(void) sprintf(trypath, "%s-%s", path, un->nodename);
 
-		DBGPRN(errfp, "Loading device-specific parameters: %s\n", path);
+		DBGPRN(errfp, "Loading device-specific parameters: %s\n",
+			trypath);
 
-		if ((fp = fopen(path, "r")) == NULL) {
-			DBGPRN(errfp, "Cannot open %s\n", path);
+		fp = fopen(trypath, "r");
+
+		if (fp == NULL && !notry2) {
+			DBGPRN(errfp, "Cannot open %s\n", trypath);
+
+			/* Try generic config file */
+			(void) strcpy(trypath, path);
+
+			DBGPRN(errfp,
+				"Loading device-specific parameters: %s\n",
+				trypath);
+			fp = fopen(trypath, "r");
+		}
+
+		if (fp == NULL) {
+			DBGPRN(errfp, "Cannot open %s\n", trypath);
 			(void) close(pfd[1]);
 
 			if (priv && !di_isdemo())
@@ -1716,10 +2136,37 @@ di_devspec_parmload(char *path, bool_t priv)
 		break;
 	}
 #else
-	DBGPRN(errfp, "Loading device-specific parameters: %s\n", path);
+	un = util_get_uname();
 
-	if ((fp = fopen(path, "r")) == NULL) {
-		DBGPRN(errfp, "Cannot open %s\n", path);
+	notry2 = FALSE;
+	if (((int) strlen(path) + (int) strlen(un->nodename) + 2)
+	    > FILE_PATH_SZ) {
+		DBGPRN(errfp, "NOTICE: %s: %s\n",
+			"Host-specific config files not used",
+			"Name too long");
+		(void) strcpy(trypath, path);
+		notry2 = TRUE;
+	}
+	else
+		/* Try host-specific config file first */
+		(void) sprintf(trypath, "%s-%s", path, un->nodename);
+
+	DBGPRN(errfp, "Loading device-specific parameters: %s\n", trypath);
+	fp = fopen(trypath, "r");
+
+	if (fp == NULL && !notry2) {
+		DBGPRN(errfp, "Cannot open %s\n", trypath);
+
+		/* Try generic config file */
+		(void) strcpy(trypath, path);
+
+		DBGPRN(errfp, "Loading device-specific parameters: %s\n",
+			trypath);
+		fp = fopen(trypath, "r");
+	}
+
+	if (fp == NULL) {
+		DBGPRN(errfp, "Cannot open %s\n", trypath);
 		if (priv && !di_isdemo()) {
 			/* Cannot open master device-specific
 			 * config file.
@@ -1732,8 +2179,8 @@ di_devspec_parmload(char *path, bool_t priv)
 #endif	/* __VMS */
 
 	/* Allocate temporary buffer */
-	if ((buf = (char *) MEM_ALLOC(PARM_BUF_SZ)) == NULL ||
-	    (parm = (char *) MEM_ALLOC(PARM_BUF_SZ)) == NULL) {
+	if ((buf = (char *) MEM_ALLOC("buf", PARM_BUF_SZ)) == NULL ||
+	    (parm = (char *) MEM_ALLOC("parmbuf", PARM_BUF_SZ)) == NULL) {
 		DI_FATAL(app_data.str_nomemory);
 		return;
 	}
@@ -1758,6 +2205,7 @@ di_devspec_parmload(char *path, bool_t priv)
 					MEM_FREE(app_data.devlist);
 
 				app_data.devlist = (char *) MEM_ALLOC(
+					"app_data.devlist",
 					strlen(parm) + 1
 				);
 				if (app_data.devlist == NULL) {
@@ -1851,6 +2299,16 @@ di_devspec_parmload(char *path, bool_t priv)
 				app_data.pause_supp = util_stob(parm);
 				continue;
 			}
+			if (sscanf(buf, "strictPauseResume: %s\n",
+				   parm) > 0) {
+				app_data.strict_pause_resume = util_stob(parm);
+				continue;
+			}
+			if (sscanf(buf, "playPausePlay: %s\n",
+				   parm) > 0) {
+				app_data.play_pause_play = util_stob(parm);
+				continue;
+			}
 			if (sscanf(buf, "caddyLockSupport: %s\n",
 				   parm) > 0) {
 				app_data.caddylock_supp = util_stob(parm);
@@ -1866,6 +2324,24 @@ di_devspec_parmload(char *path, bool_t priv)
 				app_data.play_notur = util_stob(parm);
 				continue;
 			}
+			if (sscanf(buf, "tocLBA: %s\n",
+				   parm) > 0) {
+				app_data.toc_lba = util_stob(parm);
+				continue;
+			}
+			if (sscanf(buf, "subChannelLBA: %s\n",
+				   parm) > 0) {
+				app_data.subq_lba = util_stob(parm);
+				continue;
+			}
+			if (sscanf(buf, "driveBlockSize: %s\n", parm) > 0) {
+				app_data.drv_blksz = atoi(parm);
+				continue;
+			}
+			if (sscanf(buf, "spinUpInterval: %s\n", parm) > 0) {
+				app_data.spinup_interval = atoi(parm);
+				continue;
+			}
 		}
 
 		/* These are general parameters that can be
@@ -1915,6 +2391,10 @@ di_devspec_parmload(char *path, bool_t priv)
 			app_data.done_eject = util_stob(parm);
 			continue;
 		}
+		if (sscanf(buf, "exitOnDone: %s\n", parm) > 0) {
+			app_data.done_exit = util_stob(parm);
+			continue;
+		}
 		if (sscanf(buf, "ejectOnExit: %s\n", parm) > 0) {
 			app_data.exit_eject = util_stob(parm);
 			continue;
@@ -2043,6 +2523,9 @@ di_devspec_parmload(char *path, bool_t priv)
 		else if (app_data.startup_vol < -1)
 			app_data.startup_vol = -1;
 
+		if (app_data.drv_blksz == 0 ||
+		    (app_data.drv_blksz % 512) != 0)
+			app_data.drv_blksz = STD_CDROM_BLKSZ;
 	}
 }
 
@@ -2060,13 +2543,14 @@ di_devspec_parmload(char *path, bool_t priv)
 void
 di_devspec_parmsave(char *path)
 {
+	FILE		*fp;
+	char		*truestr = "True",
+			*falsestr = "False";
+#ifndef __VMS
 	int		ret;
 	pid_t		cpid;
 	waitret_t	stat_val;
-	FILE		*fp;
-	char		*truestr = "True",
-			*falsestr = "False",
-			*dirpath,
+	char		*dirpath,
 			errstr[ERR_BUF_SZ];
 
 	/* Fork child to perform actual I/O */
@@ -2108,28 +2592,28 @@ di_devspec_parmsave(char *path)
 		    return;
 	}
 
-	DBGPRN(errfp, "Setting uid to %d, gid to %d\n",
-		(int) util_get_ouid(), (int) util_get_ogid());
-
 	/* Force uid and gid to original setting */
-	if (setuid(util_get_ouid()) < 0 || setgid(util_get_ogid()) < 0)
+	if (!util_set_ougid())
 		exit(1);
 
-#ifndef __VMS
 	dirpath = util_dirname(path);
 	if (!util_mkdir(dirpath, 0755)) {
 		DBGPRN(errfp, "di_devspec_parmsave: cannot mkdir %s.\n",
 			dirpath);
 		exit(2);
 	}
-#endif
+#endif	/* __VMS */
 
 	DBGPRN(errfp, "Writing device-specific file %s\n", path);
 
 	/* Open file for writing */
 	if ((fp = fopen(path, "w")) == NULL) {
 		DBGPRN(errfp, "di_devspec_parmsave: cannot open %s.\n", path);
+#ifdef __VMS
+		return;
+#else
 		exit(2);
+#endif
 	}
 
 	/* Write banner */
@@ -2151,6 +2635,8 @@ di_devspec_parmsave(char *path)
 		       app_data.load_play ? truestr : falsestr);
 	(void) fprintf(fp, "ejectOnDone:\t\t%s\n",
 		       app_data.done_eject ? truestr : falsestr);
+	(void) fprintf(fp, "exitOnDone:\t\t%s\n",
+		       app_data.done_exit ? truestr : falsestr);
 	(void) fprintf(fp, "ejectOnExit:\t\t%s\n",
 		       app_data.exit_eject ? truestr : falsestr);
 	(void) fprintf(fp, "stopOnExit:\t\t%s\n",
@@ -2168,14 +2654,20 @@ di_devspec_parmsave(char *path)
 	(void) fprintf(fp, "reversePlay:\t\t%s\n",
 		       app_data.reverse ? truestr : falsestr);
 
-	if (fclose(fp) != 0)
+	if (fclose(fp) != 0) {
+#ifdef __VMS
+		return;
+#else
 		exit(1);
+#endif
+	}
 
 	(void) chmod(path, 0644);
 
 	/* Child exits here */
+#ifndef __VMS
 	exit(0);
-
+#endif
 	/*NOTREACHED*/
 }
 
@@ -2357,12 +2849,15 @@ di_devreg(int fd, char *path)
 	if (path == NULL || *path == '\0')
 		return FALSE;
 
-	rp = (di_devreg_t *)(void *) MEM_ALLOC(sizeof(di_devreg_t));
+	rp = (di_devreg_t *)(void *) MEM_ALLOC(
+		"di_devreg_t",
+		sizeof(di_devreg_t)
+	);
 	if (rp == NULL) {
 		DI_FATAL(app_data.str_nomemory);
 		return FALSE;
 	}
-	rp->path = (char *) MEM_ALLOC(strlen(path) + 1);
+	rp->path = (char *) MEM_ALLOC("rp->path", strlen(path) + 1);
 	if (rp == NULL) {
 		DI_FATAL(app_data.str_nomemory);
 		return FALSE;
@@ -2565,7 +3060,7 @@ di_curprog_pos(curstat_t *s)
  *	s - Pointer to the curstat_t structure.
  *	clear_toc - Whether the trkinfo CD table-of-contents 
  *		should be cleared.
- *	eject - Whether the medium is being ejected (clear cur_disc)
+ *	eject - Whether the medium is being ejected
  *
  * Return:
  *	Nothing.
@@ -2650,29 +3145,38 @@ di_reset_shuffle(curstat_t *s)
 {
 	sword32_t	i,
 			j,
-			n;
+			n,
+			x;
 
 	srand((unsigned) time(NULL));
 	s->prog_cnt = 0;
-	s->prog_tot = s->tot_trks;
 
-	for (i = 0; i < MAXTRACK; i++) {
-		if (i >= (int) s->prog_tot) {
-			s->playorder[i] = -1;
-			continue;
-		}
-
-		do {
-			n = rand() % (int) s->prog_tot;
-			for (j = 0; j < i; j++) {
-				if (n == s->playorder[j])
-					break;
-			}
-		} while (j < i);
-
-		s->playorder[i] = n;
+	/* Initialize the play-list with only audio tracks */
+	n = 0;
+	for (i = 0; i < (int) s->tot_trks; i++) {
+		if (s->trkinfo[i].type == TYP_AUDIO)
+			s->playorder[n++] = i;
 	}
 
+	s->prog_tot = (byte_t) n;
+
+	/* Initialize rest of list */
+	for (; n < MAXTRACK; n++)
+		s->playorder[n] = -1;
+
+	/*
+	 * Now shuffle the playlist by swapping random pairs:
+	 * this is the most efficient alogorithm for a linear
+	 * distribution.
+	 */
+	for (i = (int) s->prog_tot - 1; i > 0; i--) {
+		j = rand() % (i + 1);
+		if (j != i) {
+			x = s->playorder[i];
+			s->playorder[i] = s->playorder[j];
+			s->playorder[j] = x;
+		}
+	}
 
 	if (app_data.debug) {
 		(void) fprintf(errfp, "\nShuffle tracks: ");
@@ -2790,3 +3294,53 @@ di_unscale_vol(int v)
 }
 
 
+/*
+ * di_xlate_blk
+ *	Translate the CD-ROM drive's native LBA/length into an
+ *	LBA/length that is based on 2048-byte blocks.  This is
+ *	needed to support drives that are configured to a block
+ *	size other than 2048 (such as Sun or SGI).
+ *
+ * Args:
+ *	addr - The LBA address from the drive
+ *
+ * Return:
+ *	The converted LBA.
+ */
+word32_t
+di_xlate_blk(word32_t addr)
+{
+	if (app_data.drv_blksz == STD_CDROM_BLKSZ)
+		return (addr);
+	else if (app_data.drv_blksz < STD_CDROM_BLKSZ)
+		return (addr / (STD_CDROM_BLKSZ / app_data.drv_blksz));
+	else
+		return (addr * (app_data.drv_blksz / STD_CDROM_BLKSZ));
+}
+
+
+/*
+ * di_unxlate_blk
+ *	Translate an LBA/length that is based on 2048-byte blocks
+ *	into the CD-ROM drive's native LBA/length.  This is needed
+ *	to support drives that are configured to a block size other
+ *	than 2048 (such as Sun or SGI).
+ *
+ * Args:
+ *	addr - The LBA address from the drive
+ *
+ * Return:
+ *	The converted LBA
+ */
+word32_t
+di_unxlate_blk(word32_t addr)
+{
+	if (app_data.drv_blksz == STD_CDROM_BLKSZ)
+		return (addr);
+	else if (app_data.drv_blksz < STD_CDROM_BLKSZ)
+		return (addr * (STD_CDROM_BLKSZ / app_data.drv_blksz));
+	else
+		return (addr / (app_data.drv_blksz / STD_CDROM_BLKSZ));
+}
+
+
