@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -22,7 +22,7 @@
 #define __LIBDI_H__
 
 #ifndef LINT
-static char *_libdi_h_ident_ = "@(#)libdi.h	6.38 98/05/03";
+static char *_libdi_h_ident_ = "@(#)libdi.h	6.43 00/01/20";
 #endif
 
 /* Max number of libdi modules */
@@ -104,6 +104,10 @@ static char *_libdi_h_ident_ = "@(#)libdi.h	6.38 98/05/03";
 	if (di_clinfo != NULL && di_clinfo->progclear != NULL)		\
 		di_clinfo->progclear(s);				\
 }
+#define STOPSCAN(s)		{					\
+	if (di_clinfo != NULL && di_clinfo->chgr_scan_stop != NULL)	\
+		di_clinfo->chgr_scan_stop(s);				\
+}
 
 
 /* Message dialog macros */
@@ -249,6 +253,7 @@ typedef struct {
 	void		(*dbclear)(curstat_t *, bool_t);
 	void		(*dbget)(curstat_t *);
 	void		(*progclear)(curstat_t *);
+	void		(*chgr_scan_stop)(curstat_t *);
 	void		(*fatal_msg)(char *, char *);
 	void		(*warning_msg)(char *, char *);
 	void		(*info_msg)(char *, char *);
@@ -307,6 +312,8 @@ extern void	di_start(curstat_t *);
 extern void	di_icon(curstat_t *, bool_t);
 extern void	di_halt(curstat_t *);
 extern void	di_dump_curstat(curstat_t *);
+extern urltmpl_ent_t *di_sites_list(void);
+extern urltmpl_ent_t *di_srcheng_list(void);
 extern void	di_common_parmload(char *, bool_t);
 extern void	di_devspec_parmload(char *, bool_t);
 extern void	di_devspec_parmsave(char *);
@@ -326,6 +333,8 @@ extern int	di_taper_vol(int);
 extern int	di_untaper_vol(int);
 extern int	di_scale_vol(int);
 extern int	di_unscale_vol(int);
+extern word32_t	di_xlate_blk(word32_t);
+extern word32_t	di_unxlate_blk(word32_t);
 
 
 #endif	/* __LIBDI_H__ */
