@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -33,7 +33,7 @@
  *   operating system.
  */
 #ifndef LINT
-static char *_os_bsdi_c_ident_ = "@(#)os_bsdi.c	6.31 98/05/10";
+static char *_os_bsdi_c_ident_ = "@(#)os_bsdi.c	6.36 00/01/24";
 #endif
 
 #include "common_d/appenv.h"
@@ -166,11 +166,11 @@ pthru_send(
 	}
 
 	(void) memset(&suc, 0, sizeof(scsi_user_cdb_t));
-
-	memcpy(suc.suc_cdb, cmdpt, cmdlen);
+	(void) memcpy(suc.suc_cdb, cmdpt, cmdlen);
 	suc.suc_cdblen = cmdlen;
 	suc.suc_datalen = datalen;
 	suc.suc_data = (caddr_t) datapt;
+	suc.suc_timeout = (u_int) (tmout ? tmout : DFLT_CMD_TIMEOUT);
 
 	switch (rw) {
 	case OP_NODATA:
@@ -227,7 +227,9 @@ pthru_open(char *path)
 	int		fd;
 	struct stat	stbuf;
 	char		errstr[ERR_BUF_SZ];
+#ifdef OS_BSDI_2
 	int		val;
+#endif
 
 	/* Check for validity of device node */
 	if (stat(path, &stbuf) < 0) {
