@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef LINT
-static char *_os_cdsim_c_ident_ = "@(#)os_cdsim.c	6.16 98/05/10";
+static char *_os_cdsim_c_ident_ = "@(#)os_cdsim.c	6.19 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -904,7 +904,7 @@ cdsim_main(void)
 
 	(void) fprintf(errfp,
 		       "CD-ROM simulator version %s (pid=%d) starting...\n",
-		       CDSIM_VERS, getpid());
+		       CDSIM_VERS, (int) getpid());
 
 	/* Install signal handlers */
 	signal(SIGINT, cdsim_sig);
