@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -20,7 +20,7 @@
  */
 
 /*
- *   Digital UNIX (OSF/1) and Ultrix support
+ *   OSF1, Digital UNIX, Compaq Tru64 UNIX and Ultrix support
  *
  *   Contributing author: Matt Thomas
  *   E-Mail: thomas@lkg.dec.com
@@ -34,7 +34,7 @@
  *   purposes only.
  */
 #ifndef LINT
-static char *_os_dec_c_ident_ = "@(#)os_dec.c	6.33 98/05/10";
+static char *_os_dec_c_ident_ = "@(#)os_dec.c	6.36 00/02/02";
 #endif
 
 #include "common_d/appenv.h"
@@ -250,16 +250,33 @@ pthru_open(char *path)
 
 		if (ioctl(fd, DEVIOCGET, &devget) >= 0) {
 #ifdef __osf__
-			lun = devget.slave_num % 8;
-			devget.slave_num /= 8;
+			/* OSF1, Digital UNIX, Tru64 UNIX */
+			device_info_t   devi;
+
+			if (ioctl(fd, DEVGETINFO, &devi) >= 0) {
+				bus = devi.v1.businfo.bus.scsi.bus_num;
+				target = devi.v1.businfo.bus.scsi.tgt_id;
+				lun = devi.v1.businfo.bus.scsi.lun;
+			}
+			else {
+				DBGPRN(errfp,
+					"DEVGETINFO failed: errno=%d: %s\n",
+					errno, "Using DEVIOCGET data.");
+				bus = ((devget.slave_num & 0700) >> 6);
+				target = ((devget.slave_num & 0070) >> 3);
+				lun = (devget.slave_num & 0007);
+			}
 #else
+			/* Ultrix */
+			bus = ((devget.slave_num & 0070) >> 3);
+			target = (devget.slave_num & 0007);
 			lun = 0;
 #endif
-			target = devget.slave_num % 8;
-			devget.slave_num /= 8;
-			bus = devget.slave_num % 8;
 			(void) close(fd);
 
+			DBGPRN(errfp, "Device bus,target,lun=%d,%d,%d\n",
+				bus, target, lun);
+
 			if ((fd = open(DEV_CAM, O_RDWR, 0)) >= 0 ||
 			    (fd = open(DEV_CAM, O_RDONLY, 0)) >= 0) {
 
