@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -22,13 +22,26 @@
 /*
  *   FreeBSD/NetBSD/OpenBSD support
  *
- *   Contributing author: Gennady B. Sorokopud
+ *   Contributing author: Gennady B. Sorokopud (SCIOCCOMMAND version)
  *   E-Mail: gena@netvision.net.il
  *
+ *   Contributing author: Kenneth D. Merry (CAM version)
+ *   E-Mail: ken@kdm.org
+ *
  *   This software fragment contains code that interfaces the
  *   application to the FreeBSD (version 2.0.5 or later), NetBSD
  *   (version 1.0A or later) and OpenBSD (2.x and later) operating
  *   systems.
+ *
+ *   SCIOCCOMMAND notes:
+ *   The SCIOCCOMMAND code is used for FreeBSD 2.x, NetBSD and OpenBSD.
+ *   Generic SCSI-support is required in the kernel configuration file.
+ *   Also be sure "SCIOCCOMMAND" in the file "/usr/include/sys/scsiio.h"
+ *   is not just a dummy.
+ *
+ *   CAM notes:
+ *   The CAM code is only used for FreeBSD 3.x and later.  It is enabled
+ *   when compiled with -DFREEBSD_CAM.
  */
 #ifndef __OS_FNBSD_H__
 #define __OS_FNBSD_H__
@@ -37,11 +50,17 @@
     defined(DI_SCSIPT) && !defined(DEMO_ONLY)
 
 #ifndef LINT
-static char *_os_fnbsd_h_ident_ = "@(#)os_fnbsd.h	6.14 98/01/02";
+static char *_os_fnbsd_h_ident_ = "@(#)os_fnbsd.h	6.17 99/12/27";
 #endif
 
+#ifdef FREEBSD_CAM
+#include <cam/cam.h>
+#include <cam/cam_ccb.h>
+#include <cam/scsi/scsi_message.h>
+#include <camlib.h>
+#else
 #include <sys/scsiio.h>
-
+#endif
 
 #define OS_MODULE	/* Indicate that this is compiled on a supported OS */
 #define SETUID_ROOT	/* Setuid root privilege is required */
