@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -32,7 +32,7 @@
  *   only.
  */
 #ifndef LINT
-static char *_os_hpux_c_ident_ = "@(#)os_hpux.c	6.33 98/05/10";
+static char *_os_hpux_c_ident_ = "@(#)os_hpux.c	6.36 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -47,7 +47,9 @@ extern bool_t		scsipt_notrom_error;
 extern FILE		*errfp;
 extern di_client_t	*di_clinfo;
 
+#ifdef HPUX_EXCLUSIVE_OPEN
 STATIC struct utsname	utsn;
+#endif
 
 
 /*
@@ -273,6 +275,7 @@ pthru_open(char *path)
 		return -1;
 	}
 
+#ifdef HPUX_EXCLUSIVE_OPEN
 	/* Find out the machine type */
 	if (uname(&utsn) < 0) {
 		DBGPRN(errfp, "uname(2) failed (errno=%d): assume 9000/7xx\n",
@@ -287,6 +290,7 @@ pthru_open(char *path)
 		(void) close(fd);
 		return -1;
 	}
+#endif	/* HPUX_EXCLUSIVE_OPEN */
 #endif	/* __hp9000s300 */
 
 	(void) di_devreg(fd, path);
@@ -318,7 +322,7 @@ pthru_open(char *path)
 void
 pthru_close(int fd)
 {
-#ifndef __hp9000s300
+#if !defined(__hp9000s300) && defined(HPUX_EXCLUSIVE_OPEN)
 	/* Relinquish exclusive open (on Series 700 systems only) */
 	if (utsn.machine[5] == '7')
 		(void) ioctl(fd, SIOC_EXCLUSIVE, 0);
