@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
  *   application to the Linux operating system.
  */
 #ifndef LINT
-static char *_os_linux_c_ident_ = "@(#)os_linux.c	6.29 98/05/10";
+static char *_os_linux_c_ident_ = "@(#)os_linux.c	6.35 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -86,8 +86,9 @@ pthru_send(
 
 	path = di_devgetpath(fd);
 
-	/* Linux hack: use SCSI_IOCTL_TEST_UNIT_READY instead of
-	 * SCSI_IOCTL_SEND_COMMAND for test unit ready commands.
+#ifdef BUGGY_TEST_UNIT_READY
+	/* Old Linux kernel hack: use SCSI_IOCTL_TEST_UNIT_READY instead
+	 * of SCSI_IOCTL_SEND_COMMAND for test unit ready commands.
 	 */
 	if (cmdpt[0] == OP_S_TEST) {
 		DBGPRN(errfp, "\nSending SCSI_IOCTL_TEST_UNIT_READY (%s)\n",
@@ -105,13 +106,14 @@ pthru_send(
 		}
 		return FALSE;
 	}
+#endif
 
 	/* Set up SCSI pass-through command/data buffer */
 	ptbufsz = (sizeof(int) << 1) + cmdlen + sizeof(req_sense_data_t);
 	if (datapt != NULL && datalen > 0)
 		ptbufsz += datalen;
 
-	if ((ptbuf = (byte_t *) MEM_ALLOC(ptbufsz)) == NULL) {
+	if ((ptbuf = (byte_t *) MEM_ALLOC("ptbuf", ptbufsz)) == NULL) {
 		DI_FATAL(app_data.str_nomemory);
 		return FALSE;
 	}
@@ -184,7 +186,8 @@ pthru_send(
 int
 pthru_open(char *path)
 {
-	int		fd;
+	int		fd,
+			mode;
 	struct stat	stbuf;
 	char		errstr[ERR_BUF_SZ];
 
@@ -195,14 +198,20 @@ pthru_open(char *path)
 		return -1;
 	}
 
-	/* Linux CD-ROM device is a block special file! */
-	if (!S_ISBLK(stbuf.st_mode)) {
+	/* Linux CD-ROM device is a block special file, but the SCSI generic
+	 * device (for medium changer support) is a character special file.
+	 */
+	if (S_ISCHR(stbuf.st_mode))
+		mode = O_RDWR | O_EXCL | O_NONBLOCK;
+	else if (S_ISBLK(stbuf.st_mode))
+		mode = O_RDONLY | O_EXCL | O_NONBLOCK;
+	else {
 		(void) sprintf(errstr, app_data.str_noderr, path);
 		DI_FATAL(errstr);
 		return -1;
 	}
 
-	if ((fd = open(path, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
+	if ((fd = open(path, mode)) < 0) {
 		DBGPRN(errfp, "Cannot open %s: errno=%d\n", path, errno);
 		return -1;
 	}
