@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
  *   later systems to enable the auto-request sense feature.
  */
 #ifndef LINT
-static char *_os_sun_c_ident_ = "@(#)os_sun.c	6.32 98/05/10";
+static char *_os_sun_c_ident_ = "@(#)os_sun.c	6.38 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -48,6 +48,10 @@ extern di_client_t	*di_clinfo;
 
 STATIC req_sense_data_t	sense_data;	/* Request sense data buffer */
 
+#ifdef SOL2_VOLMGT
+STATIC void		sol2_vold_alrm(int);
+#endif
+
 
 /*
  * pthru_send
@@ -124,13 +128,33 @@ pthru_send(
 		util_dbgdump(title, cmdpt, cmdlen);
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) signal(SIGALRM, sol2_vold_alrm);
+		(void) alarm(tmout + 5);
+	}
+#endif
+
 	/* Send the command down via the "pass-through" interface */
 	if (ioctl(fd, USCSICMD, &ucmd) < 0) {
 		if (app_data.scsierr_msg && prnerr)
 			perror("USCSICMD ioctl failed");
+#ifdef SOL2_VOLMGT
+		if (app_data.sol2_volmgt) {
+			(void) alarm(0);
+			(void) signal(SIGALRM, SIG_DFL);
+		}
+#endif
 		return FALSE;
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+	}
+#endif
+
 	if (ucmd.uscsi_status != USCSI_STATUS_GOOD) {
 		if (app_data.scsierr_msg && prnerr) {
 			(void) fprintf(errfp,
@@ -205,24 +229,61 @@ pthru_open(char *path)
 	/* Check for validity of device node */
 	if (stat(path, &stbuf) < 0) {
 #ifdef SOL2_VOLMGT
-		if (app_data.sol2_volmgt && errno == ENOENT)
-			return -1;
+		if (app_data.sol2_volmgt) {
+			switch (errno) {
+			case ENOENT:
+			case EINTR:
+			case ESTALE:
+				return -1;
+				/*NOTREACHED*/
+			default:
+				break;
+			}
+		}
 #endif
 		(void) sprintf(errstr, app_data.str_staterr, path);
 		DI_FATAL(errstr);
 		return -1;
 	}
 	if (!S_ISCHR(stbuf.st_mode)) {
+#ifdef SOL2_VOLMGT
+		/* Some CDs have multiple slices (partitions),
+		 * so the device node becomes a directory when
+		 * vold mounts each slice.
+		 */
+		if (app_data.sol2_volmgt && S_ISDIR(stbuf.st_mode))
+			return -1;
+#endif
 		(void) sprintf(errstr, app_data.str_noderr, path);
 		DI_FATAL(errstr);
 		return -1;
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) signal(SIGALRM, sol2_vold_alrm);
+		(void) alarm(10);
+	}
+#endif
+
 	if ((fd = open(path, O_RDONLY)) < 0) {
 		DBGPRN(errfp, "Cannot open %s: errno=%d\n", path, errno);
+#ifdef SOL2_VOLMGT
+		if (app_data.sol2_volmgt) {
+			(void) alarm(0);
+			(void) signal(SIGALRM, SIG_DFL);
+		}
+#endif
 		return -1;
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+	}
+#endif
+
 	(void) di_devreg(fd, path);
 	return (fd);
 }
@@ -299,6 +360,24 @@ sol2_volmgt_eject(int fd)
 	return TRUE;
 }
 
+
+/*
+ * sol2_vold_alrm
+ *	SIGALRM handler
+ *
+ * Args:
+ *	signo - The signal number
+ *
+ * Return:
+ *	Nothing.
+ */
+/*ARGSUSED*/
+STATIC void
+sol2_vold_alrm(int signo)
+{
+	DBGPRN(errfp, "\nSIGALRM!\n");
+}
+
 #endif	/* SVR4 */
 
 
