@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -19,7 +19,7 @@
  *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef LINT
-static char *_os_svr4_c_ident_ = "@(#)os_svr4.c	6.35 98/05/11";
+static char *_os_svr4_c_ident_ = "@(#)os_svr4.c	6.38 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -385,7 +385,7 @@ pthru_send(
 	/* Align on a page boundary */
 	tmpbuf = NULL;
 	if (sp->xfer_len > 0) {
-		tmpbuf = (char *) MEM_ALLOC(2 * NBPP);
+		tmpbuf = (char *) MEM_ALLOC("xfer_buf", 2 * NBPP);
 		sp->data = tmpbuf;
 		sp->data += NBPP - ((unsigned int) sp->data & (NBPP - 1));
 	}
