@@ -1,7 +1,7 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
  *   purposes only.
  */
 #ifndef LINT
-static char *_os_vms_c_ident_ = "@(#)os_vms.c	6.28 98/05/10";
+static char *_os_vms_c_ident_ = "@(#)os_vms.c	6.31 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -45,11 +45,8 @@ extern appdata_t	app_data;
 extern bool_t		scsipt_notrom_error;
 extern FILE		*errfp;
 
-globalvalue
-	IO$_DIAGNOSE;
-
-STATIC int	context;	
-STATIC char	*str;
+STATIC int		context;	
+STATIC char		*str;
 
 
 /*
