@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -24,7 +24,7 @@
 #ifdef DI_SCSIPT
 
 #ifndef LINT
-static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.30 98/02/27";
+static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.35 00/01/18";
 #endif
 
 
@@ -79,6 +79,7 @@ static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.30 98/02/27";
 #define OP_S_MSENSE		0x1a	/* mode sense (6) */
 #define OP_S_START		0x1b	/* start/stop unit */
 #define OP_S_PREVENT		0x1e	/* prevent/allow medium removal */
+#define OP_S_INITELEMSTAT	0x07	/* initialize element status */
 
 /* 10-byte commands */
 #define OP_M_RDCAP		0x25	/* read capacity */
@@ -95,6 +96,7 @@ static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.30 98/02/27";
 #define OP_L_PLAY		0xa5	/* play audio (12) */
 #define OP_L_PLAYTR		0xa9	/* play audio track relative (12) */
 #define OP_L_MOVEMED		0xa5	/* move medium */
+#define OP_L_READELEMSTAT	0xb8	/* read element status */
 
 
 /* Data buffer lengths */
@@ -107,6 +109,8 @@ static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.30 98/02/27";
 #define SZ_DEVCAPAB		20	/* device capab header+page size */
 #define SZ_ELEMADDR		24	/* element addr header+page size */
 #define SZ_RSENSE		18	/* max request sense data size */
+#define SZ_ELEMHDR		8	/* med chgr element status header */
+#define SZ_ELEMSTAT		8	/* med chgr element status */
 
 
 /* CD-ROM Mode sense/mode select page codes */
@@ -270,50 +274,51 @@ typedef struct dev_capab {
 	unsigned int	stor_ie:1;	/* import/export element */
 	unsigned int	stor_dt:1;	/* data transfer element */
 	unsigned int	res1:4;		/* reserved */
+	unsigned int	res2:8;		/* reserved */
 
 	unsigned int	move_mt_mt:1;	/* move medium: MT->MT */
 	unsigned int	move_mt_st:1;	/* move medium: MT->ST */
 	unsigned int	move_mt_ie:1;	/* move medium: MT->IE */
 	unsigned int	move_mt_dt:1;	/* move medium: MT->DT */
-	unsigned int	res2:4;		/* reserved */
+	unsigned int	res3:4;		/* reserved */
 	unsigned int	move_st_mt:1;	/* move medium: ST->MT */
 	unsigned int	move_st_st:1;	/* move medium: ST->ST */
 	unsigned int	move_st_ie:1;	/* move medium: ST->IE */
 	unsigned int	move_st_dt:1;	/* move medium: ST->DT */
-	unsigned int	res3:4;		/* reserved */
+	unsigned int	res4:4;		/* reserved */
 	unsigned int	move_ie_mt:1;	/* move medium: IE->MT */
 	unsigned int	move_ie_st:1;	/* move medium: IE->ST */
 	unsigned int	move_ie_ie:1;	/* move medium: IE->IE */
 	unsigned int	move_ie_dt:1;	/* move medium: IE->DT */
-	unsigned int	res4:4;		/* reserved */
+	unsigned int	res5:4;		/* reserved */
 	unsigned int	move_dt_mt:1;	/* move medium: DT->MT */
 	unsigned int	move_dt_st:1;	/* move medium: DT->ST */
 	unsigned int	move_dt_ie:1;	/* move medium: DT->IE */
 	unsigned int	move_dt_dt:1;	/* move medium: DT->DT */
-	unsigned int	res5:4;		/* reserved */
+	unsigned int	res6:4;		/* reserved */
 
-	unsigned int	res6:32;	/* reserved */
+	unsigned int	res7:32;	/* reserved */
 
 	unsigned int	xchg_mt_mt:1;	/* exchange medium: MT<->MT */
 	unsigned int	xchg_mt_st:1;	/* exchange medium: MT<->ST */
 	unsigned int	xchg_mt_ie:1;	/* exchange medium: MT<->IE */
 	unsigned int	xchg_mt_dt:1;	/* exchange medium: MT<->DT */
-	unsigned int	res7:4;		/* reserved */
+	unsigned int	res8:4;		/* reserved */
 	unsigned int	xchg_st_mt:1;	/* exchange medium: ST<->MT */
 	unsigned int	xchg_st_st:1;	/* exchange medium: ST<->ST */
 	unsigned int	xchg_st_ie:1;	/* exchange medium: ST<->IE */
 	unsigned int	xchg_st_dt:1;	/* exchange medium: ST<->DT */
-	unsigned int	res8:4;		/* reserved */
+	unsigned int	res9:4;		/* reserved */
 	unsigned int	xchg_ie_mt:1;	/* exchange medium: IE<->MT */
 	unsigned int	xchg_ie_st:1;	/* exchange medium: IE<->ST */
 	unsigned int	xchg_ie_ie:1;	/* exchange medium: IE<->IE */
 	unsigned int	xchg_ie_dt:1;	/* exchange medium: IE<->DT */
-	unsigned int	res9:4;		/* reserved */
+	unsigned int	res10:4;	/* reserved */
 	unsigned int	xchg_dt_mt:1;	/* exchange medium: DT<->MT */
 	unsigned int	xchg_dt_st:1;	/* exchange medium: DT<->ST */
 	unsigned int	xchg_dt_ie:1;	/* exchange medium: DT<->IE */
 	unsigned int	xchg_dt_dt:1;	/* exchange medium: DT<->DT */
-	unsigned int	res10:4;	/* reserved */
+	unsigned int	res11:4;	/* reserved */
 #else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
 	unsigned int	res:2;		/* reserved */
 	unsigned int	pg_code:6;	/* page code */
@@ -323,46 +328,47 @@ typedef struct dev_capab {
 	unsigned int	stor_ie:1;	/* import/export element */
 	unsigned int	stor_st:1;	/* storage element */
 	unsigned int	stor_mt:1;	/* medium transport element */
+	unsigned int	res2:8;		/* reserved */
 
-	unsigned int	res2:4;		/* reserved */
+	unsigned int	res3:4;		/* reserved */
 	unsigned int	move_mt_dt:1;	/* move medium: MT->DT */
 	unsigned int	move_mt_ie:1;	/* move medium: MT->IE */
 	unsigned int	move_mt_st:1;	/* move medium: MT->ST */
 	unsigned int	move_mt_mt:1;	/* move medium: MT->MT */
-	unsigned int	res3:4;		/* reserved */
+	unsigned int	res4:4;		/* reserved */
 	unsigned int	move_st_dt:1;	/* move medium: ST->DT */
 	unsigned int	move_st_ie:1;	/* move medium: ST->IE */
 	unsigned int	move_st_st:1;	/* move medium: ST->ST */
 	unsigned int	move_st_mt:1;	/* move medium: ST->MT */
-	unsigned int	res4:4;		/* reserved */
+	unsigned int	res5:4;		/* reserved */
 	unsigned int	move_ie_dt:1;	/* move medium: IE->DT */
 	unsigned int	move_ie_ie:1;	/* move medium: IE->IE */
 	unsigned int	move_ie_st:1;	/* move medium: IE->ST */
 	unsigned int	move_ie_mt:1;	/* move medium: IE->MT */
-	unsigned int	res5:4;		/* reserved */
+	unsigned int	res6:4;		/* reserved */
 	unsigned int	move_dt_dt:1;	/* move medium: DT->DT */
 	unsigned int	move_dt_ie:1;	/* move medium: DT->IE */
 	unsigned int	move_dt_st:1;	/* move medium: DT->ST */
 	unsigned int	move_dt_mt:1;	/* move medium: DT->MT */
 
-	unsigned int	res6:32;	/* reserved */
+	unsigned int	res7:32;	/* reserved */
 
-	unsigned int	res7:4;		/* reserved */
+	unsigned int	res8:4;		/* reserved */
 	unsigned int	xchg_mt_dt:1;	/* exchange medium: MT<->DT */
 	unsigned int	xchg_mt_ie:1;	/* exchange medium: MT<->IE */
 	unsigned int	xchg_mt_st:1;	/* exchange medium: MT<->ST */
 	unsigned int	xchg_mt_mt:1;	/* exchange medium: MT<->MT */
-	unsigned int	res8:4;		/* reserved */
+	unsigned int	res9:4;		/* reserved */
 	unsigned int	xchg_st_dt:1;	/* exchange medium: ST<->DT */
 	unsigned int	xchg_st_ie:1;	/* exchange medium: ST<->IE */
 	unsigned int	xchg_st_st:1;	/* exchange medium: ST<->ST */
 	unsigned int	xchg_st_mt:1;	/* exchange medium: ST<->MT */
-	unsigned int	res9:4;		/* reserved */
+	unsigned int	res10:4;	/* reserved */
 	unsigned int	xchg_ie_dt:1;	/* exchange medium: IE<->DT */
 	unsigned int	xchg_ie_ie:1;	/* exchange medium: IE<->IE */
 	unsigned int	xchg_ie_st:1;	/* exchange medium: IE<->ST */
 	unsigned int	xchg_ie_mt:1;	/* exchange medium: IE<->MT */
-	unsigned int	res10:4;	/* reserved */
+	unsigned int	res11:4;	/* reserved */
 	unsigned int	xchg_dt_dt:1;	/* exchange medium: DT<->DT */
 	unsigned int	xchg_dt_ie:1;	/* exchange medium: DT<->IE */
 	unsigned int	xchg_dt_st:1;	/* exchange medium: DT<->ST */
@@ -516,14 +522,235 @@ typedef struct toc_trk_descr {
 } toc_trk_descr_t;
 
 
+/*
+ * Medium changer Read Element Status data structures
+ */
+
+/* Element status header */
+typedef struct {
+	unsigned int	firstelem:16;	/* first elem addr reported */
+	unsigned int	numelem:16;	/* number of elems reported */
+
+	unsigned int	datalen:32;	/* report byte cnt max=0xffffff */
+} elemhdr_t;
+
+
+/* Element types */
+#define ELEMTYP_ALL	0		/* all types */
+#define ELEMTYP_MT	1		/* medium transport */
+#define ELEMTYP_ST	2		/* storage */
+#define ELEMTYP_IE	3		/* import export */
+#define ELEMTYP_DT	4		/* data transfer */
+
+
+/* Element status page */
+typedef struct {
+	unsigned int	type:8;		/* elem type code */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	res1:6;		/* reserved */
+	unsigned int	avoltag:1;	/* avoltag */
+	unsigned int	pvoltag:1;	/* pvoltag */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	pvoltag:1;	/* pvoltag */
+	unsigned int	avoltag:1;	/* avoltag */
+	unsigned int	res1:6;		/* reserved */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	desclen:16;	/* elem descriptor length each */
+
+	unsigned int	pagelen:32;	/* descriptors byte cnt max=0xffffff */
+} elemstat_t;
+
+
+/* Medium transport element descriptor */
+typedef struct {
+	unsigned int	elemaddr:16;	/* element address */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	full:1;		/* full */
+	unsigned int	res2:1;		/* reserved */
+	unsigned int	except:1;	/* except */
+	unsigned int	res1:5;		/* reserved */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	res1:5;		/* reserved */
+	unsigned int	except:1;	/* except */
+	unsigned int	res2:1;		/* reserved */
+	unsigned int	full:1;		/* full */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	res3:8;		/* reserved */
+
+	unsigned int	asc:8;		/* additional sense code */
+	unsigned int	ascq:8;		/* additional sense code qualifier */
+	unsigned int	res4:16;	/* reserved */
+
+	unsigned int	res5:8;		/* reserved */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	res6:6;		/* reserved */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	svalid:1;	/* svalid */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	svalid:1;	/* svalid */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	res6:6;		/* reserved */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	srcaddr:16;	/* source storage elem address */
+
+	byte_t		pri_voltag[35];	/* primary volume tag info */
+	byte_t		alt_voltag[15];	/* alternate volume tag info */
+} elemdesc_mt_t;
+
+
+/* Storage element descriptor */
+typedef struct {
+	unsigned int	elemaddr:16;	/* element address */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	full:1;		/* full */
+	unsigned int	res2:1;		/* reserved */
+	unsigned int	except:1;	/* except */
+	unsigned int	access:1;	/* access */
+	unsigned int	res1:4;		/* reserved */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	res1:4;		/* reserved */
+	unsigned int	access:1;	/* access */
+	unsigned int	except:1;	/* except */
+	unsigned int	res2:1;		/* reserved */
+	unsigned int	full:1;		/* full */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	res3:8;		/* reserved */
+
+	unsigned int	asc:8;		/* additional sense code */
+	unsigned int	ascq:8;		/* additional sense code qualifier */
+	unsigned int	res4:16;	/* reserved */
+
+	unsigned int	res5:8;		/* reserved */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	res6:6;		/* reserved */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	svalid:1;	/* svalid */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	svalid:1;	/* svalid */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	res6:6;		/* reserved */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	srcaddr:16;	/* source storage elem address */
+
+	byte_t		pri_voltag[35];	/* primary volume tag info */
+	byte_t		alt_voltag[15];	/* alternate volume tag info */
+} elemdesc_st_t;
+
+
+/* Import/export element descriptor */
+typedef struct {
+	unsigned int	elemaddr:16;	/* element address */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	full:1;		/* full */
+	unsigned int	impexp:1;	/* impexp */
+	unsigned int	except:1;	/* except */
+	unsigned int	access:1;	/* access */
+	unsigned int	exenab:1;	/* exenab */
+	unsigned int	inenab:1;	/* inenab */
+	unsigned int	res1:2;		/* reserved */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	res1:2;		/* reserved */
+	unsigned int	inenab:1;	/* inenab */
+	unsigned int	exenab:1;	/* exenab */
+	unsigned int	access:1;	/* access */
+	unsigned int	except:1;	/* except */
+	unsigned int	impexp:1;	/* impexp */
+	unsigned int	full:1;		/* full */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	res3:8;		/* reserved */
+
+	unsigned int	asc:8;		/* additional sense code */
+	unsigned int	ascq:8;		/* additional sense code qualifier */
+	unsigned int	res4:16;	/* reserved */
+
+	unsigned int	res5:8;		/* reserved */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	res6:6;		/* reserved */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	svalid:1;	/* svalid */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	svalid:1;	/* svalid */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	res6:6;		/* reserved */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	srcaddr:16;	/* source storage elem address */
+
+	byte_t		pri_voltag[35];	/* primary volume tag info */
+	byte_t		alt_voltag[15];	/* alternate volume tag info */
+} elemdesc_ie_t;
+
+
+/* Data transfer element descriptor */
+typedef struct {
+	unsigned int	elemaddr:16;	/* element address */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	full:1;		/* full */
+	unsigned int	res2:1;		/* reserved */
+	unsigned int	except:1;	/* except */
+	unsigned int	access:1;	/* access */
+	unsigned int	res1:4;		/* reserved */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	res1:4;		/* reserved */
+	unsigned int	access:1;	/* access */
+	unsigned int	except:1;	/* except */
+	unsigned int	res2:1;		/* reserved */
+	unsigned int	full:1;		/* full */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	res3:8;		/* reserved */
+
+	unsigned int	asc:8;		/* additional sense code */
+	unsigned int	ascq:8;		/* additional sense code qualifier */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	lun:3;		/* lun */
+	unsigned int	res5:1;		/* reserved */
+	unsigned int	luvalid:1;	/* luvalid */
+	unsigned int	idvalid:1;	/* idvalid */
+	unsigned int	res6:1;		/* reserved */
+	unsigned int	notbus:1;	/* notbus */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	notbus:1;	/* notbus */
+	unsigned int	res6:1;		/* reserved */
+	unsigned int	idvalid:1;	/* idvalid */
+	unsigned int	luvalid:1;	/* luvalid */
+	unsigned int	res5:1;		/* reserved */
+	unsigned int	lun:3;		/* lun */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	busaddr:8;	/* scsi bus addr */
+
+	unsigned int	res7:8;		/* reserved */
+#if _BYTE_ORDER_ == _L_ENDIAN_
+	unsigned int	res8:6;		/* reserved */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	svalid:1;	/* svalid */
+#else	/* _BYTE_ORDER_ == _B_ENDIAN_ */
+	unsigned int	svalid:1;	/* svalid */
+	unsigned int	invert:1;	/* invert */
+	unsigned int	res8:6;		/* reserved */
+#endif	/* _BYTE_ORDER_ */
+	unsigned int	srcaddr:16;	/* source storage elem address */
+
+	byte_t		pri_voltag[35];	/* primary volume tag info */
+	byte_t		alt_voltag[15];	/* alternate volume tag info */
+} elemdesc_dt_t;
+
+
+/* Union of all element descriptors */
+typedef union {
+	elemdesc_mt_t	mt;		/* medium transport elem descriptor */
+	elemdesc_st_t	st;		/* storage elem descriptor */
+	elemdesc_ie_t	ie;		/* import/export elem descriptor */
+	elemdesc_dt_t	dt;		/* data transfer elem descriptor */
+} elemdesc_t;
+
+
 /*
  * Medium changer parameters structure
  */
 typedef struct {
-	word32_t	mtbase;		/* Medium transport element base addr */
-	word32_t	stbase;		/* Storage element base addr */
-	word32_t	iebase;		/* Import/export element base addr */
-	word32_t	dtbase;		/* Data transfer element base addr */
+	word16_t	mtbase;		/* Medium transport element base addr */
+	word16_t	stbase;		/* Storage element base addr */
+	word16_t	iebase;		/* Import/export element base addr */
+	word16_t	dtbase;		/* Data transfer element base addr */
 } mcparm_t;
 
 
@@ -676,11 +903,13 @@ extern char	*scsipt_vers(void);
 /*
  * Functions for vendor-unique module use only
  */
+extern bool_t	scsipt_rezero_unit(int);
+extern bool_t	scsipt_request_sense(int, byte_t *, int);
 extern bool_t	scsipt_rdsubq(int, byte_t *, byte_t, bool_t, int, bool_t);
 extern bool_t	scsipt_modesense(int, byte_t *, byte_t, byte_t, int);
 extern bool_t	scsipt_modesel(int, byte_t *, byte_t, int);
 extern bool_t	scsipt_inquiry(int, byte_t *, int);
-extern bool_t	scsipt_rdtoc(int, byte_t *, bool_t, int);
+extern bool_t	scsipt_rdtoc(int, byte_t *, int, bool_t);
 extern bool_t	scsipt_playmsf(int, msf_t *, msf_t *);
 extern bool_t	scsipt_play10(int, word32_t, word32_t);
 extern bool_t	scsipt_play12(int, word32_t, word32_t);
@@ -688,8 +917,12 @@ extern bool_t	scsipt_play_trkidx(int, int, int, int, int);
 extern bool_t	scsipt_prev_allow(int, bool_t);
 extern bool_t	scsipt_start_stop(int, bool_t, bool_t, bool_t);
 extern bool_t	scsipt_pause_resume(int, bool_t);
+extern bool_t	scsipt_move_medium(int, word16_t, word16_t, word16_t);
+extern bool_t	scsipt_init_elemstat(int);
+extern bool_t	scsipt_read_elemstat(int, byte_t *, int, int);
 extern bool_t	scsipt_disc_present(int);
 
+
 #else
 
 #define scsipt_init	NULL
