@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
  */
 
 #ifndef LINT
-static char *_slioc_c_ident_ = "@(#)slioc.c	6.110 98/05/18";
+static char *_slioc_c_ident_ = "@(#)slioc.c	6.142 00/01/31";
 #endif
 
 #include "common_d/appenv.h"
@@ -45,7 +45,8 @@ extern FILE		*errfp;
 extern di_client_t	*di_clinfo;
 
 
-STATIC bool_t	slioc_run_ab(curstat_t *),
+STATIC bool_t	slioc_pause_resume(bool_t),
+		slioc_run_ab(curstat_t *),
 		slioc_run_sample(curstat_t *),
 		slioc_run_prog(curstat_t *),
 		slioc_run_repeat(curstat_t *),
@@ -73,9 +74,11 @@ STATIC bool_t	slioc_not_open = TRUE,		/* Device not opened yet */
 		slioc_idx_pause,		/* Prev/next index pausing */
 		slioc_fake_stop,		/* Force a completion status */
 		slioc_playing,			/* Currently playing */
+		slioc_paused,			/* Currently paused */
 		slioc_bcd_hack,			/* Track numbers in BCD hack */
 		slioc_mult_autoplay,		/* Auto-play after disc chg */
-		slioc_mult_pause;		/* Pause after disc chg */
+		slioc_mult_pause,		/* Pause after disc chg */
+		slioc_override_ap;		/* Override auto-play */
 STATIC word32_t	slioc_ab_start_addr,		/* A->B mode start block */
 		slioc_ab_end_addr,		/* A->B mode end block */
 		slioc_sav_end_addr;		/* Err recov saved end addr */
@@ -98,6 +101,9 @@ STATIC iocname_t iname[] = {
 	{ CDROMVOLCTRL,		"CDROMVOLCTRL"		},
 	{ CDROMPLAYTRKIND,	"CDROMPLAYTRKIND"	},
 	{ CDROMPLAYMSF,		"CDROMPLAYMSF"		},
+#ifdef CDROMLOAD
+	{ CDROMLOAD,		"CDROMLOAD"		},
+#endif
 #ifdef linux
 	{ CDROMCLOSETRAY,	"CDROMCLOSETRAY"	},
 	{ CDROM_CHANGER_NSLOTS,	"CDROM_CHANGER_NSLOTS"	},
@@ -114,6 +120,25 @@ STATIC iocname_t iname[] = {
  *  internal routines  *
  ***********************/
 
+#ifdef SOL2_VOLMGT
+/*
+ * slioc_vold_alrm
+ *	SIGALRM handler
+ *
+ * Args:
+ *	signo - The signal number
+ *
+ * Return:
+ *	Nothing.
+ */
+/*ARGSUSED*/
+STATIC void
+slioc_vold_alrm(int signo)
+{
+	DBGPRN(errfp, "\nSIGALRM!\n");
+}
+#endif
+
 
 /*
  * slioc_send
@@ -145,22 +170,30 @@ slioc_send(int cmd, void *arg, int arglen, bool_t prnerr, int *retloc)
 	if (app_data.debug) {
 		for (i = 0; iname[i].name != NULL; i++) {
 			if (iname[i].cmd == cmd) {
-				(void) fprintf(errfp, "\nIOCTL: %s arg=0x%x ",
-					       iname[i].name, (int) arg);
+				(void) fprintf(errfp, "\nIOCTL: %s arg=0x%lx ",
+					       iname[i].name,
+					       (unsigned long) arg);
 				break;
 			}
 		}
 		if (iname[i].name == NULL)
-			(void) fprintf(errfp, "\nIOCTL: 0x%x arg=0x%x ",
-				       cmd, (int) arg);
+			(void) fprintf(errfp, "\nIOCTL: 0x%x arg=0x%lx ",
+				       cmd, (unsigned long) arg);
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) signal(SIGALRM, slioc_vold_alrm);
+		(void) alarm(15);
+	}
+#endif
+
 #ifdef __QNX__
 	switch (cmd) {
 	case CDROMVOLCTRL:
 	case CDROMPLAYTRKIND:
-#ifdef CDROMCLOSETRAY
-	case CDROMCLOSETRAY:
+#ifdef CDROMLOAD
+	case CDROMLOAD:
 #endif
 		ret = ioctl(slioc_fd, cmd, arg);
 		break;
@@ -196,9 +229,26 @@ slioc_send(int cmd, void *arg, int arglen, bool_t prnerr, int *retloc)
 
 			errno = saverrno;
 		}
+
+#ifdef SOL2_VOLMGT
+		if (app_data.sol2_volmgt) {
+			saverrno = errno;
+			(void) alarm(0);
+			(void) signal(SIGALRM, SIG_DFL);
+			errno = saverrno;
+		}
+#endif
+
 		return FALSE;
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+	}
+#endif
+
 	if (retloc != NULL)
 		*retloc = ret;
 
@@ -222,17 +272,29 @@ slioc_open(curstat_t *s)
 {
 	struct stat	stbuf;
 	char		errstr[ERR_BUF_SZ];
+	int		openflgs;
+#ifdef linux
 	int		i,
 			nslots,
 			curslot,
 			ret;
 	static bool_t	first = TRUE;
+#endif
 
 	/* Check for validity of device node */
 	if (stat(s->curdev, &stbuf) < 0) {
-#if (defined(sun) || defined(__sun__)) && defined(SVR4)
-		if (app_data.sol2_volmgt && errno == ENOENT)
-			return FALSE;
+#ifdef SOL2_VOLMGT
+		if (app_data.sol2_volmgt) {
+			switch (errno) {
+			case ENOENT:
+			case EINTR:
+			case ESTALE:
+				return FALSE;
+				/*NOTREACHED*/
+			default:
+				break;
+			}
+		}
 #endif
 		(void) sprintf(errstr, app_data.str_staterr, s->curdev);
 		DI_FATAL(errstr);
@@ -246,16 +308,50 @@ slioc_open(curstat_t *s)
 	if (!S_ISCHR(stbuf.st_mode))
 #endif
 	{
+#ifdef SOL2_VOLMGT
+		/* Some CDs have multiple slices (partitions),
+		 * so the device node becomes a directory when
+		 * vold mounts each slice.
+		 */
+		if (app_data.sol2_volmgt && S_ISDIR(stbuf.st_mode))
+			return FALSE;
+#endif
 		(void) sprintf(errstr, app_data.str_noderr, s->curdev);
 		DI_FATAL(errstr);
 		return FALSE;
 	}
 
-	if ((slioc_fd = open(s->curdev, O_RDONLY | O_EXCL | O_NONBLOCK)) < 0) {
+#ifdef linux
+	openflgs = O_RDONLY | O_EXCL | O_NONBLOCK;
+#else
+	openflgs = O_RDONLY;
+#endif
+
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) signal(SIGALRM, slioc_vold_alrm);
+		(void) alarm(10);
+	}
+#endif
+
+	if ((slioc_fd = open(s->curdev, openflgs)) < 0) {
 		DBGPRN(errfp, "Cannot open %s: errno=%d\n", s->curdev, errno);
+#ifdef SOL2_VOLMGT
+		if (app_data.sol2_volmgt) {
+			(void) alarm(0);
+			(void) signal(SIGALRM, SIG_DFL);
+		}
+#endif
 		return FALSE;
 	}
 
+#ifdef SOL2_VOLMGT
+	if (app_data.sol2_volmgt) {
+		(void) alarm(0);
+		(void) signal(SIGALRM, SIG_DFL);
+	}
+#endif
+
 #ifdef linux
 	/* CD changer setup */
 	if (app_data.numdiscs > 1 && first) {
@@ -368,7 +464,8 @@ slioc_open(curstat_t *s)
 
 		DBGPRN(errfp, "Changing to disc %d\n", s->cur_disc);
 
-		if (!slioc_send(CDROM_SELECT_DISC, (void *) (s->cur_disc - 1),
+		if (!slioc_send(CDROM_SELECT_DISC,
+				(void *)(long) (s->cur_disc - 1),
 				0, app_data.debug, NULL)) {
 			DBGPRN(errfp, "%s\n\nClose: %s\n",
 				"Disc change failed.", s->curdev);
@@ -423,18 +520,18 @@ slioc_close(void)
  *
  * Args:
  *	buf - Pointer to the return data buffer
- *	msf - Whether to use MSF or logical block address format
+ *	msf - Whether to use MSF or logical block address data format
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
 STATIC bool_t
-slioc_rdsubq(struct cdrom_subchnl *sub, byte_t msf)
+slioc_rdsubq(struct cdrom_subchnl *sub, bool_t msf)
 {
 	bool_t	ret;
 
-	sub->cdsc_format = msf;
+	sub->cdsc_format = msf ? CDROM_MSF : CDROM_LBA;
 	ret = slioc_send(CDROMSUBCHNL, sub, sizeof(struct cdrom_subchnl),
 			 TRUE, NULL);
 
@@ -456,13 +553,15 @@ slioc_rdsubq(struct cdrom_subchnl *sub, byte_t msf)
  *	h - address of pointer to array of toc entrys will be allocated
  *	    by this routine
  *	start - Starting track number for which the TOC data is returned
+ *	msf - Whether to use MSF or logical block address data format
  *
  * Return:
  *	TRUE - success
  *	FALSE - failure
  */
 STATIC bool_t
-slioc_rdtoc(struct cdrom_tochdr *h, struct cdrom_tocentry **e, int start)
+slioc_rdtoc(struct cdrom_tochdr *h, struct cdrom_tocentry **e,
+	    int start, bool_t msf)
 {
 	int			i,
 				j,
@@ -486,8 +585,14 @@ slioc_rdtoc(struct cdrom_tochdr *h, struct cdrom_tocentry **e, int start)
 
 	allocsz = (h->cdth_trk1 - start + 2) * sizeof(struct cdrom_tocentry);
 
-	*e = (struct cdrom_tocentry *)(void *) MEM_ALLOC(allocsz);
-	t = (struct cdrom_tocentry *)(void *) MEM_ALLOC(allocsz);
+	*e = (struct cdrom_tocentry *)(void *) MEM_ALLOC(
+		"cdrom_tocentry",
+		allocsz
+	);
+	t = (struct cdrom_tocentry *)(void *) MEM_ALLOC(
+		"cdrom_tocentry",
+		allocsz
+	);
 
 	if (*e == NULL || t == NULL) {
 		DI_FATAL(app_data.str_nomemory);
@@ -503,7 +608,7 @@ slioc_rdtoc(struct cdrom_tochdr *h, struct cdrom_tocentry **e, int start)
 		t[j].cdte_track =
 			(i <= (int) h->cdth_trk1) ? i : CDROM_LEADOUT;
 
-		t[j].cdte_format = CDROM_MSF;
+		t[j].cdte_format = msf ? CDROM_MSF : CDROM_LBA;
 
 		if (!slioc_send(CDROMREADTOCENTRY, &t[j],
 				sizeof(struct cdrom_tocentry), TRUE, NULL)) {
@@ -592,7 +697,9 @@ slioc_disc_present(bool_t savstat)
 	struct cdrom_tocentry	e;
 	word32_t		a1,
 				a2;
+#ifdef linux
 	int			ret;
+#endif
 	static int		tot_trks = 0;
 	static word32_t		sav_a1 = 0,
 				sav_a2 = 0;
@@ -624,7 +731,7 @@ slioc_disc_present(bool_t savstat)
 
 	/* Fake it with CDROMSUBCHNL */
 	(void) memset((byte_t *) &sub, 0, sizeof(struct cdrom_subchnl));
-	sub.cdsc_format = CDROM_MSF;
+	sub.cdsc_format = app_data.subq_lba ? CDROM_LBA : CDROM_MSF;
 	if (!slioc_send(CDROMSUBCHNL, &sub, sizeof(struct cdrom_subchnl),
 			app_data.debug, NULL))
 		return FALSE;
@@ -682,7 +789,7 @@ slioc_disc_present(bool_t savstat)
 	}
 
 	(void) memset((byte_t *) &e, 0, sizeof(struct cdrom_tocentry));
-	e.cdte_format = CDROM_MSF;
+	e.cdte_format = app_data.toc_lba ? CDROM_LBA : CDROM_MSF;
 	e.cdte_track = h.cdth_trk0;
 	if (!slioc_send(CDROMREADTOCENTRY, &e, sizeof(struct cdrom_tocentry),
 			app_data.debug, NULL))
@@ -691,7 +798,7 @@ slioc_disc_present(bool_t savstat)
 	a1 = (word32_t) e.cdte_addr.lba;
 
 	(void) memset((byte_t *) &e, 0, sizeof(struct cdrom_tocentry));
-	e.cdte_format = CDROM_MSF;
+	e.cdte_format = app_data.toc_lba ? CDROM_LBA : CDROM_MSF;
 	e.cdte_track = h.cdth_trk1;
 	if (!slioc_send(CDROMREADTOCENTRY, &e, sizeof(struct cdrom_tocentry),
 			app_data.debug, NULL))
@@ -808,13 +915,20 @@ slioc_start_stop(bool_t start, bool_t loej)
 {
 	bool_t	ret;
 
+	if (app_data.strict_pause_resume && slioc_paused)
+		(void) slioc_pause_resume(TRUE);
+
 	if (start) {
 		if (loej)
 #ifdef CDROMCLOSETRAY
 			ret = slioc_send(CDROMCLOSETRAY, NULL, 0, TRUE, NULL);
+#else
+#ifdef CDROMLOAD
+			ret = slioc_send(CDROMLOAD, NULL, 0, TRUE, NULL);
 #else
 			ret = FALSE;
-#endif
+#endif	/* CDROMLOAD */
+#endif	/* CDROMCLOSETRAY */
 		else
 			ret = slioc_send(CDROMSTART, NULL, 0, TRUE, NULL);
 	}
@@ -832,8 +946,19 @@ slioc_start_stop(bool_t start, bool_t loej)
 	 * don't handle new commands well when the CD is loading/ejecting
 	 * with the IMMED bit set in the Start/Stop Unit command.
 	 */
-	if (ret && loej)
-		(void) sleep((app_data.ins_interval + 1000 - 1) / 1000);
+	if (ret) {
+		if (loej) {
+			int	n;
+
+			n = (app_data.ins_interval + 1000 - 1) / 1000;
+			if (start)
+				n *= 2;
+
+			(void) sleep(n);
+		}
+		else if (start && app_data.spinup_interval > 0)
+			(void) sleep(app_data.spinup_interval);
+	}
 
 	return (ret);
 
@@ -851,13 +976,22 @@ slioc_start_stop(bool_t start, bool_t loej)
  *	TRUE - success
  *	FALSE - failure
  */
-bool_t
+STATIC bool_t
 slioc_pause_resume(bool_t resume)
 {
-	return (
-		slioc_send(resume ? CDROMRESUME : CDROMPAUSE, NULL, 0,
-		TRUE, NULL)
+	bool_t	ret;
+
+	if (!app_data.pause_supp)
+		return FALSE;
+
+	ret = slioc_send(
+		resume ? CDROMRESUME : CDROMPAUSE,
+		NULL, 0, TRUE, NULL
 	);
+	if (ret)
+		slioc_paused = !resume;
+
+	return (ret);
 }
 
 
@@ -942,12 +1076,22 @@ slioc_do_playaudio(
 	do_playti = (addr_fmt & ADDR_TRKIDX) && app_data.playti_supp;
 
 	if (do_playmsf || do_playti) {
-		if (slioc_playing)
-			/* Pause playback first */
-			(void) slioc_pause_resume(FALSE);
-		else
+		if (slioc_paused) {
+			if (app_data.strict_pause_resume) {
+				/* Resume first */
+				(void) slioc_pause_resume(TRUE);
+			}
+		}
+		else if (slioc_playing) {
+			if (app_data.play_pause_play) {
+				/* Pause first */
+				(void) slioc_pause_resume(FALSE);
+			}
+		}
+		else {
 			/* Spin up CD */
 			(void) slioc_start_stop(TRUE, FALSE);
+		}
 	}
 
 	if (do_playmsf)
@@ -956,8 +1100,10 @@ slioc_do_playaudio(
 	if (!ret && do_playti)
 		ret = slioc_play_trkidx(trk, idx, trk, idx);
 
-	if (ret)
+	if (ret) {
 		slioc_playing = TRUE;
+		slioc_paused = FALSE;
+	}
 
 	return (ret);
 }
@@ -1062,7 +1208,7 @@ slioc_get_playstatus(curstat_t *s)
 	in_slioc_get_playstatus = TRUE;
 
 	(void) memset((byte_t *) &sub, 0, sizeof(struct cdrom_subchnl));
-	if (!slioc_rdsubq(&sub, CDROM_MSF)) {
+	if (!slioc_rdsubq(&sub, !app_data.subq_lba)) {
 		/* Check to see if the disc had been manually ejected */
 		if (!slioc_disc_ready(s)) {
 			slioc_sav_end_addr = 0;
@@ -1089,27 +1235,55 @@ slioc_get_playstatus(curstat_t *s)
 
 	/* Check the subchannel data */
 
-	s->cur_tot_frame = sub.cdsc_absaddr.msf.frame;
-	s->cur_tot_sec = sub.cdsc_absaddr.msf.second;
-	s->cur_tot_min = sub.cdsc_absaddr.msf.minute;
-	util_msftoblk(
-		s->cur_tot_min,
-		s->cur_tot_sec,
-		s->cur_tot_frame,
-		&s->cur_tot_addr,
-		MSF_OFFSET
-	);
+	if (sub.cdsc_format == CDROM_LBA) {
+		/* LBA mode */
+		s->cur_tot_addr = di_xlate_blk(sub.cdsc_absaddr.lba);
+		util_blktomsf(
+			s->cur_tot_addr,
+			&s->cur_tot_min,
+			&s->cur_tot_sec,
+			&s->cur_tot_frame,
+			MSF_OFFSET
+		);
 
-	s->cur_trk_frame = sub.cdsc_reladdr.msf.frame;
-	s->cur_trk_sec = sub.cdsc_reladdr.msf.second;
-	s->cur_trk_min = sub.cdsc_reladdr.msf.minute;
-	util_msftoblk(
-		s->cur_trk_min,
-		s->cur_trk_sec,
-		s->cur_trk_frame,
-		&s->cur_trk_addr,
-		0
-	);
+		s->cur_trk_addr = di_xlate_blk(sub.cdsc_reladdr.lba);
+		util_blktomsf(
+			s->cur_trk_addr,
+			&s->cur_trk_min,
+			&s->cur_trk_sec,
+			&s->cur_trk_frame,
+#ifdef linux
+			/* Note: Linux pre-adjusts for MSF_OFFSET */
+			MSF_OFFSET
+#else
+			0
+#endif
+		);
+	}
+	else {
+		/* MSF mode */
+		s->cur_tot_frame = sub.cdsc_absaddr.msf.frame;
+		s->cur_tot_sec = sub.cdsc_absaddr.msf.second;
+		s->cur_tot_min = sub.cdsc_absaddr.msf.minute;
+		util_msftoblk(
+			s->cur_tot_min,
+			s->cur_tot_sec,
+			s->cur_tot_frame,
+			&s->cur_tot_addr,
+			MSF_OFFSET
+		);
+
+		s->cur_trk_frame = sub.cdsc_reladdr.msf.frame;
+		s->cur_trk_sec = sub.cdsc_reladdr.msf.second;
+		s->cur_trk_min = sub.cdsc_reladdr.msf.minute;
+		util_msftoblk(
+			s->cur_trk_min,
+			s->cur_trk_sec,
+			s->cur_trk_frame,
+			&s->cur_trk_addr,
+			0
+		);
+	}
 
 	/* Update time display */
 	DPY_TIME(s, FALSE);
@@ -1208,6 +1382,7 @@ slioc_get_playstatus(curstat_t *s)
 	case CDROM_AUDIO_COMPLETED:
 	case CDROM_AUDIO_NO_STATUS:
 	case CDROM_AUDIO_INVALID:
+	default:
 		done = TRUE;
 
 		if (!slioc_fake_stop)
@@ -1239,11 +1414,6 @@ slioc_get_playstatus(curstat_t *s)
 		}
 
 		break;
-
-	default:
-		/* Something is wrong with the data. */
-		done = FALSE;
-		break;
 	}
 
 	if (done) {
@@ -1326,6 +1496,10 @@ slioc_get_playstatus(curstat_t *s)
 			/* Spin down the disc */
 			slioc_start_stop(FALSE, FALSE);
 		}
+		if (app_data.done_exit) {
+			/* Exit */
+			di_clinfo->quit(s);
+		}
 
 		in_slioc_get_playstatus = FALSE;
 		return FALSE;
@@ -1354,6 +1528,9 @@ slioc_cfg_vol(int vol, curstat_t *s, bool_t query)
 	struct cdrom_volctrl	volctrl;
 	static bool_t		first = TRUE;
 
+	if (!app_data.mselvol_supp)
+		return 0;
+
 	(void) memset((byte_t *) &volctrl, 0, sizeof(struct cdrom_volctrl));
 
 	if (query) {
@@ -1488,7 +1665,7 @@ slioc_get_toc(curstat_t *s)
 	struct cdrom_tocentry	*e,
 				*p;
 
-	if (!slioc_rdtoc(&h, &e, 0))
+	if (!slioc_rdtoc(&h, &e, 0, !app_data.toc_lba))
 		return FALSE;
 
 	/* Fill curstat structure with TOC data */
@@ -1499,17 +1676,33 @@ slioc_get_toc(curstat_t *s)
 
 	for (i = 0; i <= (int) ntrks; i++) {
 		s->trkinfo[i].trkno = p->cdte_track;
-		s->trkinfo[i].type = TYP_AUDIO;	/* shrug */
-		s->trkinfo[i].min = p->cdte_addr.msf.minute;
-		s->trkinfo[i].sec = p->cdte_addr.msf.second;
-		s->trkinfo[i].frame = p->cdte_addr.msf.frame;
-		util_msftoblk(
-			s->trkinfo[i].min,
-			s->trkinfo[i].sec,
-			s->trkinfo[i].frame,
-			&s->trkinfo[i].addr,
-			MSF_OFFSET
-		);
+		s->trkinfo[i].type =
+		    (p->cdte_ctrl & CDROM_DATA_TRACK) ? TYP_DATA : TYP_AUDIO;
+
+		if (p->cdte_format == CDROM_LBA) {
+			/* LBA mode */
+			s->trkinfo[i].addr = di_xlate_blk(p->cdte_addr.lba);
+			util_blktomsf(
+				s->trkinfo[i].addr,
+				&s->trkinfo[i].min,
+				&s->trkinfo[i].sec,
+				&s->trkinfo[i].frame,
+				MSF_OFFSET
+			);
+		}
+		else {
+			/* MSF mode */
+			s->trkinfo[i].min = p->cdte_addr.msf.minute;
+			s->trkinfo[i].sec = p->cdte_addr.msf.second;
+			s->trkinfo[i].frame = p->cdte_addr.msf.frame;
+			util_msftoblk(
+				s->trkinfo[i].min,
+				s->trkinfo[i].sec,
+				s->trkinfo[i].frame,
+				&s->trkinfo[i].addr,
+				MSF_OFFSET
+			);
+		}
 
 		if (p->cdte_track == CDROM_LEADOUT || i == (MAXTRACK - 1)) {
 			s->tot_min = s->trkinfo[i].min;
@@ -1594,6 +1787,9 @@ slioc_stop_stat_poll(void)
 STATIC void
 slioc_start_insert_poll(curstat_t *s)
 {
+	int	delay;
+	bool_t	first = TRUE;
+
 	if (slioc_insert_polling || app_data.ins_disable ||
 	    (s->mode != MOD_BUSY && s->mode != MOD_NODISC))
 		return;
@@ -1607,10 +1803,17 @@ slioc_start_insert_poll(curstat_t *s)
 	if (slioc_ins_interval < 500)
 		slioc_ins_interval = 500;
 
+	if (first) {
+		first = FALSE;
+		delay = 50;
+	}
+	else
+		delay = slioc_ins_interval;
+
 	/* Start poll timer */
 	if (di_clinfo->timeout != NULL) {
 		slioc_insert_id = di_clinfo->timeout(
-			slioc_ins_interval,
+			delay,
 			slioc_insert_poll,
 			(byte_t *) s
 		);
@@ -1762,7 +1965,7 @@ slioc_disc_ready(curstat_t *s)
 						/* Try the next disc */
 						s->cur_disc++;
 					}
-					else {
+					else if (!s->chgrscan) {
 						/* Go to the first disc */
 						s->cur_disc = s->first_disc;
 
@@ -1774,6 +1977,10 @@ slioc_disc_ready(curstat_t *s)
 							FALSE;
 						}
 					}
+					else {
+						/* Done scanning - no disc */
+						STOPSCAN(s);
+					}
 				}
 			}
 
@@ -1866,7 +2073,8 @@ slioc_disc_ready(curstat_t *s)
 		(void) sleep(1);
 
 		/* Start auto-play */
-		slioc_play_pause(s);
+		if (!slioc_override_ap)
+			slioc_play_pause(s);
 
 		if (slioc_mult_pause) {
 			slioc_mult_pause = FALSE;
@@ -2208,8 +2416,10 @@ slioc_run_sample(curstat_t *s)
 			MSF_OFFSET
 		);
 
-		if (slioc_do_playaudio(ADDR_BLK | ADDR_MSF,
-				 saddr, eaddr, &smsf, &emsf, 0, 0)) {
+		/* Sample only audio tracks */
+		if (s->trkinfo[slioc_next_sam].type != TYP_AUDIO ||
+		    slioc_do_playaudio(ADDR_BLK | ADDR_MSF,
+				       saddr, eaddr, &smsf, &emsf, 0, 0)) {
 			slioc_next_sam++;
 			return TRUE;
 		}
@@ -2320,7 +2530,7 @@ slioc_run_prog(curstat_t *s)
 	);
 
 	if (s->mode == MOD_PAUSE) {
-		slioc_pause_resume(FALSE);
+		(void) slioc_pause_resume(FALSE);
 
 		/* Restore volume */
 		slioc_mute_off(s);
@@ -2386,7 +2596,7 @@ slioc_run_repeat(curstat_t *s)
 		);
 
 		if (s->mode == MOD_PAUSE) {
-			slioc_pause_resume(FALSE);
+			(void) slioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			slioc_mute_off(s);
@@ -2648,12 +2858,25 @@ slioc_shuffle(curstat_t *s, bool_t enable)
 void
 slioc_load_eject(curstat_t *s)
 {
-	if (!slioc_disc_ready(s)) {
+	int	i;
+	bool_t	ret;
+
+	if (!slioc_disc_ready(s) && app_data.load_supp) {
 		/* Disc not ready: try loading the disc */
 		if (!slioc_start_stop(TRUE, TRUE))
 			DO_BEEP();
 
-		return;
+		/* Wait for CD to become ready */
+		for (i = 0; i < 3; i++) {
+			if ((ret = slioc_disc_ready(s)) == TRUE)
+				break;
+			sleep(1);
+		}
+
+		if (ret || !app_data.eject_supp)
+			return;
+
+		/* Fall through and open tray again */
 	}
 
 	/* Eject the disc */
@@ -2687,7 +2910,7 @@ slioc_load_eject(curstat_t *s)
 	DBCLEAR(s, FALSE);
 	DPY_ALL(s);
 
-	slioc_start_stop(FALSE, TRUE);
+	(void) slioc_start_stop(FALSE, TRUE);
 
 	if (app_data.eject_exit)
 		di_clinfo->quit(s);
@@ -2901,11 +3124,16 @@ slioc_play_pause(curstat_t *s)
 	msf_t		start_msf,
 			end_msf;
 
+	slioc_override_ap = TRUE;
+
 	if (!slioc_disc_ready(s)) {
+		slioc_override_ap = FALSE;
 		DO_BEEP();
 		return;
 	}
 
+	slioc_override_ap = FALSE;
+
 	if (s->mode == MOD_NODISC)
 		s->mode = MOD_STOP;
 
@@ -3083,10 +3311,8 @@ slioc_stop(curstat_t *s, bool_t stop_disc)
 	 * immediately go into play state again.  Not spinning down
 	 * the drive makes things a little faster...
 	 */
-	if (!slioc_disc_ready(s)) {
-		DO_BEEP();
+	if (!slioc_disc_ready(s))
 		return;
-	}
 
 	switch (s->mode) {
 	case MOD_PLAY:
@@ -3111,7 +3337,6 @@ slioc_stop(curstat_t *s, bool_t stop_disc)
 		break;
 
 	default:
-		DO_BEEP();
 		break;
 	}
 }
@@ -3165,11 +3390,13 @@ slioc_chgdisc(curstat_t *s)
 
 	s->rptcnt = sav_rptcnt;
 
-	/* Close CD-ROM device */
-	DBGPRN(errfp, "\nClose CD-ROM: %s\n", s->curdev);
-	slioc_close();
+	if (app_data.eject_close) {
+		/* Close CD-ROM device */
+		DBGPRN(errfp, "\nClose CD-ROM: %s\n", s->curdev);
+		slioc_close();
 
-	slioc_not_open = TRUE;
+		slioc_not_open = TRUE;
+	}
 
 	/* Load desired disc */
 	slioc_disc_ready(s);
@@ -3334,7 +3561,7 @@ slioc_prevtrk(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				slioc_pause_resume(FALSE);
+				(void) slioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				slioc_mute_off(s);
@@ -3482,7 +3709,7 @@ slioc_nexttrk(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				slioc_pause_resume(FALSE);
+				(void) slioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				slioc_mute_off(s);
@@ -3584,6 +3811,9 @@ slioc_previdx(curstat_t *s)
 			return;
 		}
 
+		/* A small delay to make sure the command took effect */
+		util_delayms(10);
+
 		slioc_idx_pause = TRUE;
 
 		if (!slioc_pause_resume(FALSE)) {
@@ -3626,7 +3856,7 @@ slioc_previdx(curstat_t *s)
 		slioc_idx_pause = FALSE;
 
 		if (s->mode == MOD_PAUSE) {
-			slioc_pause_resume(FALSE);
+			(void) slioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			slioc_mute_off(s);
@@ -3701,6 +3931,9 @@ slioc_nextidx(curstat_t *s)
 			return;
 		}
 
+		/* A small delay to make sure the command took effect */
+		util_delayms(10);
+
 		slioc_idx_pause = TRUE;
 
 		if (!slioc_pause_resume(FALSE)) {
@@ -3743,7 +3976,7 @@ slioc_nextidx(curstat_t *s)
 		slioc_idx_pause = FALSE;
 
 		if (s->mode == MOD_PAUSE) {
-			slioc_pause_resume(FALSE);
+			(void) slioc_pause_resume(FALSE);
 
 			/* Restore volume */
 			slioc_mute_off(s);
@@ -3874,7 +4107,7 @@ slioc_rew(curstat_t *s, bool_t start)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				slioc_pause_resume(FALSE);
+				(void) slioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				slioc_mute_off(s);
@@ -4004,7 +4237,7 @@ slioc_ff(curstat_t *s, bool_t start)
 				}
 			}
 			if (s->mode == MOD_PAUSE) {
-				slioc_pause_resume(FALSE);
+				(void) slioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				slioc_mute_off(s);
@@ -4096,7 +4329,7 @@ slioc_warp(curstat_t *s)
 			}
 
 			if (s->mode == MOD_PAUSE) {
-				slioc_pause_resume(FALSE);
+				(void) slioc_pause_resume(FALSE);
 
 				/* Restore volume */
 				slioc_mute_off(s);
@@ -4158,7 +4391,10 @@ void
 slioc_start(curstat_t *s)
 {
 	/* Check to see if disc is ready */
-	(void) slioc_disc_ready(s);
+	if (di_clinfo->timeout != NULL)
+		slioc_start_insert_poll(s);
+	else
+		(void) slioc_disc_ready(s);
 
 	/* Update display */
 	DPY_ALL(s);
