@@ -1,7 +1,7 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This library is free software; you can redistribute it and/or
@@ -26,7 +26,7 @@
  *   used here for identification purposes only.
  */
 #ifndef LINT
-static char *_vu_chin_c_ident_ = "@(#)vu_chin.c	6.24 98/05/10";
+static char *_vu_chin_c_ident_ = "@(#)vu_chin.c	6.27 00/01/18";
 #endif
 
 #include "common_d/appenv.h"
@@ -190,7 +190,7 @@ chin_get_playstatus(curstat_t *s, byte_t *audio_status)
 
 	(void) memset(buf, 0, sizeof(buf));
 
-	if (!scsipt_rdsubq(devfd, buf, SUB_ALL, 1, 0, TRUE)) {
+	if (!scsipt_rdsubq(devfd, buf, SUB_ALL, 1, 0, !app_data.subq_lba)) {
 		if (!di_check_disc(s))
 			/* Someone ejected the disc manually */
 			return FALSE;
@@ -248,27 +248,50 @@ chin_get_playstatus(curstat_t *s, byte_t *audio_status)
 			DPY_INDEX(s);
 		}
 
-		s->cur_tot_min = p->abs_addr.msf.min;
-		s->cur_tot_sec = p->abs_addr.msf.sec;
-		s->cur_tot_frame = p->abs_addr.msf.frame;
-		util_msftoblk(
-			s->cur_tot_min,
-			s->cur_tot_sec,
-			s->cur_tot_frame,
-			&s->cur_tot_addr,
-			MSF_OFFSET
-		);
+		if (app_data.subq_lba) {
+			/* LBA mode */
+			s->cur_tot_addr = util_bswap32(p->abs_addr.logical);
+			util_blktomsf(
+				s->cur_tot_addr,
+				&s->cur_tot_min,
+				&s->cur_tot_sec,
+				&s->cur_tot_frame,
+				MSF_OFFSET
+			);
 
-		s->cur_trk_min = p->rel_addr.msf.min;
-		s->cur_trk_sec = p->rel_addr.msf.sec;
-		s->cur_trk_frame = p->rel_addr.msf.frame;
-		util_msftoblk(
-			s->cur_trk_min,
-			s->cur_trk_sec,
-			s->cur_trk_frame,
-			&s->cur_trk_addr,
-			0
-		);
+			s->cur_trk_addr = util_bswap32(p->rel_addr.logical);
+			util_blktomsf(
+				s->cur_trk_addr,
+				&s->cur_trk_min,
+				&s->cur_trk_sec,
+				&s->cur_trk_frame,
+				0
+			);
+		}
+		else {
+			/* MSF mode */
+			s->cur_tot_min = p->abs_addr.msf.min;
+			s->cur_tot_sec = p->abs_addr.msf.sec;
+			s->cur_tot_frame = p->abs_addr.msf.frame;
+			util_msftoblk(
+				s->cur_tot_min,
+				s->cur_tot_sec,
+				s->cur_tot_frame,
+				&s->cur_tot_addr,
+				MSF_OFFSET
+			);
+
+			s->cur_trk_min = p->rel_addr.msf.min;
+			s->cur_trk_sec = p->rel_addr.msf.sec;
+			s->cur_trk_frame = p->rel_addr.msf.frame;
+			util_msftoblk(
+				s->cur_trk_min,
+				s->cur_trk_sec,
+				s->cur_trk_frame,
+				&s->cur_trk_addr,
+				0
+			);
+		}
 
 		break;
 	default:
@@ -310,7 +333,7 @@ chin_get_toc(curstat_t *s)
 
 	(void) memset(buf, 0, sizeof(buf));
 
-	if (!scsipt_rdtoc(devfd, buf, TRUE, 0))
+	if (!scsipt_rdtoc(devfd, buf, 0, !app_data.toc_lba))
 		return FALSE;
 
 	/* Fill curstat structure with TOC data */
@@ -364,16 +387,32 @@ chin_get_toc(curstat_t *s)
 
 		s->trkinfo[i].type = (p->trktype == 0) ?
 			TYP_AUDIO : TYP_DATA;
-		s->trkinfo[i].min = p->abs_addr.msf.min;
-		s->trkinfo[i].sec = p->abs_addr.msf.sec;
-		s->trkinfo[i].frame = p->abs_addr.msf.frame;
-		util_msftoblk(
-			s->trkinfo[i].min,
-			s->trkinfo[i].sec,
-			s->trkinfo[i].frame,
-			&s->trkinfo[i].addr,
-			MSF_OFFSET
-		);
+
+		if (app_data.toc_lba) {
+			/* LBA mode */
+			s->trkinfo[i].addr =
+				util_bswap32(p->abs_addr.logical);
+			util_blktomsf(
+				s->trkinfo[i].addr,
+				&s->trkinfo[i].min,
+				&s->trkinfo[i].sec,
+				&s->trkinfo[i].frame,
+				MSF_OFFSET
+			);
+		}
+		else {
+			/* MSF mode */
+			s->trkinfo[i].min = p->abs_addr.msf.min;
+			s->trkinfo[i].sec = p->abs_addr.msf.sec;
+			s->trkinfo[i].frame = p->abs_addr.msf.frame;
+			util_msftoblk(
+				s->trkinfo[i].min,
+				s->trkinfo[i].sec,
+				s->trkinfo[i].frame,
+				&s->trkinfo[i].addr,
+				MSF_OFFSET
+			);
+		}
 
 		if (p->trkno == LEAD_OUT_TRACK ||
 		    s->trkinfo[i-1].trkno == s->last_trk ||
