@@ -6,12 +6,7 @@
  *			[-h serverhost[:port]] [-c cgipath] [-l level]
  *			[-d] command ...
  *
- *   When compiled with -DPATCH_DOMAIN, cddbcmd uses getdomainname()
- *   to get the local host's domain name and appends it to the
- *   local host name.
- *
- *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -31,25 +26,64 @@
  */
 
 #ifndef LINT
-static char *_cddbcmd_c_ident_ = "@(#)cddbcmd.c	6.12 98/04/23";
+static char *_cddbcmd_c_ident_ = "@(#)cddbcmd.c	6.30 00/01/26";
+#endif
+
+#include <sys/types.h>
+
+#ifndef NO_STDLIB_H
+#include <stdlib.h>
+#endif
+#ifndef NO_UNISTD_H
+#include <unistd.h>
 #endif
 
 #include <stdio.h>
 #include <ctype.h>
-#include <sys/types.h>
-#include <string.h>
-#include <signal.h>
 #include <errno.h>
+#include <signal.h>
 #include <pwd.h>
 #include <sys/socket.h>
 #include <sys/time.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 #include <netdb.h>
+
+#ifdef BSDCOMPAT
+#include <strings.h>
+
+#ifndef strchr
+#define strchr	index
+#endif
+#ifndef strrchr
+#define strrchr	rindex
+#endif
+
+#else
+#include <string.h>
+#endif
+
 #ifdef _AIX
 #include <sys/select.h>
 #endif
+
+#if !defined(USE_TERMIOS) && !defined(USE_TERMIO) && !defined(USE_SGTTY)
+#define USE_TERMIOS     /* Make USE_TERMIOS the default if not specified */
+#endif
+
+#ifdef USE_TERMIOS
+#include <termios.h>
+#endif
+#ifdef USE_TERMIO
+#include <termio.h>
+#endif
+#ifdef USE_SGTTY
+#include <sgtty.h>
+#endif
+
 #include "common_d/patchlevel.h"
 
+
 #define CLIENT		"xmcd"
 #define CGIPATH		"/~cddb/cddb.cgi"
 #define DFLHOST		"cddb.cddb.com"
@@ -69,16 +103,39 @@ static char *_cddbcmd_c_ident_ = "@(#)cddbcmd.c	6.12 98/04/23";
 #define PORT_CDDBP	888
 #define PORT_HTTP	80
 
-#define CDDB_PROTO	3
+#define CDDBP_PROTO_LVL	4
 
-extern char		*getenv();
+#ifdef SOCKS
+#define SOCKSINIT(x)	SOCKSinit(x)
+#define SOCKET		socket
+#define CONNECT		Rconnect
+#else
+#define SOCKSINIT(x)
+#define SOCKET		socket
+#define CONNECT		connect
+#endif  /* SOCKS */
+
+extern char		*ttyname(),
+			*getenv();
+extern int		getopt();
 extern int		optind;
 extern char		*optarg;
 
-int			debug;
-int			proto;
-char			user[MAXUSERNAMELEN];
-char			myhost[MAXHOSTNAMELEN];
+int			server_connect(char *, int, char *, int, char *,
+				       int, char *, int);
+
+int			debug,
+			proto,
+			serverport,
+			proxyport,
+			cddbproto;
+char			user[MAXUSERNAMELEN],
+			myhost[MAXHOSTNAMELEN],
+			*auth_buf,
+			*serverhost,
+			*proxyhost,
+			*cgipath,
+			cmd[CMD_BUFLEN];
 
 
 /*
@@ -170,6 +227,309 @@ my_fputs(char *buf, FILE *fp)
 }
 
 
+/*
+ * do_echo
+ *	Turn off/on echo mode.  It is assumed that the program starts
+ *	with echo mode enabled.
+ *
+ * Args:
+ *	fp - File pointer
+ *	mode - 1 = enable echo, 0 = disable echo
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+do_echo(FILE *fp, int mode)
+{
+	int			fd = fileno(fp);
+#ifdef USE_TERMIOS
+	struct termios		tio;
+
+	(void) tcgetattr(fd, &tio);
+
+	if (mode)
+		tio.c_lflag |= ECHO;
+	else
+		tio.c_lflag &= ~ECHO;
+
+	(void) tcsetattr(fd, TCSADRAIN, &tio);
+#endif	/* USE_TERMIOS */
+
+#ifdef USE_TERMIO
+	struct termio		tio;
+
+	(void) ioctl(fd, TCGETA, &tio);
+
+	if (mode)
+		tio.c_lflag |= ECHO;
+	else
+		tio.c_lflag &= ~ECHO;
+
+	(void) ioctl(fd, TCSETAW, &tio);
+#endif	/* USE_TERMIO */
+
+#ifdef USE_SGTTY
+	struct sgttyb		tio;
+
+	(void) ioctl(fd, TIOCGETP, &tio);
+
+	if (mode)
+		tio.sg_flags |= ECHO;
+	else
+		tio.sg_flags &= ~ECHO;
+
+	(void) ioctl(fd, TIOCSETP, &tio);
+#endif	/* USE_SGTTY */
+}
+
+
+/*
+ * Data used by b64encode
+ */
+char	b64map[] = {
+	'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
+	'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
+	'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
+	'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
+	'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7',
+	'8', '9', '+', '/'
+};
+
+#define B64_PAD		'='
+
+
+/*
+ * b64encode
+ *	Base64 encoding function
+ *
+ * Args:
+ *	ibuf - Input buffer pointer
+ &	len - Length of data in input buffer
+ &	obuf - Output buffer pointer
+ *	brklines - Whether the encoded output should be broken
+ *		   up into multiple lines (i.e., newlines are
+ *		   inserted every 64 characters in accordance
+ *		   to RFC 1521
+ *
+ *	It is assumed that the caller has pre-allocated an output
+ *	buffer large enough to hold the encoded data, which should
+ *	be 33% larger than the input data length (i.e., for every
+ *	three bytes of input, there will be four bytes of output).
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+b64encode(unsigned char *ibuf, int len, unsigned char *obuf, int brklines)
+{
+	int		i, j, k, n,
+			c[4];
+	unsigned char	sbuf[4];
+
+	for (i = k = 0; (i + 3) <= len; i += 3, ibuf += 3) {
+		c[0] = ((int) ibuf[0] >> 2);
+		c[1] = ((((int) ibuf[0] & 0x03) << 4) |
+			(((int) ibuf[1] & 0xf0) >> 4));
+		c[2] = ((((int) ibuf[1] & 0x0f) << 2) |
+			(((int) ibuf[2] & 0xc0) >> 6));
+		c[3] = ((int) ibuf[2] & 0x3f);
+
+		for (j = 0; j < 4; j++)
+			*obuf++ = b64map[c[j]];
+
+		if (brklines && ++k == 16) {
+			k = 0;
+			*obuf++ = '\n';
+		}
+	}
+
+	if (i < len) {
+		n = len - i;
+		(void) strncpy((char *) sbuf, (char *) ibuf, n);
+		for (j = n; j < 3; j++)
+			sbuf[j] = (unsigned char) 0;
+
+		n++;
+		ibuf = sbuf;
+		c[0] = ((int) ibuf[0] >> 2);
+		c[1] = ((((int) ibuf[0] & 0x03) << 4) |
+			(((int) ibuf[1] & 0xf0) >> 4));
+		c[2] = ((((int) ibuf[1] & 0x0f) << 2) |
+			(((int) ibuf[2] & 0xc0) >> 6));
+		c[3] = ((int) ibuf[2] & 0x3f);
+
+		for (j = 0; j < 4; j++)
+			*obuf++ = (j < n) ? b64map[c[j]] : B64_PAD;
+
+		if (brklines && ++k == 16)
+			*obuf++ = '\n';
+	}
+
+	if (brklines)
+		*obuf++ = '\n';
+
+	*obuf = '\0';
+}
+
+
+/*
+ * http_set_auth
+ *	Set proxy authorization user name and password.
+ *
+ * Args:
+ *	name - Proxy user name
+ *	passwd - Proxy password
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+http_set_auth(char *name, char *passwd)
+{
+	int	i,
+		j,
+		n1,
+		n2;
+	char	*buf;
+
+	if (name == NULL || passwd == NULL)
+		return;
+
+	i = strlen(name);
+	j = strlen(passwd);
+	n1 = i + j + 2;
+	n2 = (n1 * 4 / 3) + 8;
+
+	if ((buf = (char *) malloc(n1)) == NULL) {
+		fprintf(stderr, "Out of memory.\n");
+		exit(1);
+	}
+
+	(void) sprintf(buf, "%s:%s", name, passwd);
+
+	(void) memset(name, 0, i);
+	(void) memset(passwd, 0, j);
+
+	if (auth_buf != NULL)
+		free(auth_buf);
+
+	if ((auth_buf = (char *) malloc(n2)) == NULL) {
+		fprintf(stderr, "Out of memory.\n");
+		exit(1);
+	}
+
+	/* Base64 encode the name/password pair */
+	b64encode(
+		(unsigned char *) buf,
+		strlen(buf),
+		(unsigned char *) auth_buf,
+		0
+	);
+
+	(void) memset(buf, 0, n1);
+	free(buf);
+}
+
+
+/*
+ * get_proxy_auth
+ *	Ask the user to enter user and password for proxy authorization.
+ *
+ * Args:
+ *	None.
+ *
+ * Return:
+ *	0 on success
+ *	errno on failure
+ */
+int
+get_proxy_auth(int retrycnt)
+{
+	FILE		*ttyfp;
+	char		*ttypath,
+			*name,
+			*pass;
+	char		input[64];
+
+	if ((ttypath = ttyname(2)) == NULL)
+		ttypath = "/dev/tty";
+	if ((ttyfp = fopen(ttypath, "r+")) == NULL) {
+		if (debug)
+			fprintf(stderr, "Cannot open /dev/tty.\n");
+		return EFAULT;
+	}
+
+	/* Change to unbuffered tty I/O */
+	setbuf(ttyfp, NULL);
+
+	if (retrycnt == 0)
+		(void) fprintf(ttyfp, "Proxy Authorization is required.\n");
+	else {
+		(void) fprintf(ttyfp, "Proxy Authorization failed.");
+		if (retrycnt < 3)
+			(void) fprintf(ttyfp, "  Try again.\n");
+		else {
+			(void) fprintf(ttyfp, "  Too many tries.\n\n");
+			(void) fclose(ttyfp);
+			return EFAULT;
+		}
+	}
+
+	(void) fprintf(ttyfp, "%s\n\n",
+		"Please enter your proxy user name and password.");
+
+	/* Get user name */
+	(void) fprintf(ttyfp, "Username: ");
+	if (fgets(input, 63, ttyfp) == NULL) {
+		(void) fprintf(ttyfp, "\n");
+		return 0;
+	}
+	input[strlen(input)-1] = '\0';
+	if (input[0] == '\0')
+		return 0;
+
+	name = (char *) malloc(strlen(input) + 1);
+	if (name == NULL) {
+		fprintf(stderr, "Out of memory.\n");
+		exit(1);
+	}
+	(void) strcpy(name, input);
+	(void) memset(input, 0, 64);
+
+	/* Get password */
+	(void) fprintf(ttyfp, "Password: ");
+	do_echo(ttyfp, 0);
+	if (fgets(input, 63, ttyfp) == NULL) {
+		do_echo(ttyfp, 1);
+		(void) fprintf(ttyfp, "\n");
+		return 0;
+	}
+	do_echo(ttyfp, 1);
+	input[strlen(input)-1] = '\0';
+	if (input[0] == '\0')
+		pass = NULL;
+	else {
+		pass = (char *) malloc(strlen(input) + 1);
+		if (pass == NULL) {
+			fprintf(stderr, "Out of memory.\n");
+			exit(1);
+		}
+		(void) strcpy(pass, input);
+		(void) memset(input, 0, 64);
+	}
+
+	http_set_auth(name, pass == NULL ? "" : pass);
+
+	free(name);
+	free(pass);
+
+	(void) fprintf(ttyfp, "\n\n");
+	(void) fclose(ttyfp);
+	return 0;
+}
+
+
 /*
  * http_cmd
  *	Perform user-specified operation via http
@@ -180,9 +540,11 @@ my_fputs(char *buf, FILE *fp)
  *	serverhost - CDDB server host name
  *	serverport - CDDB server TCP port number
  *	cgipath - The CGI path
+ *	cddbproto - CDDB protocol level
  *	fp - socket stream
  *	buf - command and response buffer
  *	len - command and response buffer length
+ *	depth - recursion depth
  *
  * Return:
  *	0 on success
@@ -195,13 +557,16 @@ http_cmd(
 	char	*serverhost,
 	int	serverport,
 	char	*cgipath,
+	int	cddbproto,
 	FILE	*fp,
 	char	*buf,
-	int	len
+	int	len,
+	int	depth
 )
 {
 	int	n,
-		code;
+		code,
+		ret;
 	char	*p,
 		*q,
 		str[CMD_BUFLEN],
@@ -228,12 +593,21 @@ http_cmd(
 	(void) sprintf(buf,
 	       "GET %s?cmd=%s&hello=%s+%s+%s+v%s%sPL%d&proto=%d HTTP/1.0\r\n",
 		       str2, str, user, myhost,
-		       CLIENT, VERSION, VERSION_EXT, PATCHLEVEL, CDDB_PROTO);
+		       CLIENT, VERSION, VERSION_EXT,
+		       PATCHLEVEL, cddbproto);
 	n = strlen(buf);
 	if (my_fputs(buf, fp) < n)
 		return (errno);
 
+#ifdef NO_MASQUERADE
 	(void) sprintf(buf, "User-Agent: %s/%s%sPL%d\r\n",
+#else
+	/* Believe it or not, this is how MS Internet Explorer does it.
+	 * so don't blame me...
+	 */
+	(void) sprintf(buf, "User-Agent: %s (compatible; %s %s%sPL%d)\r\n",
+		       "Mozilla/4.7",
+#endif
 		       CLIENT, VERSION, VERSION_EXT, PATCHLEVEL);
 	n = strlen(buf);
 	if (my_fputs(buf, fp) < n)
@@ -249,6 +623,14 @@ http_cmd(
 	if (my_fputs(buf, fp) < n)
 		return (errno);
 
+	if (proxyhost != NULL && auth_buf != NULL) {
+		(void) sprintf(buf, "Proxy-Authorization: Basic %s\r\n",
+			       auth_buf);
+		n = strlen(buf);
+		if (my_fputs(buf, fp) < n)
+			return (errno);
+	}
+
 	(void) strcpy(buf, "\r\n");
 	n = strlen(buf);
 	if (my_fputs(buf, fp) < n)
@@ -257,11 +639,59 @@ http_cmd(
 	/* Check HTTP banner */
 	if (my_fgets(buf, len, fp) == NULL)
 		return EFAULT;
-	if (strncmp(buf, "HTTP", 4) != 0) {
+	if (strncmp(buf, "HTTP/", 5) != 0) {
 		(void) fprintf(stderr, "HTTP protocol error.\n");
 		return EFAULT;
 	}
 
+	/* Check HTTP status code */
+	q = strchr(buf, ' ');
+	if (q != NULL && isdigit(*(q+1))) {
+		int	httpcode,
+			class;
+
+		httpcode = atoi(q+1);
+		class = httpcode / 100;
+
+		switch (class) {
+		case 2:	/* Successful */
+			break;
+		case 3:	/* Redirection */
+			(void) fprintf(stderr,
+				       "HTTP redirection status %d\n",
+				       httpcode);
+			break;
+		case 4:	/* Client error */
+			if (httpcode == 407 && proxyhost != NULL) {
+				/* Proxy authorization failed */
+				if ((ret = get_proxy_auth(depth)) != 0)
+					return (ret);
+				
+				/* Close the socket */
+				fclose(fp);
+
+				/* Recurse */
+				return (server_connect(proxyhost, proxyport,
+						       serverhost, serverport,
+						       cgipath, cddbproto,
+						       cmd, ++depth));
+			}
+			else
+				(void) fprintf(stderr,
+					       "HTTP client error: status %d\n",
+					       httpcode);
+			break;
+		case 5:	/* Server error */
+			(void) fprintf(stderr,
+				       "HTTP server error: status %d\n",
+				       httpcode);
+			break;
+		default:
+			(void) fprintf(stderr, "HTTP status %d\n", httpcode);
+			break;
+		}
+	}
+
 	/* Skip the rest of the HTTP junk */
 	while (my_fgets(buf, len, fp) != NULL) {
 		if (isdigit(buf[0]) && isdigit(buf[1]) && isdigit(buf[2]) &&
@@ -601,7 +1031,8 @@ server_connect(
 	int	serverport,
 	char	*cgipath,
 	int	cddbproto,
-	char	*cmd
+	char	*cmd,
+	int	depth
 )
 {
 	struct hostent		*hp;
@@ -646,14 +1077,14 @@ server_connect(
 	}
 
 	/* Open socket */
-	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+	if ((fd = SOCKET(AF_INET, SOCK_STREAM, 0)) < 0) {
 		ret = errno;
 		perror("Socket failed");
 		return (ret);
 	}
 
 	/* Connect to server */
-	if (connect(fd, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
+	if (CONNECT(fd, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
 		ret = errno;
 		perror("Connect failed");
 		close(fd);
@@ -706,7 +1137,8 @@ server_connect(
 
 	case PROTO_HTTP:
 		if ((ret = http_cmd(cmd, proxyhost, serverhost, serverport,
-				    cgipath, fp, buf, CMD_BUFLEN)) != 0) {
+				    cgipath, cddbproto, fp, buf, CMD_BUFLEN,
+				    depth)) != 0) {
 			free(buf);
 			return (ret);
 		}
@@ -727,17 +1159,16 @@ server_connect(
  *	Print command line usage help
  *
  * Args:
- *	progname - The program name
+ *	None.
  *
  * Return:
  *	0 on success
  *	errno on failure
  */
 void
-usage(char *progname)
+usage(void)
 {
-	(void) fprintf(stderr, "Usage: %s %s\n\t\t%s\n\t\t%s\n",
-		       progname,
+	(void) fprintf(stderr, "Usage: cddbcmd\t%s\n\t\t%s\n\t\t%s\n",
 		       "[-m <cddbp | http>] [-p proxyhost[:port]]",
 		       "[-h serverhost[:port]] [-c cgipath] [-l level]",
 		       "[-d] command ...");
@@ -753,24 +1184,19 @@ main(int argc, char **argv)
 {
 	int		i,
 			n,
-			serverport,
-			proxyport,
-			cddbproto,
 			ret,
 			c;
 	uid_t		ouid;
 	struct passwd	*pw;
+	struct hostent	*he;
 	char		*cp,
-			*serverhost,
-			*proxyhost,
-			*cgipath,
-			cmd[CMD_BUFLEN];
+			**ap;
 
 	serverhost = DFLHOST;
 	proxyhost = NULL;
 	proto = PROTO_CDDBP;
 	cgipath = CGIPATH;
-	cddbproto = CDDB_PROTO;
+	cddbproto = CDDBP_PROTO_LVL;
 	serverport = -1;
 	proxyport = PORT_HTTP;
 
@@ -784,7 +1210,7 @@ main(int argc, char **argv)
 			else {
 				(void) fprintf(stderr,
 					"Invalid arg used with -%c\n", c);
-				usage(argv[0]);
+				usage();
 				exit(1);
 			}
 			break;
@@ -818,19 +1244,19 @@ main(int argc, char **argv)
 			break;
 
 		case '?':
-			usage(argv[0]);
+			usage();
 			exit(0);
 
 		default:
 			(void) fprintf(stderr, "Unknown option -%c\n", c);
-			usage(argv[0]);
+			usage();
 			exit(1);
 		}
 	}
 
 	if (argc < (optind + 1)) {
 		(void) fprintf(stderr, "No server command specified.\n");
-		usage(argv[0]);
+		usage();
 		exit(1);
 	}
 
@@ -870,18 +1296,29 @@ main(int argc, char **argv)
 			"proxyhost", (proxyhost == NULL) ? "-" : proxyhost,
 			"proxyport", proxyport);
 
-	if (gethostname(myhost, MAXHOSTNAMELEN) < 0)
+	/* Try to get the host name */
+	if (gethostname(myhost, MAXHOSTNAMELEN-1) < 0 ||
+	    (he = gethostbyname(myhost)) == NULL) {
 		(void) strcpy(myhost, "unknown");
-
-#ifdef PATCH_DOMAIN
-	if ((cp = (char *) strchr(myhost, '@')) == NULL)
-		cp = myhost + strlen(myhost);
-
-	if (getdomainname(cp+1, MAXHOSTNAMELEN - strlen(myhost) - 1) == 0)
-		*cp = '@';
-	else
-		*cp = '\0';
-#endif
+	}
+	else {
+		/* Look for a a fully-qualified hostname
+		 * (with domain)
+		 */
+		if (strchr(he->h_name, '.') != NULL) {
+			(void) strncpy(myhost, he->h_name, MAXHOSTNAMELEN-1);
+		}
+		else {
+			for (ap = he->h_aliases; *ap != NULL; ap++) {
+				if (strchr(*ap, '.') != NULL) {
+				    (void) strncpy(myhost, *ap,
+						   MAXHOSTNAMELEN-1);
+				    break;
+				}
+			}
+		}
+	}
+	myhost[MAXHOSTNAMELEN-1] = '\0';
 
 	ouid = getuid();
 
@@ -896,9 +1333,11 @@ main(int argc, char **argv)
 
 	user[MAXUSERNAMELEN-1] = '\0';
 
+	SOCKSINIT("xmcd");
+
 	ret = server_connect(proxyhost, proxyport,
 			     serverhost, serverport,
-			     cgipath, cddbproto, cmd);
+			     cgipath, cddbproto, cmd, 0);
 
 	exit((ret == 0) ? 0 : errno);
 	/*NOTREACHED*/
