@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __CDFUNC_H__
 
 #ifndef LINT
-static char *_cdfunc_h_ident_ = "@(#)cdfunc.h	6.20 98/04/23";
+static char *_cdfunc_h_ident_ = "@(#)cdfunc.h	6.30 99/12/27";
 #endif
 
 
@@ -41,16 +41,17 @@ static char *_cdfunc_h_ident_ = "@(#)cdfunc.h	6.20 98/04/23";
 #define CURS_KEYPAD	0x0002
 #define CURS_OPTIONS	0x0004
 #define CURS_DBPROG	0x0008
-#define CURS_WWWWARP	0x0010
+#define CURS_DLIST	0x0010
 #define CURS_DBEXTD	0x0020
 #define CURS_DBEXTT	0x0040
 #define CURS_DIRSEL	0x0080
 #define CURS_LINKSEL	0x0100
-#define CURS_HELP	0x0200
-#define CURS_ALL	( CURS_MAIN   | CURS_KEYPAD  | CURS_OPTIONS |	\
-			  CURS_DBPROG | CURS_WWWWARP | CURS_DBEXTD  |	\
-			  CURS_DBEXTT |	CURS_DIRSEL  | CURS_LINKSEL |	\
-			  CURS_HELP )
+#define CURS_WWWWARP	0x0200
+#define CURS_HELP	0x0400
+#define CURS_ALL	( CURS_MAIN    | CURS_KEYPAD | CURS_OPTIONS |	\
+			  CURS_DBPROG  | CURS_DLIST  | CURS_DBEXTD  |	\
+			  CURS_DBEXTT  | CURS_DIRSEL | CURS_LINKSEL |	\
+			  CURS_WWWWARP | CURS_HELP )
 
 /* Keypad modes */
 #define KPMODE_DISC	0
@@ -86,14 +87,17 @@ extern int	unscale_warp(curstat_t *, int, int);
 extern long	cd_timeout(word32_t, void (*)(), byte_t *);
 extern void	cd_untimeout(long);
 extern void	cd_beep(void);
+extern void	cd_dialog_setpos(Widget);
 extern void	cd_info_popup(char *, char *);
 extern void	cd_info_popup_auto(char *, char *);
+extern void	cd_info_popdown(byte_t *);
 extern void	cd_warning_popup(char *, char *);
 extern void	cd_fatal_popup(char *, char *);
 extern void	cd_confirm_popup(char *, char *, XtCallbackProc, XtPointer,
 			XtCallbackProc, XtPointer);
+extern void	cd_working_popup(char *, char *, XtCallbackProc, XtPointer);
+extern void	cd_working_popdown(void);
 
-extern void	cd_playprog(curstat_t *);
 extern void	cd_init(curstat_t *);
 extern void	cd_start(curstat_t *);
 extern void	cd_icon(curstat_t *, bool_t);
@@ -101,6 +105,7 @@ extern void	cd_halt(curstat_t *);
 extern void	cd_quit(curstat_t *);
 
 extern void	cd_busycurs(bool_t, int);
+extern char	*cd_hostname(void);
 extern void	onsig(int);
 
 
@@ -128,6 +133,8 @@ extern void	cd_keypad_popdown(Widget, XtPointer, XtPointer);
 extern void	cd_keypad_mode(Widget, XtPointer, XtPointer);
 extern void	cd_keypad_num(Widget, XtPointer, XtPointer);
 extern void	cd_keypad_clear(Widget, XtPointer, XtPointer);
+extern void	cd_keypad_dsbl_modes_yes(Widget, XtPointer, XtPointer);
+extern void	cd_keypad_dsbl_modes_no(Widget, XtPointer, XtPointer);
 extern void	cd_keypad_enter(Widget, XtPointer, XtPointer);
 extern void	cd_warp(Widget, XtPointer, XtPointer);
 extern void	cd_options_popup(Widget, XtPointer, XtPointer);
@@ -139,13 +146,15 @@ extern void	cd_balance(Widget, XtPointer, XtPointer);
 extern void	cd_balance_center(Widget, XtPointer, XtPointer);
 extern void	cd_about(Widget, XtPointer, XtPointer);
 extern void	cd_help_popup(Widget, XtPointer, XtPointer);
-extern void	cd_help_popdown(Widget, XtPointer, XtPointer);
-extern void	cd_warning_popdown(Widget, XtPointer, XtPointer);
-extern void	cd_fatal_popdown(Widget, XtPointer, XtPointer);
+extern void	cd_help_cancel(Widget, XtPointer, XtPointer);
+extern void	cd_info_ok(Widget, XtPointer, XtPointer);
+extern void	cd_warning_ok(Widget, XtPointer, XtPointer);
+extern void	cd_fatal_ok(Widget, XtPointer, XtPointer);
 extern void	cd_rmcallback(Widget, XtPointer, XtPointer);
 extern void	cd_shell_focus_chg(Widget, XtPointer, XtPointer);
 extern void	cd_exit(Widget, XtPointer, XtPointer);
 extern void	cd_tooltip_cancel(Widget, XtPointer, XtPointer);
+extern void	cd_not_implemented(Widget, XtPointer, XtPointer);
 
 /* Event handlers */
 extern void	cd_focus_chg(Widget, XtPointer, XEvent *);
