@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __DBPROG_H__
 
 #ifndef LINT
-static char *_dbprog_h_ident_ = "@(#)dbprog.h	6.22 98/05/19";
+static char *_dbprog_h_ident_ = "@(#)dbprog.h	6.36 99/12/27";
 #endif
 
 
@@ -31,15 +31,18 @@ static char *_dbprog_h_ident_ = "@(#)dbprog.h	6.22 98/05/19";
 #define UNDEF_STR	"??"
 #define PGM_SEPCHAR	','
 #define EXTT_FLGSTR	"*"
+
 #define TIME_TOTAL	1
 #define TIME_TRACK	2
+
 #define DIRSEL_SAVE	1
 #define DIRSEL_LINK	2
 
-#define WWWWARP_1	0
-#define WWWWARP_2	1
-#define WWWWARP_3	2
-#define WWWWARP_SRCHENG	3
+#define DLIST_HIST	1
+#define DLIST_CHGR	2
+
+#define HISTLIST_FMT	"%.3s %02d %02d:%02d  %.127s"
+#define CHGRLIST_FMT	"Disc %-4d  %.127s"
 
 
 /* Public functions */
@@ -48,14 +51,19 @@ extern void	dbprog_curtrkupd(curstat_t *);
 extern void	dbprog_progclear(curstat_t *);
 extern void	dbprog_dbclear(curstat_t *, bool_t);
 extern void	dbprog_dbget(curstat_t *);
+extern void	dbprog_chgr_scan_stop(curstat_t *);
 extern void	dbprog_init(curstat_t *);
 extern bool_t	dbprog_chgsave(curstat_t *);
 extern char	*dbprog_curdtitle(curstat_t *);
 extern char	*dbprog_curttitle(curstat_t *);
+extern cddb_incore_t
+		*dbprog_curdb(curstat_t *);
+extern int	dbprog_curseltrk(curstat_t *);
 extern bool_t	dbprog_pgm_parse(curstat_t *);
 
 /* Callback functions */
 extern void	dbprog_popup(Widget, XtPointer, XtPointer);
+extern void	dbprog_rmtdsbl(Widget, XtPointer, XtPointer);
 extern void	dbprog_txtline_verify(Widget, XtPointer, XtPointer);
 extern void	dbprog_dtitle_new(Widget, XtPointer, XtPointer);
 extern void	dbprog_trklist_play(Widget, XtPointer, XtPointer);
@@ -66,7 +74,7 @@ extern void	dbprog_pgmseq_verify(Widget, XtPointer, XtPointer);
 extern void	dbprog_pgmseq_txtchg(Widget, XtPointer, XtPointer);
 extern void	dbprog_addpgm(Widget, XtPointer, XtPointer);
 extern void	dbprog_clrpgm(Widget, XtPointer, XtPointer);
-extern void	dbprog_send(Widget, XtPointer, XtPointer);
+extern void	dbprog_submit(Widget, XtPointer, XtPointer);
 extern void	dbprog_save(Widget, XtPointer, XtPointer);
 extern void	dbprog_load(Widget, XtPointer, XtPointer);
 extern void	dbprog_link(Widget, XtPointer, XtPointer);
@@ -81,6 +89,7 @@ extern void	dbprog_extd_clear(Widget, XtPointer, XtPointer);
 extern void	dbprog_extd_cancel(Widget, XtPointer, XtPointer);
 extern void	dbprog_extt_prev(Widget, XtPointer, XtPointer);
 extern void	dbprog_extt_next(Widget, XtPointer, XtPointer);
+extern void	dbprog_extt_autotrk(Widget, XtPointer, XtPointer);
 extern void	dbprog_extt_ok(Widget, XtPointer, XtPointer);
 extern void	dbprog_extt_clear(Widget, XtPointer, XtPointer);
 extern void	dbprog_extt_cancel(Widget, XtPointer, XtPointer);
@@ -98,10 +107,16 @@ extern void	dbprog_auth_name(Widget, XtPointer, XtPointer);
 extern void	dbprog_auth_password_vfy(Widget, XtPointer, XtPointer);
 extern void	dbprog_auth_ok(Widget, XtPointer, XtPointer);
 extern void	dbprog_auth_cancel(Widget, XtPointer, XtPointer);
-extern void	dbprog_wwwwarp_select(Widget, XtPointer, XtPointer);
-extern void	dbprog_wwwwarp_cancel(Widget, XtPointer, XtPointer);
-extern void	dbprog_wwwwarp_go(Widget, XtPointer, XtPointer);
-extern void	dbprog_wwwwarp(Widget, XtPointer, XtPointer);
-extern void	dbprog_srcheng_sel(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_cancel(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_mode(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_select(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_show(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_goto(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_delete(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_delall(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_delall_yes(Widget, XtPointer, XtPointer);
+extern void	dbprog_dlist_rescan(Widget, XtPointer, XtPointer);
+extern void	dbprog_scan_stop_btn(Widget, XtPointer, XtPointer);
 
 #endif	/* __DBPROG_H__ */
