@@ -1,7 +1,7 @@
 /*
- *   xmcd - Motif(tm) CD Audio Player
+ *   Xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,20 +20,21 @@
  *
  */
 #ifndef LINT
-static char *_geom_c_ident_ = "@(#)geom.c	6.28 98/05/19";
+static char *_geom_c_ident_ = "@(#)geom.c	6.52 00/01/15";
 #endif
 
 #include "common_d/appenv.h"
+#include "common_d/util.h"
 #include "xmcd_d/xmcd.h"
 #include "xmcd_d/widget.h"
 #include "xmcd_d/geom.h"
 
 
-#define WM_FUDGE	15		/* WM decoration fudge factor */
 #define WLIST_SIZE	30		/* Widget list size */
 
 
 extern appdata_t	app_data;
+extern FILE		*errfp;
 
 STATIC int		main_mode;	/* Main window mode */
 
@@ -56,13 +57,6 @@ STATIC int		main_mode;	/* Main window mode */
 STATIC void
 geom_normal_force(widgets_t *m)
 {
-	/* Set normal mode main window size */
-	XtVaSetValues(m->toplevel,
-		XmNwidth, app_data.normal_width,
-		XmNheight, app_data.normal_height,
-		NULL
-	);
-
 	XtVaSetValues(m->main.mode_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -330,15 +324,15 @@ geom_normal_force(widgets_t *m)
 		XmNbottomPosition, BOTTOM_KEYPAD,
 		NULL
 	);
-	XtVaSetValues(m->main.help_btn,
+	XtVaSetValues(m->main.wwwwarp_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
 		XmNtopAttachment, XmATTACH_POSITION,
 		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_HELP,
-		XmNrightPosition, RIGHT_HELP,
-		XmNtopPosition, TOP_HELP,
-		XmNbottomPosition, BOTTOM_HELP,
+		XmNleftPosition, LEFT_WWWWARP,
+		XmNrightPosition, RIGHT_WWWWARP,
+		XmNtopPosition, TOP_WWWWARP,
+		XmNbottomPosition, BOTTOM_WWWWARP,
 		NULL
 	);
 	XtVaSetValues(m->main.level_scale,
@@ -478,13 +472,6 @@ geom_normal_force(widgets_t *m)
 STATIC void
 geom_basic_force(widgets_t *m)
 {
-	/* Set basic mode main window size */
-	XtVaSetValues(m->toplevel,
-		XmNwidth, app_data.basic_width,
-		XmNheight, app_data.basic_height,
-		NULL
-	);
-
 	XtVaSetValues(m->main.mode_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -613,22 +600,30 @@ geom_main_force(widgets_t *m)
 {
 	Display		*display = XtDisplay(m->toplevel);
 	int		screen = DefaultScreen(display),
-			i;
+			i,
+			x_delta,
+			y_delta;
 	Dimension	width,
 			height,
+			cwidth,
+			cheight,
 			swidth,
 			sheight;
-	Position	x,
-			y,
+	Position	abs_x,
+			abs_y,
 			new_x,
-			new_y;
-	static Widget	wlist[WLIST_SIZE];
+			new_y,
+			wmcorr_x,
+			wmcorr_y;
 	static int	wlist_size = 0;
+	static Widget	wlist[WLIST_SIZE];
 	static bool_t	first = TRUE;
 
-	if (first) {
-		first = FALSE;
+	swidth = (Dimension) XDisplayWidth(display, screen);
+	sheight = (Dimension) XDisplayHeight(display, screen);
+	XtTranslateCoords(m->toplevel, 0, 0, &abs_x, &abs_y);
 
+	if (first) {
 		/* Set up the list of main window widgets that gets
 		 * managed and unmanaged when switching modes.
 		 */
@@ -645,7 +640,7 @@ geom_main_force(widgets_t *m)
 		wlist[i++] = m->main.dbprog_btn;
 		wlist[i++] = m->main.dbprog_btn;
 		wlist[i++] = m->main.options_btn;
-		wlist[i++] = m->main.help_btn;
+		wlist[i++] = m->main.wwwwarp_btn;
 		wlist[i++] = m->main.time_btn;
 		wlist[i++] = m->main.ab_btn;
 		wlist[i++] = m->main.sample_btn;
@@ -657,49 +652,222 @@ geom_main_force(widgets_t *m)
 		wlist[i++] = m->main.rew_btn;
 		wlist[i++] = m->main.ff_btn;
 		wlist_size = i;
+
+		wmcorr_x = wmcorr_y = 0;
+
+		/* Current window width */
+		switch (main_mode) {
+		case MAIN_BASIC:
+			cwidth = (Dimension) app_data.basic_width;
+			cheight = (Dimension) app_data.basic_height;
+			break;
+		case MAIN_NORMAL:
+		default:
+			cwidth = (Dimension) app_data.normal_width;
+			cheight = (Dimension) app_data.normal_height;
+			break;
+		}
+	}
+	else {
+		/* Hack: Get window manager correction factor.
+		 * This works around the problem with some window
+		 * managers where setting the XmNx and XmNy positions
+		 * of the toplevel does not take into account of the
+		 * window manager decorations.
+		 */
+		XtVaSetValues(m->toplevel,
+			XmNx, abs_x + 1,
+			XmNy, abs_y + 1,
+			NULL
+		);
+		for (i = 0; i < 10; i++)
+			event_loop(0);
+
+		XtTranslateCoords(m->toplevel, 0, 0, &new_x, &new_y);
+
+		wmcorr_x = new_x - abs_x - 1;
+		wmcorr_y = new_y - abs_y - 1;
+
+		XtVaSetValues(m->toplevel,
+			XmNx, abs_x - wmcorr_x,
+			XmNy, abs_y - wmcorr_y,
+			NULL
+		);
+
+		DBGPRN(errfp,
+			"Window manager correction factor: (%d,%d)\n",
+			wmcorr_x, wmcorr_y);
+
+		/* Get current window width */
+		XtVaGetValues(m->toplevel,
+			XmNwidth, &cwidth,
+			XmNheight, &cheight,
+			NULL
+		);
 	}
 
-	if (main_mode == MAIN_NORMAL) {
-		XtManageChildren((WidgetList) wlist, wlist_size);
-		geom_normal_force(m);
-	}
-	else if (main_mode == MAIN_BASIC) {
+	switch (main_mode) {
+	case MAIN_BASIC:
+		if (first) {
+			/* HACK: on startup, if the mode is basic,
+			 * set to normal first so that all widgets
+			 * get properly managed.
+			 */
+			geom_normal_force(m);
+		}
+		width = (Dimension) app_data.basic_width;
+		height = (Dimension) app_data.basic_height;
+		x_delta = (int) (cwidth - width);
+		y_delta = (int) (cheight - height);
 		XtUnmanageChildren((WidgetList) wlist, wlist_size);
 		geom_basic_force(m);
+		break;
+	case MAIN_NORMAL:
+	default:
+		width = (Dimension) app_data.normal_width;
+		height = (Dimension) app_data.normal_height;
+		x_delta = (int) (width - cwidth);
+		y_delta = (int) (height - cheight);
+		if (!first)
+			XtManageChildren((WidgetList) wlist, wlist_size);
+		geom_normal_force(m);
+		break;
 	}
 
-	/* Make sure the full window will appear entirely within
-	 * the screen boundaries after resizing.  Note that this
-	 * tries to accommodate window manager decorations around
-	 * the window (with the WM_FUDGE factor), but it's not always
-	 * sufficient.
+	/* Set window coordinatess according to the
+	 * modeChangeGravity parameter
 	 */
-	swidth = (Dimension) XDisplayWidth(display, screen);
-	sheight = (Dimension) XDisplayHeight(display, screen);
+	switch (app_data.modechg_grav) {
+	case 1:
+		/* Top edge */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x - (x_delta / 2);
+			new_y = abs_y;
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x + (x_delta / 2);
+			new_y = abs_y;
+		}
+		break;
+	case 2:
+		/* Upper right corner */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x - x_delta;
+			new_y = abs_y;
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x + x_delta;
+			new_y = abs_y;
+		}
+		break;
+	case 3:
+		/* Right edge */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x - x_delta;
+			new_y = abs_y - (y_delta / 2);
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x + x_delta;
+			new_y = abs_y + (y_delta / 2);
+		}
+		break;
+	case 4:
+		/* Lower right corner */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x - x_delta;
+			new_y = abs_y - y_delta;
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x + x_delta;
+			new_y = abs_y + y_delta;
+		}
+		break;
+	case 5:
+		/* Bottom edge */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x - (x_delta / 2);
+			new_y = abs_y - y_delta;
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x + (x_delta / 2);
+			new_y = abs_y + y_delta;
+		}
+		break;
+	case 6:
+		/* Lower left corner */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x;
+			new_y = abs_y - y_delta;
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x;
+			new_y = abs_y + y_delta;
+		}
+		break;
+	case 7:
+		/* Left edge */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x;
+			new_y = abs_y - (y_delta / 2);
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x;
+			new_y = abs_y + (y_delta / 2);
+		}
+		break;
+	case 8:
+		/* Center of window */
+		if (main_mode == MAIN_NORMAL) {
+			new_x = abs_x - (x_delta / 2);
+			new_y = abs_y - (y_delta / 2);
+		}
+		else if (main_mode == MAIN_BASIC) {
+			new_x = abs_x + (x_delta / 2);
+			new_y = abs_y + (y_delta / 2);
+		}
+		break;
+	case 0:
+	default:
+		/* Upper left corner */
+		new_x = abs_x;
+		new_y = abs_y;
+		break;
+	}
 
-	XtVaGetValues(m->toplevel,
-		XmNx, &x,
-		XmNy, &y,
-		XmNwidth, &width,
-		XmNheight, &height,
-		NULL
-	);
+	/* Make sure the window will appear within
+	 * the screen boundaries after resizing.
+	 */
+	if (new_x < wmcorr_x)
+		new_x = wmcorr_x;
+	else if ((Dimension) (new_x + width + wmcorr_x) > swidth)
+		new_x = swidth - width - wmcorr_x;
 
-	new_x = x;
-	new_y = y;
+	if (new_y < wmcorr_y)
+		new_y = wmcorr_y;
+	else if ((Dimension) (new_y + height + wmcorr_x) > sheight)
+		new_y = sheight - height - wmcorr_x;
 
-	if (x < 0)
-		new_x = WM_FUDGE;
-	else if ((Dimension) (x + width) > swidth)
-		new_x = swidth - width - WM_FUDGE;
+	/* Set new main window location and size */
+	if (new_x == abs_x && new_y == abs_y) {
+		/* Set size only */
+		XtVaSetValues(m->toplevel,
+			XmNwidth, width,
+			XmNheight, height,
+			NULL
+		);
+	}
+	else {
+		/* Set location and size */
+		XtVaSetValues(m->toplevel,
+			XmNx, new_x - wmcorr_x,
+			XmNy, new_y - wmcorr_y,
+			XmNwidth, width,
+			XmNheight, height,
+			NULL
+		);
+	}
 
-	if (y < 0)
-		new_y = WM_FUDGE;
-	else if ((Dimension) (y + height) > sheight)
-		new_y = sheight - height - WM_FUDGE;
-
-	if (new_x != x || new_y != y)
-		XtVaSetValues(m->toplevel, XmNx, new_x, XmNy, new_y, NULL);
+	first = FALSE;
 }
 
 
@@ -1096,6 +1264,11 @@ geom_options_force(widgets_t *m)
 		XmNrecomputeSize, False,
 		NULL
 	);
+	XtVaSetValues(m->options.done_exit_btn,
+		XmNheight, 16,
+		XmNrecomputeSize, False,
+		NULL
+	);
 
 	XtVaSetValues(m->options.eject_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
@@ -1371,378 +1544,370 @@ geom_options_force(widgets_t *m)
 STATIC void
 geom_dbprog_force(widgets_t *m)
 {
-	XtVaSetValues(m->dbprog.tottime_lbl,
+	XtVaSetValues(m->dbprog.discid_ind,
 		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_TOTTIMELBL,
-		XmNrightPosition, RIGHT_TOTTIMELBL,
-		XmNtopPosition, TOP_TOTTIMELBL,
-		XmNbottomPosition, BOTTOM_TOTTIMELBL,
+		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_FORM,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_DISCIDIND,
+		XmNtopOffset, TOFF_DISCIDIND,
+		XmNbottomOffset, BOFF_DISCIDIND,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.tottime_ind,
 		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_TOTTIMEIND,
-		XmNrightPosition, RIGHT_TOTTIMEIND,
-		XmNtopPosition, TOP_TOTTIMEIND,
-		XmNbottomPosition, BOTTOM_TOTTIMEIND,
+		XmNtopWidget, m->dbprog.discid_ind,
+		XmNtopOffset, TOFF_TOTTIMEIND,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.logo_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_FORM,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_LOGO,
 		XmNrightPosition, RIGHT_LOGO,
-		XmNtopPosition, TOP_LOGO,
-		XmNbottomPosition, BOTTOM_LOGO,
+		XmNtopOffset, TOFF_LOGO,
 		NULL
 	);
 
-	XtVaSetValues(m->dbprog.wwwwarp_btn,
+	XtVaSetValues(m->dbprog.dlist_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_WEB,
-		XmNrightPosition, RIGHT_WEB,
-		XmNtopPosition, TOP_WEB,
-		XmNbottomPosition, BOTTOM_WEB,
+		XmNtopAttachment, XmATTACH_FORM,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_DLIST,
+		XmNrightPosition, RIGHT_DLIST,
+		XmNtopOffset, TOFF_DLIST,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.dtitle_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_DTITLELBL,
 		XmNrightPosition, RIGHT_DTITLELBL,
-		XmNtopPosition, TOP_DTITLELBL,
-		XmNbottomPosition, BOTTOM_DTITLELBL,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.extd_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_EXTDLBL,
-		XmNrightPosition, RIGHT_EXTDLBL,
-		XmNtopPosition, TOP_EXTDLBL,
-		XmNbottomPosition, BOTTOM_EXTDLBL,
+		XmNtopWidget, m->dbprog.logo_lbl,
+		XmNtopOffset, TOFF_DTITLELBL,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.dtitle_txt,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_DTITLE,
 		XmNrightPosition, RIGHT_DTITLE,
-		XmNtopPosition, TOP_DTITLE,
-		XmNbottomPosition, BOTTOM_DTITLE,
+		XmNtopWidget, m->dbprog.dtitle_lbl,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.extd_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_OPPOSITE_WIDGET,
 		XmNleftPosition, LEFT_EXTD,
 		XmNrightPosition, RIGHT_EXTD,
-		XmNtopPosition, TOP_EXTD,
-		XmNbottomPosition, BOTTOM_EXTD,
+		XmNbottomWidget, m->dbprog.dtitle_txt,
+		XmNbottomOffset, BOFF_EXTD,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.extd_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_EXTDLBL,
+		XmNrightPosition, RIGHT_EXTDLBL,
+		XmNbottomWidget, m->dbprog.extd_btn,
+		XmNbottomOffset, BOFF_EXTDLBL,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.dbprog_sep1,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
 		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_DBPROGSEP1,
 		XmNrightPosition, RIGHT_DBPROGSEP1,
-		XmNtopPosition, TOP_DBPROGSEP1,
+		XmNtopWidget, m->dbprog.dtitle_txt,
+		XmNtopOffset, TOFF_DBPROGSEP1,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.trklist_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_TRKLISTLBL,
 		XmNrightPosition, RIGHT_TRKLISTLBL,
-		XmNtopPosition, TOP_TRKLISTLBL,
-		XmNbottomPosition, BOTTOM_TRKLISTLBL,
-		NULL
-	);
-
-	XtVaSetValues(XtParent(m->dbprog.trk_list),
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_TRKLIST,
-		XmNrightPosition, RIGHT_TRKLIST,
-		XmNtopPosition, TOP_TRKLIST,
-		XmNbottomPosition, BOTTOM_TRKLIST,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.radio_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_RADIOLBL,
-		XmNrightPosition, RIGHT_RADIOLBL,
-		XmNtopPosition, TOP_RADIOLBL,
-		XmNbottomPosition, BOTTOM_RADIOLBL,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.radio_frm,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_RADIOBOX,
-		XmNrightPosition, RIGHT_RADIOBOX,
-		XmNtopPosition, TOP_RADIOBOX,
-		XmNbottomPosition, BOTTOM_RADIOBOX,
-		XmNshadowType, XmSHADOW_ETCHED_IN,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.tottime_btn,
-		XmNheight, 16,
-		XmNrecomputeSize, False,
-		NULL
-	);
-	XtVaSetValues(m->dbprog.trktime_btn,
-		XmNheight, 16,
-		XmNrecomputeSize, False,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.discid_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_DISCIDLBL,
-		XmNrightPosition, RIGHT_DISCIDLBL,
-		XmNtopPosition, TOP_DISCIDLBL,
-		XmNbottomPosition, BOTTOM_DISCIDLBL,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.discid_frm,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_DISCIDFRM,
-		XmNrightPosition, RIGHT_DISCIDFRM,
-		XmNtopPosition, TOP_DISCIDFRM,
-		XmNbottomPosition, BOTTOM_DISCIDFRM,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.ttitle_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_TTITLELBL,
-		XmNrightPosition, RIGHT_TTITLELBL,
-		XmNtopPosition, TOP_TTITLELBL,
-		XmNbottomPosition, BOTTOM_TTITLELBL,
-		XmNshadowType, XmSHADOW_ETCHED_IN,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.extt_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_EXTTLBL,
-		XmNrightPosition, RIGHT_EXTTLBL,
-		XmNtopPosition, TOP_EXTTLBL,
-		XmNbottomPosition, BOTTOM_EXTTLBL,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.ttitle_txt,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_TTITLE,
-		XmNrightPosition, RIGHT_TTITLE,
-		XmNtopPosition, TOP_TTITLE,
-		XmNbottomPosition, BOTTOM_TTITLE,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.extt_btn,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_EXTT,
-		XmNrightPosition, RIGHT_EXTT,
-		XmNtopPosition, TOP_EXTT,
-		XmNbottomPosition, BOTTOM_EXTT,
+		XmNtopWidget, m->dbprog.dbprog_sep1,
+		XmNtopOffset, TOFF_TRKLISTLBL,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.pgm_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_PGMLBL,
 		XmNrightPosition, RIGHT_PGMLBL,
-		XmNtopPosition, TOP_PGMLBL,
-		XmNbottomPosition, BOTTOM_PGMLBL,
+		XmNtopWidget, m->dbprog.dbprog_sep1,
+		XmNtopOffset, TOFF_PGMLBL,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.addpgm_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_ADDPGM,
 		XmNrightPosition, RIGHT_ADDPGM,
-		XmNtopPosition, TOP_ADDPGM,
-		XmNbottomPosition, BOTTOM_ADDPGM,
+		XmNtopWidget, m->dbprog.pgm_lbl,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.clrpgm_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
 		XmNleftPosition, LEFT_CLRPGM,
 		XmNrightPosition, RIGHT_CLRPGM,
-		XmNtopPosition, TOP_CLRPGM,
-		XmNbottomPosition, BOTTOM_CLRPGM,
+		XmNtopWidget, m->dbprog.addpgm_btn,
 		NULL
 	);
 
-	XtVaSetValues(m->dbprog.pgmseq_lbl,
+	XtVaSetValues(m->dbprog.radio_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_PGMSEQLBL,
-		XmNrightPosition, RIGHT_PGMSEQLBL,
-		XmNtopPosition, TOP_PGMSEQLBL,
-		XmNbottomPosition, BOTTOM_PGMSEQLBL,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.pgmseq_txt,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_PGMSEQ,
-		XmNrightPosition, RIGHT_PGMSEQ,
-		XmNtopPosition, TOP_PGMSEQ,
-		XmNbottomPosition, BOTTOM_PGMSEQ,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.send_btn,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_POSITION,
-		XmNleftPosition, LEFT_SEND,
-		XmNrightPosition, RIGHT_SEND,
-		XmNtopPosition, TOP_SEND,
-		XmNbottomPosition, BOTTOM_SEND,
-		NULL
-	);
-
-	XtVaSetValues(m->dbprog.dbprog_sep2,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
 		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_DBPROGSEP2,
-		XmNrightPosition, RIGHT_DBPROGSEP2,
-		XmNtopPosition, TOP_DBPROGSEP2,
+		XmNleftPosition, LEFT_RADIOLBL,
+		XmNrightPosition, RIGHT_RADIOLBL,
+		XmNtopWidget, m->dbprog.clrpgm_btn,
+		XmNtopOffset, TOFF_RADIOLBL,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.radio_frm,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_RADIOBOX,
+		XmNrightPosition, RIGHT_RADIOBOX,
+		XmNtopWidget, m->dbprog.radio_lbl,
+		XmNshadowType, XmSHADOW_ETCHED_IN,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.tottime_btn,
+		XmNrecomputeSize, False,
+		NULL
+	);
+	XtVaSetValues(m->dbprog.trktime_btn,
+		XmNrecomputeSize, False,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.extt_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_EXTTLBL,
+		XmNrightPosition, RIGHT_EXTTLBL,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNtopWidget, m->dbprog.radio_frm,
+		XmNtopOffset, TOFF_EXTTLBL,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.extt_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_EXTT,
+		XmNrightPosition, RIGHT_EXTT,
+		XmNtopWidget, m->dbprog.extt_lbl,
+		XmNtopOffset, TOFF_EXTT,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.savedb_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_FORM,
 		XmNleftPosition, LEFT_SAVEDB,
 		XmNrightPosition, RIGHT_SAVEDB,
-		XmNtopPosition, TOP_SAVEDB,
+		XmNbottomOffset, BOFF_SAVEDB,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.linkdb_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_FORM,
 		XmNleftPosition, LEFT_LINK,
 		XmNrightPosition, RIGHT_LINK,
-		XmNtopPosition, TOP_LINK,
+		XmNbottomOffset, BOFF_LINK,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.loaddb_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_FORM,
 		XmNleftPosition, LEFT_LOADDB,
 		XmNrightPosition, RIGHT_LOADDB,
-		XmNtopPosition, TOP_LOADDB,
+		XmNbottomOffset, BOFF_LOADDB,
 		NULL
 	);
 
 	XtVaSetValues(m->dbprog.cancel_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_FORM,
 		XmNleftPosition, LEFT_DPCANCEL,
 		XmNrightPosition, RIGHT_DPCANCEL,
-		XmNtopPosition, TOP_DPCANCEL,
+		XmNbottomOffset, BOFF_DPCANCEL,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.dbprog_sep2,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_DBPROGSEP2,
+		XmNrightPosition, RIGHT_DBPROGSEP2,
+		XmNbottomWidget, m->dbprog.cancel_btn,
+		XmNbottomOffset, BOFF_DBPROGSEP2,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.pgmseq_txt,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_PGMSEQ,
+		XmNrightPosition, RIGHT_PGMSEQ,
+		XmNbottomWidget, m->dbprog.dbprog_sep2,
+		XmNbottomOffset, BOFF_PGMSEQLBL,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.pgmseq_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_PGMSEQLBL,
+		XmNrightPosition, RIGHT_PGMSEQLBL,
+		XmNbottomWidget, m->dbprog.pgmseq_txt,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.cddb_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_SENDLBL,
+		XmNrightPosition, RIGHT_SENDLBL,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNbottomWidget, m->dbprog.rmtdsbl_btn,
+		XmNbottomOffset, BOFF_SENDLBL,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.rmtdsbl_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_RMTDSBL,
+		XmNrightPosition, RIGHT_RMTDSBL,
+		XmNbottomWidget, m->dbprog.submit_btn,
+		XmNbottomOffset, BOFF_RMTDSBL,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.submit_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_SEND,
+		XmNrightPosition, RIGHT_SEND,
+		XmNbottomWidget, m->dbprog.dbprog_sep2,
+		XmNbottomOffset, BOFF_SEND,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.ttitle_txt,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_TTITLE,
+		XmNrightPosition, RIGHT_TTITLE,
+		XmNbottomWidget, m->dbprog.pgmseq_lbl,
+		XmNbottomOffset, BOFF_TTITLE,
+		NULL
+	);
+
+	XtVaSetValues(m->dbprog.ttitle_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_TTITLELBL,
+		XmNrightPosition, RIGHT_TTITLELBL,
+		XmNbottomWidget, m->dbprog.ttitle_txt,
+		XmNbottomOffset, BOFF_TTITLELBL,
+		NULL
+	);
+
+	XtVaSetValues(XtParent(m->dbprog.trk_list),
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_TRKLIST,
+		XmNrightPosition, RIGHT_TRKLIST,
+		XmNtopWidget, m->dbprog.trklist_lbl,
+		XmNbottomWidget, m->dbprog.ttitle_lbl,
+		XmNleftOffset, LOFF_TRKLIST,
+		XmNbottomOffset, BOFF_TRKLIST,
 		NULL
 	);
 }
 
 
 /*
- * geom_wwwwarp_force
- *	Set the geometry of the widgets in the wwwWarp window.
+ * geom_dlist_force
+ *	Set the geometry of the widgets in the Disc List window.
  *
  * Args:
  *	m - Pointer to the main widgets structure.
@@ -1751,81 +1916,125 @@ geom_dbprog_force(widgets_t *m)
  *	Nothing.
  */
 STATIC void
-geom_wwwwarp_force(widgets_t *m)
+geom_dlist_force(widgets_t *m)
 {
-	XtVaSetValues(m->wwwwarp.wwwwarp_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLPIXMAP,
-		XmNrightPosition, RIGHT_WLPIXMAP,
-		XmNtopPosition, TOP_WLPIXMAP,
-		NULL
-	);
-
-	XtVaSetValues(m->wwwwarp.sel_lbl,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLSELLBL,
-		XmNrightPosition, RIGHT_WLSELLBL,
-		XmNtopPosition, TOP_WLSELLBL,
-		NULL
-	);
-
-	XtVaSetValues(m->wwwwarp.sel_radbox,
-		XmNleftAttachment, XmATTACH_POSITION,
-		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
-		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLRADBOX,
-		XmNrightPosition, RIGHT_WLRADBOX,
-		XmNtopPosition, TOP_WLRADBOX,
-		NULL
-	);
-
-	XtVaSetValues(m->wwwwarp.srchsite_opt,
+	XtVaSetValues(m->dlist.type_opt,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_FORM,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_DLISTOPT,
+		XmNtopOffset, TOFF_DLISTOPT,
+		NULL
+	);
+
+	XtVaSetValues(m->dlist.disclist_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
 		XmNtopAttachment, XmATTACH_WIDGET,
 		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLSRCHSITES,
-		XmNtopWidget, m->wwwwarp.sel_radbox,
+		XmNleftPosition, LEFT_DLISTLBL,
+		XmNrightPosition, RIGHT_DLISTLBL,
+		XmNtopWidget, m->dlist.type_opt,
+		XmNtopOffset, TOFF_DLISTLBL,
 		NULL
 	);
 
-	XtVaSetValues(m->wwwwarp.wwwwarp_sep,
+	XtVaSetValues(m->dlist.show_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_OPPOSITE_WIDGET,
 		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLSEP,
-		XmNrightPosition, RIGHT_WLSEP,
-		XmNtopPosition, TOP_WLSEP,
+		XmNleftPosition, LEFT_DLSHOWBTN,
+		XmNrightPosition, RIGHT_DLSHOWBTN,
+		XmNtopWidget, m->dlist.disclist_lbl,
+		XmNtopOffset, TOFF_DLSHOWBTN,
 		NULL
 	);
 
-	XtVaSetValues(m->wwwwarp.go_btn,
+	XtVaSetValues(m->dlist.goto_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
 		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLGO,
-		XmNrightPosition, RIGHT_WLGO,
-		XmNtopPosition, TOP_WLGO,
+		XmNleftPosition, LEFT_DLGOTOBTN,
+		XmNrightPosition, RIGHT_DLGOTOBTN,
+		XmNtopWidget, m->dlist.show_btn,
+		XmNtopOffset, TOFF_DLGOTOBTN,
 		NULL
 	);
 
-	XtVaSetValues(m->wwwwarp.cancel_btn,
+	XtVaSetValues(m->dlist.del_btn,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
-		XmNtopAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
 		XmNbottomAttachment, XmATTACH_NONE,
-		XmNleftPosition, LEFT_WLCANCEL,
-		XmNrightPosition, RIGHT_WLCANCEL,
-		XmNtopPosition, TOP_WLCANCEL,
+		XmNleftPosition, LEFT_DLDELBTN,
+		XmNrightPosition, RIGHT_DLDELBTN,
+		XmNtopWidget, m->dlist.goto_btn,
+		XmNtopOffset, TOFF_DLDELBTN,
+		NULL
+	);
+
+	XtVaSetValues(m->dlist.delall_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_DLDELALLBTN,
+		XmNrightPosition, RIGHT_DLDELALLBTN,
+		XmNtopWidget, m->dlist.del_btn,
+		XmNtopOffset, TOFF_DLDELALLBTN,
+		NULL
+	);
+
+	XtVaSetValues(m->dlist.rescan_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_DLRESCANBTN,
+		XmNrightPosition, RIGHT_DLRESCANBTN,
+		XmNtopWidget, m->dlist.delall_btn,
+		XmNtopOffset, TOFF_DLRESCANBTN,
+		NULL
+	);
+
+	XtVaSetValues(m->dlist.dlist_sep,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_DLISTSEP,
+		XmNrightPosition, RIGHT_DLISTSEP,
+		XmNbottomWidget, m->dlist.cancel_btn,
+		XmNbottomOffset, BOFF_DLISTSEP,
+		NULL
+	);
+
+	XtVaSetValues(m->dlist.cancel_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_NONE,
+		XmNbottomAttachment, XmATTACH_FORM,
+		XmNleftPosition, LEFT_DLISTCANCEL,
+		XmNrightPosition, RIGHT_DLISTCANCEL,
+		XmNbottomOffset, BOFF_DLISTCANCEL,
+		NULL
+	);
+
+	XtVaSetValues(XtParent(m->dlist.disc_list),
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_WIDGET,
+		XmNleftPosition, LEFT_DISCLIST,
+		XmNrightPosition, RIGHT_DISCLIST,
+		XmNtopWidget, m->dlist.disclist_lbl,
+		XmNbottomWidget, m->dlist.dlist_sep,
+		XmNleftOffset, LOFF_DISCLIST,
+		XmNtopOffset, TOFF_DISCLIST,
+		XmNbottomOffset, BOFF_DISCLIST,
 		NULL
 	);
 }
@@ -1977,6 +2186,17 @@ geom_extt_force(widgets_t *m)
 		NULL
 	);
 
+	XtVaSetValues(m->dbextt.autotrk_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_POSITION,
+		XmNleftPosition, LEFT_EXTTAUTOTRKBTN,
+		XmNtopPosition, TOP_EXTTAUTOTRKBTN,
+		XmNbottomPosition, BOTTOM_EXTTAUTOTRKBTN,
+		NULL
+	);
+
 	XtVaSetValues(m->dbextt.trk_lbl,
 		XmNleftAttachment, XmATTACH_POSITION,
 		XmNrightAttachment, XmATTACH_POSITION,
@@ -2047,6 +2267,108 @@ geom_extt_force(widgets_t *m)
 }
 
 
+/*
+ * geom_wwwwarp_force
+ *	Set the geometry of the widgets in the wwwWarp window.
+ *
+ * Args:
+ *	m - Pointer to the main widgets structure.
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+geom_wwwwarp_force(widgets_t *m)
+{
+	XtVaSetValues(m->wwwwarp.wwwwarp_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLPIXMAP,
+		XmNrightPosition, RIGHT_WLPIXMAP,
+		XmNtopPosition, TOP_WLPIXMAP,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.help_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_HELP,
+		XmNrightPosition, RIGHT_HELP,
+		XmNtopPosition, TOP_HELP,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.sel_lbl,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLSELLBL,
+		XmNrightPosition, RIGHT_WLSELLBL,
+		XmNtopPosition, TOP_WLSELLBL,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.sel_radbox,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLRADBOX,
+		XmNrightPosition, RIGHT_WLRADBOX,
+		XmNtopPosition, TOP_WLRADBOX,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.srchsite_opt,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_NONE,
+		XmNtopAttachment, XmATTACH_WIDGET,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLSRCHSITES,
+		XmNtopWidget, m->wwwwarp.sel_radbox,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.wwwwarp_sep,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLSEP,
+		XmNrightPosition, RIGHT_WLSEP,
+		XmNtopPosition, TOP_WLSEP,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.go_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLGO,
+		XmNrightPosition, RIGHT_WLGO,
+		XmNtopPosition, TOP_WLGO,
+		NULL
+	);
+
+	XtVaSetValues(m->wwwwarp.cancel_btn,
+		XmNleftAttachment, XmATTACH_POSITION,
+		XmNrightAttachment, XmATTACH_POSITION,
+		XmNtopAttachment, XmATTACH_POSITION,
+		XmNbottomAttachment, XmATTACH_NONE,
+		XmNleftPosition, LEFT_WLCANCEL,
+		XmNrightPosition, RIGHT_WLCANCEL,
+		XmNtopPosition, TOP_WLCANCEL,
+		NULL
+	);
+}
+
+
 /*
  * geom_help_force
  *	Set the geometry of the widgets in the help display window.
@@ -2392,9 +2714,10 @@ geom_force(widgets_t *m)
 	geom_keypad_force(m);
 	geom_options_force(m);
 	geom_dbprog_force(m);
-	geom_wwwwarp_force(m);
+	geom_dlist_force(m);
 	geom_extd_force(m);
 	geom_extt_force(m);
+	geom_wwwwarp_force(m);
 	geom_help_force(m);
 	geom_dirsel_force(m);
 	geom_linksel_force(m);
