@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif CD Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __GEOM_H__
 
 #ifndef LINT
-static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
+static char *_geom_h_ident_ = "@(#)geom.h	6.40 00/01/15";
 #endif
 
 /* Location of widgets relative to form origin, express in percentage of
@@ -136,10 +136,10 @@ static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
 #define TOP_KEYPAD		65
 #define BOTTOM_KEYPAD		80
 
-#define LEFT_HELP		24
-#define RIGHT_HELP		34
-#define TOP_HELP		80
-#define BOTTOM_HELP		100
+#define LEFT_WWWWARP		24
+#define RIGHT_WWWWARP		34
+#define TOP_WWWWARP		80
+#define BOTTOM_WWWWARP		100
 
 #define LEFT_LEVEL		34
 #define RIGHT_LEVEL		74
@@ -344,77 +344,77 @@ static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
 /* Options window widgets */
 #define LEFT_LOAD_LBL		6
 #define RIGHT_LOAD_LBL		47
-#define TOP_LOAD_LBL		2
-#define BOTTOM_LOAD_LBL		6
+#define TOP_LOAD_LBL		1
+#define BOTTOM_LOAD_LBL		5
 
 #define LEFT_LOAD_CHKFRM	6
 #define RIGHT_LOAD_CHKFRM	47
-#define TOP_LOAD_CHKFRM		6
+#define TOP_LOAD_CHKFRM		5
 
 #define LEFT_LOAD_RADFRM	6
 #define RIGHT_LOAD_RADFRM	47
-#define TOP_LOAD_RADFRM		12
-#define BOTTOM_LOAD_RADFRM	27
+#define TOP_LOAD_RADFRM		11
+#define BOTTOM_LOAD_RADFRM	26
 
 #define LEFT_EXIT_LBL		6
 #define RIGHT_EXIT_LBL		47
-#define TOP_EXIT_LBL		28
-#define BOTTOM_EXIT_LBL		32
+#define TOP_EXIT_LBL		26
+#define BOTTOM_EXIT_LBL		30
 
 #define LEFT_EXIT_RADFRM	6
 #define RIGHT_EXIT_RADFRM	47
-#define TOP_EXIT_RADFRM		32
-#define BOTTOM_EXIT_RADFRM	47
+#define TOP_EXIT_RADFRM		30
+#define BOTTOM_EXIT_RADFRM	45
 
 #define LEFT_DONE_LBL		6
 #define RIGHT_DONE_LBL		47
-#define TOP_DONE_LBL		48
-#define BOTTOM_DONE_LBL		52
+#define TOP_DONE_LBL		45
+#define BOTTOM_DONE_LBL		49
 
 #define LEFT_DONE_CHKFRM	6
 #define RIGHT_DONE_CHKFRM	47
-#define TOP_DONE_CHKFRM		52
-#define BOTTOM_DONE_CHKFRM	58
+#define TOP_DONE_CHKFRM		49
+#define BOTTOM_DONE_CHKFRM	60
 
 #define LEFT_EJECT_LBL		6
 #define RIGHT_EJECT_LBL		47
-#define TOP_EJECT_LBL		59
-#define BOTTOM_EJECT_LBL	63
+#define TOP_EJECT_LBL		60
+#define BOTTOM_EJECT_LBL	64
 
 #define LEFT_EJECT_CHKFRM	6
 #define RIGHT_EJECT_CHKFRM	47
-#define TOP_EJECT_CHKFRM	63
-#define BOTTOM_EJECT_CHKFRM	69
+#define TOP_EJECT_CHKFRM	64
+#define BOTTOM_EJECT_CHKFRM	70
 
 #define LEFT_CHG_LBL		53
 #define RIGHT_CHG_LBL		94
-#define TOP_CHG_LBL		2
-#define BOTTOM_CHG_LBL		6
+#define TOP_CHG_LBL		1
+#define BOTTOM_CHG_LBL		5
 
 #define LEFT_CHG_CHKFRM		53
 #define RIGHT_CHG_CHKFRM	94
-#define TOP_CHG_CHKFRM		6
-#define BOTTOM_CHG_CHKFRM	21
+#define TOP_CHG_CHKFRM		5
+#define BOTTOM_CHG_CHKFRM	16
 
 #define LEFT_CHROUTE_LBL	53
 #define RIGHT_CHROUTE_LBL	94
-#define TOP_CHROUTE_LBL		22
-#define BOTTOM_CHROUTE_LBL	26
+#define TOP_CHROUTE_LBL		20
+#define BOTTOM_CHROUTE_LBL	24
 
 #define LEFT_CHROUTE_RADFRM	53
 #define RIGHT_CHROUTE_RADFRM	94
-#define TOP_CHROUTE_RADFRM	26
-#define BOTTOM_CHROUTE_RADFRM	49
+#define TOP_CHROUTE_RADFRM	24
+#define BOTTOM_CHROUTE_RADFRM	47
 
 #define LEFT_VOLTP_LBL		53
 #define RIGHT_VOLTP_LBL		94
-#define TOP_VOLTP_LBL		50
-#define BOTTOM_VOLTP_LBL	54
+#define TOP_VOLTP_LBL		51
+#define BOTTOM_VOLTP_LBL	55
 
 #define LEFT_VOLTP_RADFRM	53
 #define RIGHT_VOLTP_RADFRM	94
-#define TOP_VOLTP_RADFRM	54
-#define BOTTOM_VOLTP_RADFRM	69
+#define TOP_VOLTP_RADFRM	55
+#define BOTTOM_VOLTP_RADFRM	70
 
 #define LEFT_BAL_LBL		20
 #define RIGHT_BAL_LBL		80
@@ -458,30 +458,24 @@ static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
 #define TOP_OPTOK_BTN		91
 
 /* Database/Program window widgets */
-#define LEFT_TOTTIMELBL		2
-#define RIGHT_TOTTIMELBL	15
-#define TOP_TOTTIMELBL		1
-#define BOTTOM_TOTTIMELBL	5
+#define LEFT_DISCIDIND		2
+#define TOFF_DISCIDIND		4
+#define BOFF_DISCIDIND		0
 
-#define LEFT_TOTTIMEIND		14
-#define RIGHT_TOTTIMEIND	24
-#define TOP_TOTTIMEIND		1
-#define BOTTOM_TOTTIMEIND	5
+#define LEFT_TOTTIMEIND		2
+#define TOFF_TOTTIMEIND		0
 
-#define LEFT_LOGO		31
-#define RIGHT_LOGO		51
-#define TOP_LOGO		1
-#define BOTTOM_LOGO		11
+#define LEFT_LOGO		33
+#define RIGHT_LOGO		49
+#define TOFF_LOGO		6
 
-#define LEFT_WEB		81
-#define RIGHT_WEB		98
-#define TOP_WEB			2
-#define BOTTOM_WEB		7
+#define LEFT_DLIST		81
+#define RIGHT_DLIST		98
+#define TOFF_DLIST		8
 
 #define LEFT_DTITLELBL		2
 #define RIGHT_DTITLELBL		79
-#define TOP_DTITLELBL		11
-#define BOTTOM_DTITLELBL	14
+#define TOFF_DTITLELBL		0
 
 #define LEFT_DTITLE		2
 #define RIGHT_DTITLE		79
@@ -490,144 +484,138 @@ static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
 
 #define LEFT_EXTDLBL		81
 #define RIGHT_EXTDLBL		98
-#define TOP_EXTDLBL		14
-#define BOTTOM_EXTDLBL		17
+#define BOFF_EXTDLBL		0
 
 #define LEFT_EXTD		81
 #define RIGHT_EXTD		98
-#define TOP_EXTD		17
-#define BOTTOM_EXTD		22
+#define BOFF_EXTD		0
 
 #define LEFT_DBPROGSEP1		0
 #define RIGHT_DBPROGSEP1	100
-#define TOP_DBPROGSEP1		24
+#define TOFF_DBPROGSEP1		8
 
 #define LEFT_TRKLISTLBL		2
 #define RIGHT_TRKLISTLBL	79
-#define TOP_TRKLISTLBL		26
-#define BOTTOM_TRKLISTLBL	29
+#define TOFF_TRKLISTLBL		6
 
 #define LEFT_TRKLIST		2
 #define RIGHT_TRKLIST		79
-#define TOP_TRKLIST		29
-#define BOTTOM_TRKLIST		61
+#define LOFF_TRKLIST		2
+#define BOFF_TRKLIST		4
 
 #define LEFT_RADIOLBL		81
 #define RIGHT_RADIOLBL		98
-#define TOP_RADIOLBL		40
-#define BOTTOM_RADIOLBL		43
+#define TOFF_RADIOLBL		4
 
 #define LEFT_RADIOBOX		82
 #define RIGHT_RADIOBOX		97
-#define TOP_RADIOBOX		43
-#define BOTTOM_RADIOBOX		53
-
-#define LEFT_DISCIDLBL		81
-#define RIGHT_DISCIDLBL		98
-#define TOP_DISCIDLBL		68
-#define BOTTOM_DISCIDLBL	71
-
-#define LEFT_DISCIDFRM		82
-#define RIGHT_DISCIDFRM		97
-#define TOP_DISCIDFRM		71
-#define BOTTOM_DISCIDFRM	79
 
 #define LEFT_TTITLELBL		2
 #define RIGHT_TTITLELBL		79
-#define TOP_TTITLELBL		62
-#define BOTTOM_TTITLELBL	65
+#define BOFF_TTITLELBL		0
 
 #define LEFT_TTITLE		2
 #define RIGHT_TTITLE		79
-#define TOP_TTITLE		65
-#define BOTTOM_TTITLE		73
+#define BOFF_TTITLE		2
 
 #define LEFT_EXTTLBL		81
 #define RIGHT_EXTTLBL		98
-#define TOP_EXTTLBL		54
-#define BOTTOM_EXTTLBL		57
+#define TOFF_EXTTLBL		4
 
 #define LEFT_EXTT		81
 #define RIGHT_EXTT		98
-#define TOP_EXTT		57
-#define BOTTOM_EXTT		62
+#define TOFF_EXTT		0
 
 #define LEFT_PGMLBL		81
 #define RIGHT_PGMLBL		98
-#define TOP_PGMLBL		26
-#define BOTTOM_PGMLBL		29
+#define TOFF_PGMLBL		6
 
 #define LEFT_ADDPGM		81
 #define RIGHT_ADDPGM		98
-#define TOP_ADDPGM		29
-#define BOTTOM_ADDPGM		34
 
 #define LEFT_CLRPGM		81
 #define RIGHT_CLRPGM		98
-#define TOP_CLRPGM		34
-#define BOTTOM_CLRPGM		39
 
 #define LEFT_PGMSEQLBL		2
 #define RIGHT_PGMSEQLBL		79
-#define TOP_PGMSEQLBL		74
-#define BOTTOM_PGMSEQLBL	77
+#define BOFF_PGMSEQLBL		8
 
 #define LEFT_PGMSEQ		2
 #define RIGHT_PGMSEQ		79
-#define TOP_PGMSEQ		77
-#define BOTTOM_PGMSEQ		85
+
+#define LEFT_SENDLBL		81
+#define RIGHT_SENDLBL		98
+#define BOFF_SENDLBL		0
 
 #define LEFT_SEND		81
 #define RIGHT_SEND		98
-#define TOP_SEND		80
-#define BOTTOM_SEND		85
+#define BOFF_SEND		8
+
+#define LEFT_RMTDSBL		81
+#define RIGHT_RMTDSBL		98
+#define BOFF_RMTDSBL		2
 
 #define LEFT_DBPROGSEP2		0
 #define RIGHT_DBPROGSEP2	100
-#define TOP_DBPROGSEP2		87
+#define BOFF_DBPROGSEP2		16
 
 #define LEFT_SAVEDB		6
 #define RIGHT_SAVEDB		25
-#define TOP_SAVEDB		91
+#define BOFF_SAVEDB		14
 
 #define LEFT_LINK		29
 #define RIGHT_LINK		48
-#define TOP_LINK		91
+#define BOFF_LINK		14
 
 #define LEFT_LOADDB		52
 #define RIGHT_LOADDB		71
-#define TOP_LOADDB		91
+#define BOFF_LOADDB		14
 
 #define LEFT_DPCANCEL		75
 #define RIGHT_DPCANCEL		94
-#define TOP_DPCANCEL		91
+#define BOFF_DPCANCEL		14
 
-/* wwwWarp widgets */
-#define LEFT_WLPIXMAP		30
-#define RIGHT_WLPIXMAP		70
-#define TOP_WLPIXMAP		3
+/* Disc List window widgets */
+#define LEFT_DLISTOPT		2
+#define TOFF_DLISTOPT		8
 
-#define LEFT_WLSELLBL		25
-#define RIGHT_WLSELLBL		75
-#define TOP_WLSELLBL		18
+#define LEFT_DLISTLBL		2
+#define RIGHT_DLISTLBL		78
+#define TOFF_DLISTLBL		6
 
-#define LEFT_WLRADBOX		8
-#define RIGHT_WLRADBOX		92
-#define TOP_WLRADBOX		26
+#define LEFT_DLSHOWBTN		80
+#define RIGHT_DLSHOWBTN		98
+#define TOFF_DLSHOWBTN		10
 
-#define LEFT_WLSRCHSITES	8
+#define LEFT_DLGOTOBTN		80
+#define RIGHT_DLGOTOBTN		98
+#define TOFF_DLGOTOBTN		2
 
-#define LEFT_WLSEP		0
-#define RIGHT_WLSEP		100
-#define TOP_WLSEP		83
+#define LEFT_DLDELBTN		80
+#define RIGHT_DLDELBTN		98
+#define TOFF_DLDELBTN		2
 
-#define LEFT_WLGO		15
-#define RIGHT_WLGO		40
-#define TOP_WLGO		87
+#define LEFT_DLDELALLBTN	80
+#define RIGHT_DLDELALLBTN	98
+#define TOFF_DLDELALLBTN	2
 
-#define LEFT_WLCANCEL		60
-#define RIGHT_WLCANCEL		85
-#define TOP_WLCANCEL		87
+#define LEFT_DLRESCANBTN	80
+#define RIGHT_DLRESCANBTN	98
+#define TOFF_DLRESCANBTN	2
+
+#define LEFT_DLISTSEP		0
+#define RIGHT_DLISTSEP		100
+#define BOFF_DLISTSEP		10
+
+#define LEFT_DLISTCANCEL	40
+#define RIGHT_DLISTCANCEL	60
+#define BOFF_DLISTCANCEL	10
+
+#define LEFT_DISCLIST		2
+#define RIGHT_DISCLIST		78
+#define LOFF_DISCLIST		2
+#define TOFF_DISCLIST		2
+#define BOFF_DISCLIST		8
 
 /* Disc Extended Info editor widgets */
 #define LEFT_DISCNUMLBL		40
@@ -677,6 +665,10 @@ static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
 #define TOP_EXTTTRKNUMLBL	2
 #define BOTTOM_EXTTTRKNUMLBL	7
 
+#define LEFT_EXTTAUTOTRKBTN	2
+#define TOP_EXTTAUTOTRKBTN	2
+#define BOTTOM_EXTTAUTOTRKBTN	7
+
 #define LEFT_TRKLBL		2
 #define RIGHT_TRKLBL		98
 #define TOP_TRKLBL		7
@@ -703,6 +695,37 @@ static char *_geom_h_ident_ = "@(#)geom.h	6.28 98/05/19";
 #define RIGHT_DTCANCEL		90
 #define TOP_DTCANCEL		89
 
+/* wwwWarp widgets */
+#define LEFT_WLPIXMAP		30
+#define RIGHT_WLPIXMAP		70
+#define TOP_WLPIXMAP		3
+
+#define LEFT_HELP		71
+#define RIGHT_HELP		98
+#define TOP_HELP		2
+
+#define LEFT_WLSELLBL		25
+#define RIGHT_WLSELLBL		75
+#define TOP_WLSELLBL		18
+
+#define LEFT_WLRADBOX		8
+#define RIGHT_WLRADBOX		92
+#define TOP_WLRADBOX		26
+
+#define LEFT_WLSRCHSITES	8
+
+#define LEFT_WLSEP		0
+#define RIGHT_WLSEP		100
+#define TOP_WLSEP		83
+
+#define LEFT_WLGO		15
+#define RIGHT_WLGO		40
+#define TOP_WLGO		87
+
+#define LEFT_WLCANCEL		60
+#define RIGHT_WLCANCEL		85
+#define TOP_WLCANCEL		87
+
 /* Help popup widgets */
 #define LEFT_TOPIC		2
 #define TOP_TOPIC		2
