@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,24 @@
 #define __HELP_H__
 
 #ifndef LINT
-static char *_help_h_ident_ = "@(#)help.h	6.12 98/05/19";
+static char *_help_h_ident_ = "@(#)help.h	6.20 99/12/27";
 #endif
 
-/* Max number of widgets supported by the help system */
-#define MAX_HELP_WIDGETS	112
+
+#define HELP_XLAT_1	1
+#define HELP_XLAT_2	2
+
+/* This structure is used to map widgets to associated help files.
+ * Instead of using XtName(), this mechanism allows us to map multiple
+ * widgets to a common help file.  Also, we can use arbitrary lengths
+ * for the widget name and still have help files with less than 14 chars
+ * in its name (necessary for compatibility with some systems).
+ */
+typedef struct {
+	Widget	*widgetp;
+	char	*hlpname;
+	int	xlat_typ;
+} wname_t;
 
 
 /* Documentation topics list structure */
