@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  *
  */
 #ifndef LINT
-static char *_hotkey_c_ident_ = "@(#)hotkey.c	6.20 98/05/14";
+static char *_hotkey_c_ident_ = "@(#)hotkey.c	6.25 99/12/27";
 #endif
 
 #include "common_d/appenv.h"
@@ -331,7 +331,10 @@ hotkey_build_grablist(
 			/* Allocate new list element */
 			if (*listhead == NULL) {
 				*listhead = g = (grablist_t *)(void *)
-					MEM_ALLOC(sizeof(grablist_t));
+					MEM_ALLOC(
+						"hotkey_listhead",
+						sizeof(grablist_t)
+					);
 
 				if (g == NULL) {
 					CD_FATAL(app_data.str_nomemory);
@@ -340,7 +343,10 @@ hotkey_build_grablist(
 			}
 			else {
 				g->next = (grablist_t *)(void *)
-					MEM_ALLOC(sizeof(grablist_t));
+					MEM_ALLOC(
+						"g->next",
+						sizeof(grablist_t)
+					);
 
 				if (g->next == NULL) {
 					CD_FATAL(app_data.str_nomemory);
@@ -416,7 +422,8 @@ hotkey_init(void)
 
 	m = &widgets;
 
-	if ((xlat_str = (char *) MEM_ALLOC(MAX_TRANSLATIONS_SZ)) == NULL) {
+	xlat_str = (char *) MEM_ALLOC("xlat_str", MAX_TRANSLATIONS_SZ);
+	if (xlat_str == NULL) {
 		CD_FATAL(app_data.str_nomemory);
 		return;
 	}
@@ -431,7 +438,7 @@ hotkey_init(void)
 		app_data.eject_key,
 		app_data.quit_key,
 		app_data.dbprog_key,
-		app_data.help_key,
+		app_data.wwwwarp_key,
 		app_data.options_key,
 		app_data.time_key,
 		app_data.ab_key,
@@ -512,6 +519,9 @@ hotkey_grabkeys(Widget form)
 	grablist_t	*list,
 			*p;
 
+	if (form == (Widget) NULL)
+		return;
+
 	if (form == widgets.main.form)
 		list = grablists[MAIN_LIST];
 	else if (form == widgets.keypad.form)
@@ -548,6 +558,9 @@ hotkey_ungrabkeys(Widget form)
 	grablist_t	*list,
 			*p;
 
+	if (form == (Widget) NULL)
+		return;
+
 	if (form == widgets.main.form)
 		list = grablists[MAIN_LIST];
 	else if (form == widgets.keypad.form)
