@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
  *
  */
 #ifndef LINT
-static char *_main_c_ident_ = "@(#)main.c	6.22 98/05/07";
+static char *_main_c_ident_ = "@(#)main.c	6.41 00/01/15";
 #endif
 
 #include "common_d/appenv.h"
@@ -28,10 +28,16 @@ static char *_main_c_ident_ = "@(#)main.c	6.22 98/05/07";
 #include "xmcd_d/xmcd.h"
 #include "xmcd_d/resource.h"
 #include "xmcd_d/widget.h"
+#include "xmcd_d/callback.h"
 #include "xmcd_d/cdfunc.h"
+#include "xmcd_d/command.h"
 #include "libdi_d/libdi.h"
 
 
+#define STARTUP_CMD_DELAY	1500	/* Delay interval before running
+					 * startup command
+					 */
+
 /* Global data */
 char			*progname,	/* The path name we are invoked with */
 			*cdisplay;	/* Command-line specified display */
@@ -43,6 +49,8 @@ FILE			*errfp;		/* Error message stream */
 /* Data global to this module only */
 STATIC curstat_t	status;		/* Current CD player status */
 STATIC XtAppContext	app_context;	/* Application context */
+STATIC Colormap		localcmap;	/* local colormap */
+STATIC char		*cmd;		/* Command string */
 
 
 /***********************
@@ -66,43 +74,6 @@ curstat_addr(void)
 }
 
 
-/***********************
- *  internal routines  *
- ***********************/
-
-
-/*
- * usage
- *	Display command line usage syntax
- *
- * Args:
- *	argc, argv
- *
- * Return:
- *	Nothing.
- */
-STATIC void
-usage(int argc, char **argv)
-{
-	int	i;
-
-	(void) fprintf(errfp, "%s\n", app_data.str_badopts);
-	for (i = 1; i < argc; i++)
-		(void) fprintf(errfp, "%s ", argv[i]);
-
-	(void) fprintf(errfp, "\n\n%s %s [-dev device] [-debug]",
-		       app_data.str_usage, argv[0]);
-
-#if defined(SVR4) && defined(sun)
-	/* Solaris 2 volume manager auto-start support */
-	(void) fprintf(errfp, " [-c device] [-X] [-o]");
-#endif
-
- 	(void) fprintf(errfp,
-	    "\n\nStandard Xt Intrinsics and Motif options are supported.\n");
-}
-
-
 /*
  * event_loop
  *	Used to handle X events while waiting on I/O.
@@ -122,6 +93,71 @@ event_loop(int flag)
 }
 
 
+/*
+ * shutdown_gui
+ *	Perform window system shutdown cleanup.
+ *
+ * Args:
+ *	None.
+ *
+ * Return:
+ *	Nothing.
+ */
+void
+shutdown_gui(void)
+{
+	if (app_data.instcmap)
+		XFreeColormap(XtDisplay(widgets.toplevel), localcmap);
+}
+
+
+/***********************
+ *  internal routines  *
+ ***********************/
+
+
+/*
+ * usage
+ *	Display command line usage syntax
+ *
+ * Args:
+ *	arg - The command line arg string that is invalid
+ *
+ * Return:
+ *	Nothing.
+ */
+STATIC void
+usage(char *arg)
+{
+	if (strcmp(arg, "-help") != 0) {
+		(void) fprintf(errfp, "%s: %s\n\n",
+			app_data.str_badopts, arg);
+	}
+
+	(void) fprintf(errfp, "%s %s %s %s %s\n            %s %s %s",
+		app_data.str_usage, PROGNAME,
+		"[-dev device]",
+		"[-instcmap]",
+		"[-remote]",
+		"[-rmthost hostname]",
+		"[-debug]",
+		"[-help]");
+
+#if defined(SVR4) && defined(sun)
+	/* Solaris 2 volume manager auto-start support */
+	(void) fprintf(errfp, "\n            [-c device] [-X] [-o]");
+#endif
+
+ 	(void) fprintf(errfp, "\n            [command [arg ...]]\n\n");
+
+	/* Display command usage */
+	cmd_usage();
+
+ 	(void) fprintf(errfp,
+	    "\nStandard Xt Intrinsics toolkit options are also supported.\n");
+}
+
+
 /*
  * main
  *	The main function
@@ -130,6 +166,13 @@ int
 main(int argc, char **argv)
 {
 	int	i;
+	Display	*display;
+#ifdef HAS_SETEUID
+	uid_t	euid,
+		ruid;
+	gid_t	egid,
+		rgid;
+#endif
 
 	/* Error message stream */
 	errfp = stderr;
@@ -171,15 +214,34 @@ main(int argc, char **argv)
 #endif
 
 	/* Snoop command line for -display before XtVaAppInitialize
-	 * consumes it
+	 * consumes it  Also, handle -help here before we attempt
+	 * to open the X display.
 	 */
 	for (i = 0; i < argc; i++) {
-		if (strcmp(argv[i], "-display") == 0) {
+		if (strcmp(argv[i], "-display") == 0)
 			cdisplay = argv[++i];
-			break;
+		else if (strcmp(argv[i], "-help") == 0) {
+			usage(argv[i]);
+			exit(1);
 		}
 	}
 
+#ifdef HAS_SETEUID
+	/* Get real IDs */
+	ruid = getuid();
+	rgid = getgid();
+
+	/* Save effective IDs */
+	euid = geteuid();
+	egid = getegid();
+
+	/* Give up root until we have a connection to the X server, so
+	 * that X authentication will work correctly.
+	 */
+	(void) seteuid(ruid);
+	(void) setegid(rgid);
+#endif
+
 	/* Initialize X toolkit */
 	widgets.toplevel = XtVaAppInitialize(
 		&app_context,
@@ -199,10 +261,59 @@ main(int argc, char **argv)
 		NULL
 	);
 		
-	/* Check command line for unknown arguments */
+	/* Build command */
 	if (argc > 1) {
-		usage(argc, argv);
-		exit(1);
+		for (i = 1;  i < argc; i++) {
+			if (argv[i][0] == '-') {
+				usage(argv[i]);
+				exit(1);
+			}
+
+			if (i == 1) {
+				cmd = (char *) MEM_ALLOC(
+					"cmd",
+					strlen(argv[i]) + 1
+				);
+				if (cmd == NULL) {
+					CD_FATAL(app_data.str_nomemory);
+					exit(1);
+				}
+				cmd[0] = '\0';
+			}
+			else {
+				cmd = (char *) MEM_REALLOC(
+					"cmd",
+					cmd,
+					strlen(cmd) + strlen(argv[i]) + 2
+				);
+				if (cmd == NULL) {
+					CD_FATAL(app_data.str_nomemory);
+					exit(1);
+				}
+				(void) strcat(cmd, " ");
+			}
+			(void) strcat(cmd, argv[i]);
+		}
+	}
+
+	display = XtDisplay(widgets.toplevel);
+
+	/* Initialize libutil */
+	util_init();
+
+	if (app_data.remotemode) {
+		/* Remote control specified */
+		cmd_init(&status, display, TRUE);
+		exit(cmd_sendrmt(display, cmd));
+	}
+
+	/* Install colormap if specified */
+	if (app_data.instcmap) {
+		localcmap = XCopyColormapAndFree(
+			display,
+			DefaultColormap(display, DefaultScreen(display))
+		);
+		XtVaSetValues(widgets.toplevel, XmNcolormap, localcmap, NULL);
 	}
 
 	/* Create all widgets */
@@ -220,15 +331,37 @@ main(int argc, char **argv)
 	/* Register callback routines */
 	register_callbacks(&widgets, &status);
 
+#ifdef HAS_SETEUID
+	/* Ok, back to root */
+	(void) seteuid(euid);
+	(void) setegid(egid);
+#endif
+
 	/* Initialize various subsystems */
 	cd_init(&status);
 
 	/* Start various subsystems */
 	cd_start(&status);
 
+	/* Start remote control */
+	cmd_init(&status, display, FALSE);
+
+	if (cmd != NULL) {
+		/* Change to watch cursor */
+		cd_busycurs(TRUE, CURS_ALL);
+
+		/* Schedule to perform start-up command */
+		(void) cd_timeout( 
+			STARTUP_CMD_DELAY,
+			cmd_startup,
+			(byte_t *) cmd
+		);
+	}
+
 	/* Main event processing loop */
 	XtAppMainLoop(app_context);
 
+	shutdown_gui();
 	exit(0);
 
 	/*NOTREACHED*/
