@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __RESOURCE_H__
 
 #ifndef LINT
-static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
+static char *_resource_h_ident_ = "@(#)resource.h	6.83 00/01/11";
 #endif
 
 
@@ -36,6 +36,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCVersion			"Version"
 #define XmcdNmainWindowMode		"mainWindowMode"
 #define XmcdCMainWindowMode		"MainWindowMode"
+#define XmcdNmodeChangeGravity		"modeChangeGravity"
+#define XmcdCModeChangeGravity		"ModeChangeGravity"
 #define XmcdNnormalMainWidth		"normalMainWidth"
 #define XmcdCNormalMainWidth		"NormalMainWidth"
 #define XmcdNnormalMainHeight		"normalMainHeight"
@@ -50,14 +52,16 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCDisplayBlinkOffInterval	"DisplayBlinkOffInterval"
 #define XmcdNmainShowFocus		"mainShowFocus"
 #define XmcdCMainShowFocus		"MainShowFocus"
+#define XmcdNinstallColormap		"installColormap"
+#define XmcdCInstallColormap		"InstallColormap"
+#define XmcdNremoteMode			"remoteMode"
+#define XmcdCRemoteMode			"RemoteMode"
+#define XmcdNremoteHost			"remoteHost"
+#define XmcdCRemoteHost			"RemoteHost"
 
 /* Common config parameters */
 #define XmcdNdevice			"device"
 #define XmcdCDevice			"Device"
-#define XmcdNcddbPath			"cddbPath"
-#define XmcdCCddbPath			"CddbPath"
-#define XmcdNcddbFileMode		"cddbFileMode"
-#define XmcdCCddbFileMode		"CddbFileMode"
 #define XmcdNcddbMailSite		"cddbMailSite"
 #define XmcdCCddbMailSite		"CddbMailSite"
 #define XmcdNcddbMailCmd		"cddbMailCmd"
@@ -84,6 +88,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCTooltipActiveInterval	"TooltipActiveInterval"
 #define XmcdNinsertPollDisable		"insertPollDisable"
 #define XmcdCInsertPollDisable		"InsertPollDisable"
+#define XmcdNcddbRemoteDisable		"cddbRemoteDisable"
+#define XmcdCCddbRemoteDisable		"CddbRemoteDisable"
 #define XmcdNcddbRemoteAutoSave		"cddbRemoteAutoSave"
 #define XmcdCCddbRemoteAutoSave		"CddbRemoteAutoSave"
 #define XmcdNcddbUseHttpProxy		"cddbUseHttpProxy"
@@ -96,6 +102,10 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCCurfileEnable		"CurfileEnable"
 #define XmcdNtooltipEnable		"tooltipEnable"
 #define XmcdCTooltipEnable		"TooltipEnable"
+#define XmcdNremoteControlEnable	"remoteControlEnable"
+#define XmcdCRemoteControlEnable	"RemoteControlEnable"
+#define XmcdNremoteControlLog		"remoteControlLog"
+#define XmcdCRemoteControlLog		"RemoteControlLog"
 #define XmcdNdebugMode			"debugMode"
 #define XmcdCDebugMode			"DebugMode"
 
@@ -144,6 +154,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCCurposFormat		"CurposFormat"
 #define XmcdNnoTURWhenPlaying		"noTURWhenPlaying"
 #define XmcdCNoTURWhenPlaying		"NoTURWhenPlaying"
+#define XmcdNspinUpInterval		"spinUpInterval"
+#define XmcdCSpinUpInterval		"SpinUpInterval"
 #define XmcdNvolumeControlTaper		"volumeControlTaper"
 #define XmcdCVolumeControlTaper		"VolumeControlTaper"
 #define XmcdNstartupVolume		"startupVolume"
@@ -228,14 +240,16 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCUnknownTrackMsg		"UnknownTrackMsg"
 #define XmcdNdataMsg			"dataMsg"
 #define XmcdCDataMsg			"DataMsg"
+#define XmcdNinfoMsg			"infoMsg"
+#define XmcdCInfoMsg			"InfoMsg"
 #define XmcdNwarningMsg			"warningMsg"
 #define XmcdCWarningMsg			"WarningMsg"
 #define XmcdNfatalMsg			"fatalMsg"
 #define XmcdCFatalMsg			"FatalMsg"
 #define XmcdNconfirmMsg			"confirmMsg"
 #define XmcdCConfirmMsg			"ConfirmMsg"
-#define XmcdNinfoMsg			"infoMsg"
-#define XmcdCInfoMsg			"InfoMsg"
+#define XmcdNworkingMsg			"workingMsg"
+#define XmcdCWorkingMsg			"WorkingMsg"
 #define XmcdNaboutMsg			"aboutMsg"
 #define XmcdCAboutMsg			"AboutMsg"
 #define XmcdNquitMsg			"quitMsg"
@@ -260,6 +274,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCNoDbMsg			"NoDbMsg"
 #define XmcdNnoCfgMsg			"noCfgMsg"
 #define XmcdCNoCfgMsg			"NoCfgMsg"
+#define XmcdNnoInfoMsg			"noInfoMsg"
+#define XmcdCNoInfoMsg			"NoInfoMsg"
 #define XmcdNnotRomMsg			"notRomMsg"
 #define XmcdCNotRomMsg			"NotRomMsg"
 #define XmcdNnotScsi2Msg		"notScsi2Msg"
@@ -320,12 +336,40 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCProxyConfigErrMsg		"ProxyConfigErrMsg"
 #define XmcdNproxyAuthFailMsg		"proxyAuthFailMsg"
 #define XmcdCProxyAuthFailMsg		"ProxyAuthFailMsg"
+#define XmcdNnoClientMsg		"noClientMsg"
+#define XmcdCNoClientMsg		"NoClientMsg"
+#define XmcdNunsuppCmdMsg		"unsuppCmdMsg"
+#define XmcdCUnsuppCmdMsg		"UnsuppCmdMsg"
+#define XmcdNbadArgMsg			"badArgMsg"
+#define XmcdCBadArgMsg			"BadArgMsg"
+#define XmcdNinvalidCmdMsg		"invalidCmdMsg"
+#define XmcdCInvalidCmdMsg		"InvalidCmdMsg"
+#define XmcdNcommandFailMsg		"commandFailMsg"
+#define XmcdCCommandFailMsg		"CommandFailMsg"
+#define XmcdNremoteNotEnabledMsg	"remoteNotEnabledMsg"
+#define XmcdCRemoteNotEnabledMsg	"RemoteNotEnabledMsg"
+#define XmcdNremoteNoCmdMsg		"remoteNoCmdMsg"
+#define XmcdCRemoteNoCmdMsg		"RemoteNoCmdMsg"
 #define XmcdNbrowserStartMsg		"browserStartMsg"
 #define XmcdCBrowserStartMsg		"BrowserStartMsg"
 #define XmcdNbrowserFailMsg		"browserFailMsg"
 #define XmcdCBrowserFailMsg		"BrowserFailMsg"
 #define XmcdNappDefFileMsg		"appDefFileMsg"
 #define XmcdCAppDefFileMsg		"AppDefFileMsg"
+#define XmcdNkpModeDisableMsg		"kpModeDisableMsg"
+#define XmcdCKpModeDisableMsg		"KpModeDisableMsg"
+#define XmcdNdeleteAllHistoryMsg	"deleteAllHistoryMsg"
+#define XmcdCDeleteAllHistoryMsg	"DeleteAllHistoryMsg"
+#define XmcdNchangerScanningMsg		"changerScanningMsg"
+#define XmcdCChangerScanningMsg		"ChangerScanningMsg"
+#define XmcdNlocalDiscography		"localDiscography"
+#define XmcdCLocalDiscography		"LocalDiscography"
+#define XmcdNfileMsg			"fileMsg"
+#define XmcdCFileMsg			"FileMsg"
+#define XmcdNbrowseCategoryMsg		"browseCategoryMsg"
+#define XmcdCBrowseCategoryMsg		"BrowseCategoryMsg"
+#define XmcdNhowToSetupMsg		"howToSetupMsg"
+#define XmcdCHowToSetupMsg		"HowToSetupMsg"
 
 /* Short-cut key definitions */
 #define XmcdNmodeKey			"modeKey"
@@ -342,8 +386,8 @@ static char *_resource_h_ident_ = "@(#)resource.h	6.63 98/05/03";
 #define XmcdCQuitKey			"QuitKey"
 #define XmcdNdbprogKey			"dbprogKey"
 #define XmcdCDbprogKey			"DbprogKey"
-#define XmcdNhelpKey			"helpKey"
-#define XmcdCHelpKey			"HelpKey"
+#define XmcdNwwwWarpKey			"wwwWarpKey"
+#define XmcdCWwwWarpKey			"WwwWarpKey"
 #define XmcdNoptionsKey			"optionsKey"
 #define XmcdCOptionsKey			"OptionsKey"
 #define XmcdNtimeKey			"timeKey"
@@ -425,6 +469,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, main_mode), XmRImmediate,
 		(XtPointer) 0,
 	},
+	{
+		XmcdNmodeChangeGravity, XmcdCModeChangeGravity,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, modechg_grav), XmRImmediate,
+		(XtPointer) 0,
+	},
 	{
 		XmcdNnormalMainWidth, XmcdCNormalMainWidth,
 		XmRInt, sizeof(int),
@@ -467,6 +517,24 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, blinkoff_interval), XmRImmediate,
 		(XtPointer) 150,
 	},
+	{
+		XmcdNinstallColormap, XmcdCInstallColormap,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, instcmap), XmRImmediate,
+		(XtPointer) False,
+	},
+	{
+		XmcdNremoteMode, XmcdCRemoteMode,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, remotemode), XmRImmediate,
+		(XtPointer) False,
+	},
+	{
+		XmcdNremoteHost, XmcdCRemoteHost,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, remotehost), XmRImmediate,
+		(XtPointer) NULL,
+	},
 
 	/* Common config parameters */
 	{
@@ -475,18 +543,6 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, device), XmRImmediate,
 		(XtPointer) NULL,
 	},
-	{
-		XmcdNcddbPath, XmcdCCddbPath,
-		XmRString, sizeof(String),
-		XtOffsetOf(appdata_t, cddb_path), XmRImmediate,
-		(XtPointer) NULL,
-	},
-	{
-		XmcdNcddbFileMode, XmcdCCddbFileMode,
-		XmRString, sizeof(String),
-		XtOffsetOf(appdata_t, cddb_filemode), XmRImmediate,
-		(XtPointer) NULL,
-	},
 	{
 		XmcdNcddbMailSite, XmcdCCddbMailSite,
 		XmRString, sizeof(String),
@@ -565,6 +621,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, ins_disable), XmRImmediate,
 		(XtPointer) False,
 	},
+	{
+		XmcdNcddbRemoteDisable, XmcdCCddbRemoteDisable,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, cddb_rmtdsbl), XmRImmediate,
+		(XtPointer) False,
+	},
 	{
 		XmcdNcddbRemoteAutoSave, XmcdCCddbRemoteAutoSave,
 		XmRBoolean, sizeof(Boolean),
@@ -601,6 +663,18 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, tooltip_enable), XmRImmediate,
 		(XtPointer) True,
 	},
+	{
+		XmcdNremoteControlEnable, XmcdCRemoteControlEnable,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, remote_enb), XmRImmediate,
+		(XtPointer) True,
+	},
+	{
+		XmcdNremoteControlLog, XmcdCRemoteControlLog,
+		XmRBoolean, sizeof(Boolean),
+		XtOffsetOf(appdata_t, remote_log), XmRImmediate,
+		(XtPointer) True,
+	},
 	{
 		XmcdNdebugMode, XmcdCDebugMode,
 		XmRBoolean, sizeof(Boolean),
@@ -741,6 +815,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, play_notur), XmRImmediate,
 		(XtPointer) False,
 	},
+	{
+		XmcdNspinUpInterval, XmcdCSpinUpInterval,
+		XmRInt, sizeof(int),
+		XtOffsetOf(appdata_t, spinup_interval), XmRImmediate,
+		(XtPointer) 2,
+	},
 	{
 		XmcdNvolumeControlTaper, XmcdCVolumeControlTaper,
 		XmRInt, sizeof(int),
@@ -989,6 +1069,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, str_data), XmRImmediate,
 		(XtPointer) STR_DATA,
 	},
+	{
+		XmcdNinfoMsg, XmcdCInfoMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_info), XmRImmediate,
+		(XtPointer) STR_INFO,
+	},
 	{
 		XmcdNwarningMsg, XmcdCWarningMsg,
 		XmRString, sizeof(String),
@@ -1008,10 +1094,10 @@ STATIC XtResource	resources[] = {
 		(XtPointer) STR_CONFIRM,
 	},
 	{
-		XmcdNinfoMsg, XmcdCInfoMsg,
+		XmcdNworkingMsg, XmcdCWorkingMsg,
 		XmRString, sizeof(String),
-		XtOffsetOf(appdata_t, str_info), XmRImmediate,
-		(XtPointer) STR_INFO,
+		XtOffsetOf(appdata_t, str_working), XmRImmediate,
+		(XtPointer) STR_WORKING,
 	},
 	{
 		XmcdNaboutMsg, XmcdCAboutMsg,
@@ -1085,6 +1171,12 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, str_nocfg), XmRImmediate,
 		(XtPointer) STR_NOCFG,
 	},
+	{
+		XmcdNnoInfoMsg, XmcdCNoInfoMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_noinfo), XmRImmediate,
+		(XtPointer) STR_NOINFO,
+	},
 	{
 		XmcdNnotRomMsg, XmcdCNotRomMsg,
 		XmRString, sizeof(String),
@@ -1265,6 +1357,48 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, str_authfail), XmRImmediate,
 		(XtPointer) STR_AUTHFAIL,
 	},
+	{
+		XmcdNnoClientMsg, XmcdCNoClientMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_noclient), XmRImmediate,
+		(XtPointer) STR_NOCLIENT,
+	},
+	{
+		XmcdNunsuppCmdMsg, XmcdCUnsuppCmdMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_unsuppcmd), XmRImmediate,
+		(XtPointer) STR_UNSUPPCMD,
+	},
+	{
+		XmcdNbadArgMsg, XmcdCBadArgMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_badarg), XmRImmediate,
+		(XtPointer) STR_BADARG,
+	},
+	{
+		XmcdNinvalidCmdMsg, XmcdCInvalidCmdMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_invcmd), XmRImmediate,
+		(XtPointer) STR_INVCMD,
+	},
+	{
+		XmcdNcommandFailMsg, XmcdCCommandFailMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_cmdfail), XmRImmediate,
+		(XtPointer) STR_CMDFAIL,
+	},
+	{
+		XmcdNremoteNotEnabledMsg, XmcdCRemoteNotEnabledMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_rmt_notenb), XmRImmediate,
+		(XtPointer) STR_RMT_NOTENB,
+	},
+	{
+		XmcdNremoteNoCmdMsg, XmcdCRemoteNoCmdMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_rmt_nocmd), XmRImmediate,
+		(XtPointer) STR_RMT_NOCMD,
+	},
 	{
 		XmcdNbrowserStartMsg, XmcdCBrowserStartMsg,
 		XmRString, sizeof(String),
@@ -1283,6 +1417,48 @@ STATIC XtResource	resources[] = {
 		XtOffsetOf(appdata_t, str_appdef), XmRImmediate,
 		(XtPointer) STR_APPDEF,
 	},
+	{
+		XmcdNkpModeDisableMsg, XmcdCKpModeDisableMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_kpmodedsbl), XmRImmediate,
+		(XtPointer) STR_KPMODEDSBL,
+	},
+	{
+		XmcdNdeleteAllHistoryMsg, XmcdCDeleteAllHistoryMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_dlist_delall), XmRImmediate,
+		(XtPointer) STR_DLIST_DELALL,
+	},
+	{
+		XmcdNchangerScanningMsg, XmcdCChangerScanningMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_chgrscan), XmRImmediate,
+		(XtPointer) STR_CHGRSCAN,
+	},
+	{
+		XmcdNlocalDiscography, XmcdCLocalDiscography,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_localdiscog), XmRImmediate,
+		(XtPointer) STR_LOCALDISCOG,
+	},
+	{
+		XmcdNfileMsg, XmcdCFileMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_file), XmRImmediate,
+		(XtPointer) STR_FILE,
+	},
+	{
+		XmcdNbrowseCategoryMsg, XmcdCBrowseCategoryMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_browse_categ), XmRImmediate,
+		(XtPointer) STR_BROWSE_CATEG,
+	},
+	{
+		XmcdNhowToSetupMsg, XmcdCHowToSetupMsg,
+		XmRString, sizeof(String),
+		XtOffsetOf(appdata_t, str_howto_setup), XmRImmediate,
+		(XtPointer) STR_HOWTO_SETUP,
+	},
 
 	/* Short-cut key definitions */
 	{
@@ -1328,9 +1504,9 @@ STATIC XtResource	resources[] = {
 		(XtPointer) "",
 	},
 	{
-		XmcdNhelpKey, XmcdCHelpKey,
+		XmcdNwwwWarpKey, XmcdCWwwWarpKey,
 		XmRString, sizeof(String),
-		XtOffsetOf(appdata_t, help_key), XmRImmediate,
+		XtOffsetOf(appdata_t, wwwwarp_key), XmRImmediate,
 		(XtPointer) "",
 	},
 	{
@@ -1505,13 +1681,16 @@ STATIC XtResource	resources[] = {
 
 
 STATIC XrmOptionDescRec	options[] = {
-	{ "-dev",	"*device",	XrmoptionSepArg,	NULL },
-	{ "-debug",	"*debugMode",	XrmoptionNoArg,		"True" },
+	{ "-dev",	"*device",	    XrmoptionSepArg,	NULL	},
+	{ "-instcmap",	"*installColormap", XrmoptionNoArg,	"True"	},
+	{ "-remote",	"*remoteMode",      XrmoptionNoArg,	"True"	},
+	{ "-rmthost",	"*remoteHost",      XrmoptionSepArg,	NULL	},
+	{ "-debug",	"*debugMode",	    XrmoptionNoArg,	"True"	},
 #if defined(SVR4) && (defined(sun) || defined(__sun__))
 	/* Solaris 2 volume manager auto-startup support */
-	{ "-c",		"*device",	XrmoptionSepArg,	NULL },
-	{ "-X",		"*exitOnEject",	XrmoptionNoArg,		"True" },
-	{ "-o",		"",		XrmoptionNoArg,		"False" },
+	{ "-c",		"*device",	    XrmoptionSepArg,	NULL	},
+	{ "-X",		"*exitOnEject",	    XrmoptionNoArg,	"True"	},
+	{ "-o",		"",		    XrmoptionNoArg,	"False"	},
 #endif
 };
 
