@@ -1,7 +1,7 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-1998  Ti Kan
+ *   Copyright (C) 1993-2000  Ti Kan
  *   E-mail: ti@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
 #define __WIDGET_H__
 
 #ifndef LINT
-static char *_widget_h_ident_ = "@(#)widget.h	6.24 98/05/21";
+static char *_widget_h_ident_ = "@(#)widget.h	6.39 00/01/15";
 #endif
 
 /* Holder for all widgets */
@@ -59,7 +59,7 @@ typedef struct {
 		Widget	ab_btn;			/* A->B mode button */
 		Widget	sample_btn;		/* Sample mode button */
 		Widget	keypad_btn;		/* Keypad button */
-		Widget	help_btn;		/* Help button */
+		Widget	wwwwarp_btn;		/* wwwWarp button */
 		Widget	level_scale;		/* Volume control slider */
 		Widget	playpause_btn;		/* Play/Pause button */
 		Widget	stop_btn;		/* Stop button */
@@ -113,6 +113,7 @@ typedef struct {
 		Widget	done_chkbox_frm;	/* Done options frame */
 		Widget	done_chkbox;		/* Done options check box */
 		Widget	done_eject_btn;		/* Auto Eject button */
+		Widget	done_exit_btn;		/* Auto Exit button */
 		Widget	eject_lbl;		/* Eject options label */
 		Widget	eject_chkbox_frm;	/* Eject options frame */
 		Widget	eject_chkbox;		/* Eject options check box */
@@ -150,25 +151,23 @@ typedef struct {
 	/* CD Database/Program window widgets */
 	struct {
 		Widget	form;			/* Form container */
-		Widget	tottime_lbl;		/* Total time label */
 		Widget	tottime_ind;		/* Total time indicator */
+		Widget	discid_ind;		/* Disc ID indicator */
 		Widget	logo_lbl;		/* Logo label */
-		Widget	wwwwarp_btn;		/* wwwWarp button */
+		Widget	dlist_btn;		/* Disc list button */
 		Widget	dtitle_lbl;		/* Disc title label */
 		Widget	dtitle_txt;		/* Disc title text */
 		Widget	extd_lbl;		/* Disc ext descr label */
 		Widget	extd_btn;		/* Disc ext descr button */
 		Widget	dbprog_sep1;		/* Separator bar */
 		Widget	trklist_lbl;		/* Track list label */
+		Widget	trklist_sw;		/* Track list scrolled window */
 		Widget	trk_list;		/* Track list */
 		Widget	radio_lbl;		/* Time dpy radio box label */
 		Widget	radio_frm;		/* Time dpy radio box Frame */
 		Widget	radio_box;		/* Time dpy radio box */
 		Widget	tottime_btn;		/* Total time button */
 		Widget	trktime_btn;		/* Track time button */
-		Widget	discid_lbl;		/* Disc ID label */
-		Widget	discid_frm;		/* Disc ID frame */
-		Widget	discid_ind;		/* Disc ID indicator */
 		Widget	ttitle_lbl;		/* Track title label */
 		Widget	ttitle_txt;		/* Track title text */
 		Widget	extt_lbl;		/* Track ext descr label */
@@ -178,7 +177,9 @@ typedef struct {
 		Widget	clrpgm_btn;		/* Clear button */
 		Widget	pgmseq_lbl;		/* Program sequence label */
 		Widget	pgmseq_txt;		/* Program sequence text */
-		Widget	send_btn;		/* Send CDDB button */
+		Widget	cddb_lbl;		/* CDDB label */
+		Widget	rmtdsbl_btn;		/* CDDB no remote button */
+		Widget	submit_btn;		/* CDDB submit button */
 		Widget	dbprog_sep2;		/* Separator bar */
 		Widget	savedb_btn;		/* Save to database button */
 		Widget	linkdb_btn;		/* Link database entry button */
@@ -186,24 +187,26 @@ typedef struct {
 		Widget	cancel_btn;		/* Cancel button */
 	} dbprog;
 
-	/* wwwWarp window widgets */
+	/* Disc list window widgets */
 	struct {
 		Widget	form;			/* Form container */
-		Widget	wwwwarp_lbl;		/* pictorial logo label */
-		Widget	sel_lbl;		/* Select one label */
-		Widget	sel_radbox;		/* Radio box */
-		Widget	site1_btn;		/* Web site 1 button */
-		Widget	site2_btn;		/* Web site 2 button */
-		Widget	site3_btn;		/* Web site 3 button */
-		Widget	srchweb_btn;		/* Search web button */
-		Widget	srchsite_menu;		/* Search sites pulldown menu */
-		Widget	srchsite_opt;		/* Search sites option menu */
-		Widget	srchsite_lbl;		/* Search sites menu label */
-		Widget	srchsite_sep;		/* Search sites lbl separator */
-		Widget	wwwwarp_sep;		/* Separator bar */
-		Widget	go_btn;			/* Go button */
+		Widget	type_opt;		/* List type option menu */
+		Widget	type_menu;		/* List type pulldown menu */
+		Widget	menu_lbl;		/* Menu title label */
+		Widget	menu_sep;		/* Menu separator bar */
+		Widget	hist_btn;		/* History menu button */
+		Widget	chgr_btn;		/* Changer menu button */
+		Widget	disclist_lbl;		/* Disc list label */
+		Widget	disclist_sw;		/* Disc list scrolled window */
+		Widget	disc_list;		/* Disc list */
+		Widget	show_btn;		/* Show button */
+		Widget	goto_btn;		/* Change to button */
+		Widget	del_btn;		/* Delete button */
+		Widget	delall_btn;		/* Delete all button */
+		Widget	rescan_btn;		/* Re-scan button */
+		Widget	dlist_sep;		/* Separator bar */
 		Widget	cancel_btn;		/* Cancel button */
-	} wwwwarp;
+	} dlist;
 
 	/* Extended disc info window widgets */
 	struct {
@@ -223,6 +226,7 @@ typedef struct {
 		Widget	prev_btn;		/* Prev track arrow button */
 		Widget	next_btn;		/* Next track arrow button */
 		Widget	trkno_lbl;		/* Track number label */
+		Widget	autotrk_btn;		/* Auto-track button */
 		Widget	trk_lbl;		/* Track ext descr label */
 		Widget	trk_txt;		/* Track ext descr text */
 		Widget	dbextt_sep;		/* Separator bar */
@@ -251,6 +255,27 @@ typedef struct {
 		Widget	cancel_btn;		/* Cancel button */
 	} linksel;
 
+	/* wwwWarp window widgets */
+	struct {
+		Widget	form;			/* Form container */
+		Widget	wwwwarp_lbl;		/* pictorial logo label */
+		Widget	help_btn;		/* Help button */
+		Widget	sel_lbl;		/* Select one label */
+		Widget	sel_radbox;		/* Radio box */
+		Widget	site1_btn;		/* Web site 1 button */
+		Widget	site2_btn;		/* Web site 2 button */
+		Widget	site3_btn;		/* Web site 3 button */
+		Widget	discog_btn;		/* Local Discograph button */
+		Widget	srchweb_btn;		/* Search web button */
+		Widget	srchsite_menu;		/* Search sites pulldown menu */
+		Widget	srchsite_opt;		/* Search sites option menu */
+		Widget	srchsite_lbl;		/* Search sites menu label */
+		Widget	srchsite_sep;		/* Search sites lbl separator */
+		Widget	wwwwarp_sep;		/* Separator bar */
+		Widget	go_btn;			/* Go button */
+		Widget	cancel_btn;		/* Cancel button */
+	} wwwwarp;
+
 	/* Help window widgets */
 	struct {
 		Widget	form;			/* Form container */
@@ -285,6 +310,7 @@ typedef struct {
 		Widget	warning;		/* Warning popup */
 		Widget	fatal;			/* Fatal error popup */
 		Widget	confirm;		/* Confirm popup */
+		Widget	working;		/* Working popup */
 		Widget	dbmatch;		/* CDDB match query popup */
 		Widget	about;			/* About popup */
 	} dialog;
@@ -323,8 +349,8 @@ typedef struct {
 		Pixmap	sample_hlpixmap;	/* Sample */
 		Pixmap	keypad_pixmap;		/* Keypad */
 		Pixmap	keypad_hlpixmap;	/* Keypad */
-		Pixmap	help_pixmap;		/* Help */
-		Pixmap	help_hlpixmap;		/* Help */
+		Pixmap	world_pixmap;		/* wwwWarp */
+		Pixmap	world_hlpixmap;		/* wwwWarp */
 		Pixmap	playpause_pixmap;	/* Play/Pause */
 		Pixmap	playpause_hlpixmap;	/* Play/Pause */
 		Pixmap	stop_pixmap;		/* Stop */
@@ -347,16 +373,16 @@ typedef struct {
 		Pixmap	ff_hlpixmap;		/* Search FF */
 	} main;
 
-	/* wwwWarp window pixmaps */
-	struct {
-		Pixmap	wwwwarp_pixmap;		/* wwwWarp logo */
-	} wwwwarp;
-
 	/* CD Database/Program window pixmaps */
 	struct {
 		Pixmap	logo_pixmap;		/* CD Logo */
 	} dbprog;
 
+	/* wwwWarp window pixmaps */
+	struct {
+		Pixmap	wwwwarp_pixmap;		/* wwwWarp logo */
+	} wwwwarp;
+
 	/* Dialog windows pixmaps */
 	struct {
 		Pixmap	xmcd_pixmap;		/* Program logo */
@@ -373,78 +399,11 @@ typedef struct {
 #define BM_PX_BWREV	5		/* white/black */
 
 
-/* Macros */
-#define register_activate_cb(w, func, arg)				\
-	XtAddCallback((w), XmNactivateCallback,				\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_arm_cb(w, func, arg)					\
-	XtAddCallback((w), XmNarmCallback,				\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_disarm_cb(w, func, arg)				\
-	XtAddCallback((w), XmNdisarmCallback,				\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_valchg_cb(w, func, arg)				\
-	XtAddCallback((w), XmNvalueChangedCallback,			\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_drag_cb(w, func, arg)					\
-	XtAddCallback((w), XmNdragCallback,				\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_modvfy_cb(w, func, arg)				\
-	XtAddCallback((w), XmNmodifyVerifyCallback,			\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_focus_cb(w, func, arg)					\
-	XtAddCallback((w), XmNfocusCallback,				\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_losefocus_cb(w, func, arg)				\
-	XtAddCallback((w), XmNlosingFocusCallback,			\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_defaction_cb(w, func, arg)				\
-	XtAddCallback((w), XmNdefaultActionCallback,			\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_browsel_cb(w, func, arg)				\
-	XtAddCallback((w), XmNbrowseSelectionCallback,			\
-		(XtCallbackProc) (func),				\
-		(XtPointer) (arg))
-
-#define register_focuschg_ev(w)						\
-	XtAddEventHandler((w), FocusChangeMask,				\
-			  False,					\
-			  (XtEventHandler) cd_focus_chg,		\
-			  (XtPointer) NULL)
-
-#define register_xingchg_ev(w)						\
-	XtAddEventHandler((w), EnterWindowMask | LeaveWindowMask,	\
-			  False,					\
-			  (XtEventHandler) cd_xing_chg,			\
-			  (XtPointer) NULL)
-
-
 /* Public function prototypes */
-extern void	add_delw_callback(Widget, XtCallbackProc, XtPointer);
-extern void	rm_delw_callback(Widget, XtCallbackProc, XtPointer);
 extern Pixmap	bm_to_px(Widget, void *, int, int, int, int);
 extern void	create_widgets(widgets_t *);
 extern void	pre_realize_config(widgets_t *);
 extern void	post_realize_config(widgets_t *, pixmaps_t *);
-extern void	register_callbacks(widgets_t *, curstat_t *);
 
 #endif /* __WIDGET_H__ */
 
